@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,7 +28,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheatManager::CheatManager(OSystem& osystem)
-  : myOSystem(osystem)
+  : myOSystem{osystem}
 {
 }
 
@@ -253,8 +253,8 @@ void CheatManager::saveCheatDatabase()
     return;
 
   stringstream out;
-  for(const auto& iter: myCheatMap)
-    out << "\"" << iter.first << "\" " << "\"" << iter.second << "\"" << endl;
+  for(const auto& [md5, cheat]: myCheatMap)
+    out << "\"" << md5 << "\" " << "\"" << cheat << "\"" << endl;
 
   try         { myOSystem.cheatFile().write(out); }
   catch(...)  { return; }
