@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -23,10 +23,19 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandlerSDL2::EventHandlerSDL2(OSystem& osystem)
-  : EventHandler(osystem)
+  : EventHandler{osystem}
 {
   ASSERT_MAIN_THREAD;
 
+#ifdef GUI_SUPPORT
+  {
+    ostringstream buf;
+    myQwertz = int('y') == int(SDL_GetKeyFromScancode(SDL_Scancode(KBDK_Z)));
+    buf << "Keyboard: " << (myQwertz ? "QWERTZ" : "QWERTY");
+    Logger::debug(buf.str());
+  }
+#endif
+
 #ifdef JOYSTICK_SUPPORT
   if(SDL_InitSubSystem(SDL_INIT_JOYSTICK) < 0)
   {
@@ -36,6 +45,8 @@ EventHandlerSDL2::EventHandlerSDL2(OSystem& osystem)
   }
   Logger::debug("EventHandlerSDL2::EventHandlerSDL2 SDL_INIT_JOYSTICK");
 #endif
+
+  SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -64,13 +75,6 @@ void EventHandlerSDL2::copyText(const string& text) const
   SDL_SetClipboardText(text.c_str());
 };
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EventHandlerSDL2::cutText(string& text) const
-{
-  copyText(text);
-  text = "";
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string EventHandlerSDL2::pasteText(string& text) const
 {
