@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -31,7 +31,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FBBackendSDL2::FBBackendSDL2(OSystem& osystem)
-  : myOSystem(osystem)
+  : myOSystem{osystem}
 {
   ASSERT_MAIN_THREAD;
 
@@ -39,9 +39,8 @@ FBBackendSDL2::FBBackendSDL2(OSystem& osystem)
   if(SDL_InitSubSystem(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0)
   {
     ostringstream buf;
-    buf << "ERROR: Couldn't initialize SDL: " << SDL_GetError() << endl;
-    Logger::error(buf.str());
-    throw runtime_error("FATAL ERROR");
+    buf << "ERROR: Couldn't initialize SDL: " << SDL_GetError();
+    throw runtime_error(buf.str());
   }
   Logger::debug("FBBackendSDL2::FBBackendSDL2 SDL_Init()");
 
