@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -24,9 +24,9 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 MouseControl::MouseControl(Console& console, const string& mode)
-  : myProps(console.properties()),
-    myLeftController(console.leftController()),
-    myRightController(console.rightController())
+  : myProps{console.properties()},
+    myLeftController{console.leftController()},
+    myRightController{console.rightController()}
 {
   istringstream m_axis(mode);
   string m_mode;
@@ -137,13 +137,17 @@ MouseControl::MouseControl(Console& console, const string& mode)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& MouseControl::next()
+const string& MouseControl::change(int direction)
 {
+  myCurrentModeNum = BSPF::clampw(myCurrentModeNum + direction, 0, int(myModeList.size() - 1));
   const MouseMode& mode = myModeList[myCurrentModeNum];
-  myCurrentModeNum = (myCurrentModeNum + 1) % myModeList.size();
 
-  myLeftController.setMouseControl(mode.xtype, mode.xid, mode.ytype, mode.yid);
-  myRightController.setMouseControl(mode.xtype, mode.xid, mode.ytype, mode.yid);
+  bool leftControl =
+    myLeftController.setMouseControl(mode.xtype, mode.xid, mode.ytype, mode.yid);
+  bool rightControl =
+    myRightController.setMouseControl(mode.xtype, mode.xid, mode.ytype, mode.yid);
+
+  myHasMouseControl = leftControl || rightControl;
 
   return mode.message;
 }
