@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -19,6 +19,9 @@
 #include "Console.hxx"
 #include "EventHandler.hxx"
 #include "PKeyboardHandler.hxx"
+#include "json_lib.hxx"
+
+using json = nlohmann::json;
 
 #ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
@@ -29,14 +32,16 @@
 
 #if defined(BSPF_MACOS) || defined(MACOS_KEYS)
 static constexpr int MOD3 = KBDM_GUI;
+static constexpr int CMD = KBDM_GUI;
+static constexpr int OPTION = KBDM_ALT;
 #else
 static constexpr int MOD3 = KBDM_ALT;
 #endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PhysicalKeyboardHandler::PhysicalKeyboardHandler(OSystem& system, EventHandler& handler)
-  : myOSystem(system),
-    myHandler(handler)
+  : myOSystem{system},
+    myHandler{handler}
 {
   Int32 version = myOSystem.settings().getInt("event_ver");
   bool updateDefaults = false;
@@ -44,22 +49,42 @@ PhysicalKeyboardHandler::PhysicalKeyboardHandler(OSystem& system, EventHandler&
   // Compare if event list version has changed so that key maps became invalid
   if (version == Event::VERSION)
   {
-    string list = myOSystem.settings().getString("keymap_emu");
-    myKeyMap.loadMapping(list, EventMode::kCommonMode);
-    list = myOSystem.settings().getString("keymap_joy");
-    myKeyMap.loadMapping(list, EventMode::kJoystickMode);
-    list = myOSystem.settings().getString("keymap_pad");
-    myKeyMap.loadMapping(list, EventMode::kPaddlesMode);
-    list = myOSystem.settings().getString("keymap_key");
-    myKeyMap.loadMapping(list, EventMode::kKeypadMode);
-    list = myOSystem.settings().getString("keymap_ui");
-    myKeyMap.loadMapping(list, EventMode::kMenuMode);
+    loadSerializedMappings(myOSystem.settings().getString("keymap_emu"), EventMode::kCommonMode);
+    loadSerializedMappings(myOSystem.settings().getString("keymap_joy"), EventMode::kJoystickMode);
+    loadSerializedMappings(myOSystem.settings().getString("keymap_pad"), EventMode::kPaddlesMode);
+    loadSerializedMappings(myOSystem.settings().getString("keymap_key"), EventMode::kKeypadMode);
+    loadSerializedMappings(myOSystem.settings().getString("keymap_ui"), EventMode::kMenuMode);
+
     updateDefaults = true;
   }
+
   myKeyMap.enableMod() = myOSystem.settings().getBool("modcombo");
 
   setDefaultMapping(Event::NoType, EventMode::kEmulationMode, updateDefaults);
   setDefaultMapping(Event::NoType, EventMode::kMenuMode, updateDefaults);
+#ifdef GUI_SUPPORT
+  setDefaultMapping(Event::NoType, EventMode::kEditMode, updateDefaults);
+#endif // DEBUG
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PhysicalKeyboardHandler::loadSerializedMappings(const string& serializedMapping, EventMode mode)
+{
+  json mapping;
+
+  try {
+    mapping = json::parse(serializedMapping);
+  } catch (const json::exception&) {
+    Logger::info("converting legacy keyboard mappings");
+
+    mapping = KeyMap::convertLegacyMapping(serializedMapping);
+  }
+
+  try {
+    myKeyMap.loadMapping(mapping, mode);
+  } catch (const json::exception&) {
+    Logger::error("ignoring bad keyboard mappings");
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -99,6 +124,17 @@ void PhysicalKeyboardHandler::setDefaultKey(EventMapping map, Event::Type event,
   // Otherwise, only reset the given event
   bool eraseAll = !updateDefaults && (event == Event::NoType);
 
+#ifdef GUI_SUPPORT
+  // Swap Y and Z for QWERTZ keyboards
+  if(mode == EventMode::kEditMode && myHandler.isQwertz())
+  {
+    if(map.key == KBDK_Z)
+      map.key = KBDK_Y;
+    else if(map.key == KBDK_Y)
+      map.key = KBDK_Z;
+  }
+#endif
+
   if (updateDefaults)
   {
     // if there is no existing mapping for the event and
@@ -151,13 +187,22 @@ void PhysicalKeyboardHandler::setDefaultMapping(Event::Type event, EventMode mod
         setDefaultKey(item, event, EventMode::kMenuMode, updateDefaults);
       break;
 
+  #ifdef GUI_SUPPORT
+    case EventMode::kEditMode:
+      // Edit mode events are always set because they are not saved
+      for(const auto& item: FixedEditMapping)
+        setDefaultKey(item, event, EventMode::kEditMode);
+      break;
+  #endif
+
     default:
       break;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PhysicalKeyboardHandler::defineControllerMappings(const Controller::Type type, Controller::Jack port)
+void PhysicalKeyboardHandler::defineControllerMappings(
+    const Controller::Type type, Controller::Jack port)
 {
   // determine controller events to use
   switch(type)
@@ -199,7 +244,8 @@ void PhysicalKeyboardHandler::enableEmulationMappings()
   myKeyMap.eraseMode(EventMode::kEmulationMode);
   enableCommonMappings();
 
-  // enable right mode first, so that in case of mapping clashes the left controller has preference
+  // enable right mode first, so that in case of mapping clashes the left
+  // controller has preference
   switch (myRightMode)
   {
     case EventMode::kPaddlesMode:
@@ -253,14 +299,16 @@ void PhysicalKeyboardHandler::enableCommonMappings()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PhysicalKeyboardHandler::enableMappings(const Event::EventSet& events, EventMode mode)
+void PhysicalKeyboardHandler::enableMappings(const Event::EventSet& events,
+                                             EventMode mode)
 {
   for (const auto& event : events)
     enableMapping(event, mode);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PhysicalKeyboardHandler::enableMapping(const Event::Type event, EventMode mode)
+void PhysicalKeyboardHandler::enableMapping(const Event::Type event,
+                                            EventMode mode)
 {
   // copy from controller mode into emulation mode
   KeyMap::MappingArray mappings = myKeyMap.getEventMapping(event, mode);
@@ -270,7 +318,8 @@ void PhysicalKeyboardHandler::enableMapping(const Event::Type event, EventMode m
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-EventMode PhysicalKeyboardHandler::getEventMode(const Event::Type event, const EventMode mode) const
+EventMode PhysicalKeyboardHandler::getEventMode(const Event::Type event,
+                                                const EventMode mode) const
 {
   if (mode == EventMode::kEmulationMode)
   {
@@ -328,11 +377,11 @@ void PhysicalKeyboardHandler::eraseMapping(Event::Type event, EventMode mode)
 void PhysicalKeyboardHandler::saveMapping()
 {
   myOSystem.settings().setValue("event_ver", Event::VERSION);
-  myOSystem.settings().setValue("keymap_emu", myKeyMap.saveMapping(EventMode::kCommonMode));
-  myOSystem.settings().setValue("keymap_joy", myKeyMap.saveMapping(EventMode::kJoystickMode));
-  myOSystem.settings().setValue("keymap_pad", myKeyMap.saveMapping(EventMode::kPaddlesMode));
-  myOSystem.settings().setValue("keymap_key", myKeyMap.saveMapping(EventMode::kKeypadMode));
-  myOSystem.settings().setValue("keymap_ui", myKeyMap.saveMapping(EventMode::kMenuMode));
+  myOSystem.settings().setValue("keymap_emu", myKeyMap.saveMapping(EventMode::kCommonMode).dump(2));
+  myOSystem.settings().setValue("keymap_joy", myKeyMap.saveMapping(EventMode::kJoystickMode).dump(2));
+  myOSystem.settings().setValue("keymap_pad", myKeyMap.saveMapping(EventMode::kPaddlesMode).dump(2));
+  myOSystem.settings().setValue("keymap_key", myKeyMap.saveMapping(EventMode::kKeypadMode).dump(2));
+  myOSystem.settings().setValue("keymap_ui", myKeyMap.saveMapping(EventMode::kMenuMode).dump(2));
   enableEmulationMappings();
 }
 
@@ -356,7 +405,7 @@ bool PhysicalKeyboardHandler::addMapping(Event::Type event, EventMode mode,
       myKeyMap.erase(EventMode::kKeypadMode, key, mod);
       myKeyMap.erase(EventMode::kCompuMateMode, key, mod);
     }
-    else if(evMode != EventMode::kMenuMode)
+    else if(evMode != EventMode::kMenuMode && evMode != EventMode::kEditMode)
     {
       // erase identical mapping for kCommonMode
       myKeyMap.erase(EventMode::kCommonMode, key, mod);
@@ -371,11 +420,12 @@ bool PhysicalKeyboardHandler::addMapping(Event::Type event, EventMode mode,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PhysicalKeyboardHandler::handleEvent(StellaKey key, StellaMod mod, bool pressed, bool repeated)
+void PhysicalKeyboardHandler::handleEvent(StellaKey key, StellaMod mod,
+                                          bool pressed, bool repeated)
 {
 #ifdef BSPF_UNIX
   // Swallow KBDK_TAB under certain conditions
-  // See commments on 'myAltKeyCounter' for more information
+  // See comments on 'myAltKeyCounter' for more information
   if(myAltKeyCounter > 1 && key == KBDK_TAB)
   {
     myAltKeyCounter = 0;
@@ -431,168 +481,226 @@ void PhysicalKeyboardHandler::handleEvent(StellaKey key, StellaMod mod, bool pre
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultCommonMapping = {
-  {Event::ConsoleSelect,            KBDK_F1},
-  {Event::ConsoleReset,             KBDK_F2},
-  {Event::ConsoleColor,             KBDK_F3},
-  {Event::Console7800Pause,         KBDK_F3, MOD3},
-  {Event::ConsoleLeftDiffA,         KBDK_F5},
-  {Event::ConsoleRightDiffA,        KBDK_F7},
-  {Event::SaveState,                KBDK_F9},
-  {Event::SaveAllStates,            KBDK_F9, MOD3},
-  {Event::PreviousState,            KBDK_F10, KBDM_SHIFT},
-  {Event::NextState,                KBDK_F10},
-  {Event::ToggleAutoSlot,           KBDK_F10, MOD3},
-  {Event::LoadState,                KBDK_F11},
-  {Event::LoadAllStates,            KBDK_F11, MOD3},
-  {Event::TakeSnapshot,             KBDK_F12},
-#ifdef BSPF_MACOS
-  {Event::TogglePauseMode,          KBDK_P, KBDM_SHIFT | MOD3},
-#else
-  {Event::TogglePauseMode,          KBDK_PAUSE},
-#endif
-  {Event::OptionsMenuMode,          KBDK_TAB},
-  {Event::CmdMenuMode,              KBDK_BACKSLASH},
-  {Event::TimeMachineMode,          KBDK_T, KBDM_SHIFT},
-  {Event::DebuggerMode,             KBDK_GRAVE},
-  {Event::ExitMode,                 KBDK_ESCAPE},
-#ifdef BSPF_MACOS
-  {Event::Quit,                     KBDK_Q, MOD3},
-#else
-  {Event::Quit,                     KBDK_Q, KBDM_CTRL},
-#endif
-  {Event::ReloadConsole,            KBDK_R, KBDM_CTRL},
-  {Event::PreviousMultiCartRom,     KBDK_R, KBDM_SHIFT | KBDM_CTRL},
+void PhysicalKeyboardHandler::toggleModKeys(bool toggle)
+{
+  bool modCombo = myOSystem.settings().getBool("modcombo");
 
-  {Event::VidmodeDecrease,          KBDK_MINUS, MOD3},
-  {Event::VidmodeIncrease,          KBDK_EQUALS, MOD3},
-  {Event::VCenterDecrease,          KBDK_PAGEUP, MOD3},
-  {Event::VCenterIncrease,          KBDK_PAGEDOWN, MOD3},
-  {Event::VSizeAdjustDecrease,      KBDK_PAGEDOWN, KBDM_SHIFT | MOD3},
-  {Event::VSizeAdjustIncrease,      KBDK_PAGEUP, KBDM_SHIFT | MOD3},
-  {Event::ToggleCorrectAspectRatio, KBDK_C, KBDM_CTRL},
-  {Event::VolumeDecrease,           KBDK_LEFTBRACKET, MOD3},
-  {Event::VolumeIncrease,           KBDK_RIGHTBRACKET, MOD3},
-  {Event::SoundToggle,              KBDK_RIGHTBRACKET, KBDM_CTRL},
+  if(toggle)
+  {
+    modCombo = !modCombo;
+    myKeyMap.enableMod() = modCombo;
+    myOSystem.settings().setValue("modcombo", modCombo);
+  }
 
-  {Event::ToggleFullScreen,         KBDK_RETURN, MOD3},
-  {Event::ToggleAdaptRefresh,       KBDK_R, MOD3},
-  {Event::OverscanDecrease,         KBDK_PAGEDOWN, KBDM_SHIFT},
-  {Event::OverscanIncrease,         KBDK_PAGEUP, KBDM_SHIFT},
-  //{Event::VidmodeStd,               KBDK_1, MOD3},
-  //{Event::VidmodeRGB,               KBDK_2, MOD3},
-  //{Event::VidmodeSVideo,            KBDK_3, MOD3},
-  //{Event::VidModeComposite,         KBDK_4, MOD3},
-  //{Event::VidModeBad,               KBDK_5, MOD3},
-  //{Event::VidModeCustom,            KBDK_6, MOD3},
-  {Event::PreviousVideoMode,        KBDK_1, KBDM_SHIFT | MOD3},
-  {Event::NextVideoMode,            KBDK_1, MOD3},
-  {Event::PreviousAttribute,        KBDK_2, KBDM_SHIFT | MOD3},
-  {Event::NextAttribute,            KBDK_2, MOD3},
-  {Event::DecreaseAttribute,        KBDK_3, KBDM_SHIFT | MOD3},
-  {Event::IncreaseAttribute,        KBDK_3, MOD3},
-  {Event::PhosphorDecrease,         KBDK_4, KBDM_SHIFT | MOD3},
-  {Event::PhosphorIncrease,         KBDK_4, MOD3},
-  {Event::TogglePhosphor,           KBDK_P, MOD3},
-  {Event::ScanlinesDecrease,        KBDK_5, KBDM_SHIFT | MOD3},
-  {Event::ScanlinesIncrease,        KBDK_5, MOD3},
-  {Event::PreviousPaletteAttribute, KBDK_9, KBDM_SHIFT | MOD3},
-  {Event::NextPaletteAttribute,     KBDK_9, MOD3},
-  {Event::PaletteAttributeDecrease, KBDK_0, KBDM_SHIFT | MOD3},
-  {Event::PaletteAttributeIncrease, KBDK_0, MOD3},
-  {Event::ToggleColorLoss,          KBDK_L, KBDM_CTRL},
-  {Event::PaletteDecrease,          KBDK_P, KBDM_SHIFT | KBDM_CTRL},
-  {Event::PaletteIncrease,          KBDK_P, KBDM_CTRL},
+  ostringstream ss;
+  ss << "Modifier key combos ";
+  ss << (modCombo ? "enabled" : "disabled");
+  myOSystem.frameBuffer().showTextMessage(ss.str());
+}
 
-#ifndef BSPF_MACOS
-  {Event::PreviousSetting,          KBDK_END},
-  {Event::NextSetting,              KBDK_HOME},
-  {Event::PreviousSettingGroup,     KBDK_END, KBDM_CTRL},
-  {Event::NextSettingGroup,         KBDK_HOME, KBDM_CTRL},
-#else
-  // HOME & END keys are swapped on Mac keyboards
-  {Event::PreviousSetting,          KBDK_HOME},
-  {Event::NextSetting,              KBDK_END},
-  {Event::PreviousSettingGroup,     KBDK_HOME, KBDM_CTRL},
-  {Event::NextSettingGroup,         KBDK_END, KBDM_CTRL},
-#endif
-  {Event::PreviousSetting,          KBDK_KP_1},
-  {Event::NextSetting,              KBDK_KP_7},
-  {Event::PreviousSettingGroup,     KBDK_KP_1, KBDM_CTRL},
-  {Event::NextSettingGroup,         KBDK_KP_7, KBDM_CTRL},
-  {Event::SettingDecrease,          KBDK_PAGEDOWN},
-  {Event::SettingDecrease,          KBDK_KP_3, KBDM_CTRL},
-  {Event::SettingIncrease,          KBDK_PAGEUP},
-  {Event::SettingIncrease,          KBDK_KP_9, KBDM_CTRL},
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::DefaultCommonMapping = {
+  { Event::ConsoleSelect,            KBDK_F1 },
+  { Event::ConsoleReset,             KBDK_F2 },
+  { Event::ConsoleColor,             KBDK_F3 },
+  { Event::Console7800Pause,         KBDK_F3, MOD3 },
+  { Event::ConsoleLeftDiffA,         KBDK_F5 },
+  { Event::ConsoleRightDiffA,        KBDK_F7 },
+  { Event::SaveState,                KBDK_F9 },
+  { Event::SaveAllStates,            KBDK_F9, MOD3 },
+  { Event::PreviousState,            KBDK_F10, KBDM_SHIFT },
+  { Event::NextState,                KBDK_F10 },
+  { Event::ToggleAutoSlot,           KBDK_F10, MOD3 },
+  { Event::LoadState,                KBDK_F11 },
+  { Event::LoadAllStates,            KBDK_F11, MOD3 },
+  { Event::TakeSnapshot,             KBDK_F12 },
+  #ifdef BSPF_MACOS
+  { Event::TogglePauseMode,          KBDK_P, KBDM_SHIFT | MOD3 },
+  #else
+  { Event::TogglePauseMode,          KBDK_PAUSE },
+  #endif
+  { Event::OptionsMenuMode,          KBDK_TAB },
+  { Event::CmdMenuMode,              KBDK_BACKSLASH },
+  { Event::TimeMachineMode,          KBDK_T, KBDM_SHIFT },
+  { Event::DebuggerMode,             KBDK_GRAVE },
+  { Event::ExitMode,                 KBDK_ESCAPE },
+  #ifdef BSPF_MACOS
+  { Event::Quit,                     KBDK_Q, MOD3 },
+  #else
+  { Event::Quit,                     KBDK_Q, KBDM_CTRL },
+  #endif
+  { Event::ReloadConsole,            KBDK_R, KBDM_CTRL },
+  { Event::PreviousMultiCartRom,     KBDK_R, KBDM_SHIFT | KBDM_CTRL },
 
-  {Event::ToggleInter,              KBDK_I, KBDM_CTRL},
-  {Event::DecreaseSpeed,            KBDK_S, KBDM_SHIFT | KBDM_CTRL},
-  {Event::IncreaseSpeed,            KBDK_S, KBDM_CTRL },
-  {Event::ToggleTurbo,              KBDK_T, KBDM_CTRL},
-  {Event::ToggleJitter,             KBDK_J, MOD3},
-  {Event::ToggleFrameStats,         KBDK_L, MOD3},
-  {Event::ToggleTimeMachine,        KBDK_T, MOD3},
+  { Event::VidmodeDecrease,          KBDK_MINUS, MOD3 },
+  { Event::VidmodeIncrease,          KBDK_EQUALS, MOD3 },
+  { Event::VCenterDecrease,          KBDK_PAGEUP, MOD3 },
+  { Event::VCenterIncrease,          KBDK_PAGEDOWN, MOD3 },
+  { Event::VSizeAdjustDecrease,      KBDK_PAGEDOWN, KBDM_SHIFT | MOD3 },
+  { Event::VSizeAdjustIncrease,      KBDK_PAGEUP, KBDM_SHIFT | MOD3 },
+  { Event::ToggleCorrectAspectRatio, KBDK_C, KBDM_CTRL },
+  { Event::VolumeDecrease,           KBDK_LEFTBRACKET, MOD3 },
+  { Event::VolumeIncrease,           KBDK_RIGHTBRACKET, MOD3 },
+  { Event::SoundToggle,              KBDK_RIGHTBRACKET, KBDM_CTRL },
 
-#ifdef PNG_SUPPORT
-  {Event::ToggleContSnapshots,      KBDK_S, MOD3 | KBDM_CTRL},
-  {Event::ToggleContSnapshotsFrame, KBDK_S, KBDM_SHIFT | MOD3 | KBDM_CTRL},
-#endif
+  { Event::ToggleFullScreen,         KBDK_RETURN, MOD3 },
+  { Event::ToggleAdaptRefresh,       KBDK_R, MOD3 },
+  { Event::OverscanDecrease,         KBDK_PAGEDOWN, KBDM_SHIFT },
+  { Event::OverscanIncrease,         KBDK_PAGEUP, KBDM_SHIFT },
+    //{Event::VidmodeStd,               KBDK_1, MOD3},
+    //{Event::VidmodeRGB,               KBDK_2, MOD3},
+    //{Event::VidmodeSVideo,            KBDK_3, MOD3},
+    //{Event::VidModeComposite,         KBDK_4, MOD3},
+    //{Event::VidModeBad,               KBDK_5, MOD3},
+    //{Event::VidModeCustom,            KBDK_6, MOD3},
+  { Event::PreviousVideoMode, KBDK_1, KBDM_SHIFT | MOD3 },
+  { Event::NextVideoMode,            KBDK_1, MOD3 },
+  { Event::PreviousAttribute,        KBDK_2, KBDM_SHIFT | MOD3 },
+  { Event::NextAttribute,            KBDK_2, MOD3 },
+  { Event::DecreaseAttribute,        KBDK_3, KBDM_SHIFT | MOD3 },
+  { Event::IncreaseAttribute,        KBDK_3, MOD3 },
+  { Event::PhosphorDecrease,         KBDK_4, KBDM_SHIFT | MOD3 },
+  { Event::PhosphorIncrease,         KBDK_4, MOD3 },
+  { Event::TogglePhosphor,           KBDK_P, MOD3 },
+  { Event::ScanlinesDecrease,        KBDK_5, KBDM_SHIFT | MOD3 },
+  { Event::ScanlinesIncrease,        KBDK_5, MOD3 },
+  { Event::PreviousPaletteAttribute, KBDK_9, KBDM_SHIFT | MOD3 },
+  { Event::NextPaletteAttribute,     KBDK_9, MOD3 },
+  { Event::PaletteAttributeDecrease, KBDK_0, KBDM_SHIFT | MOD3 },
+  { Event::PaletteAttributeIncrease, KBDK_0, MOD3 },
+  { Event::ToggleColorLoss,          KBDK_L, KBDM_CTRL },
+  { Event::PaletteDecrease,          KBDK_P, KBDM_SHIFT | KBDM_CTRL },
+  { Event::PaletteIncrease,          KBDK_P, KBDM_CTRL },
+  { Event::FormatDecrease,           KBDK_F, KBDM_SHIFT | KBDM_CTRL },
+  { Event::FormatIncrease,           KBDK_F, KBDM_CTRL },
+  #ifndef BSPF_MACOS
+  { Event::PreviousSetting,          KBDK_END },
+  { Event::NextSetting,              KBDK_HOME },
+  { Event::PreviousSettingGroup,     KBDK_END, KBDM_CTRL },
+  { Event::NextSettingGroup,         KBDK_HOME, KBDM_CTRL },
+  #else
+    // HOME & END keys are swapped on Mac keyboards
+  { Event::PreviousSetting,          KBDK_HOME },
+  { Event::NextSetting,              KBDK_END },
+  { Event::PreviousSettingGroup,     KBDK_HOME, KBDM_CTRL },
+  { Event::NextSettingGroup,         KBDK_END, KBDM_CTRL },
+  #endif
+  { Event::PreviousSetting,          KBDK_KP_1 },
+  { Event::NextSetting,              KBDK_KP_7 },
+  { Event::PreviousSettingGroup,     KBDK_KP_1, KBDM_CTRL },
+  { Event::NextSettingGroup,         KBDK_KP_7, KBDM_CTRL },
+  { Event::SettingDecrease,          KBDK_PAGEDOWN },
+  { Event::SettingDecrease,          KBDK_KP_3, KBDM_CTRL },
+  { Event::SettingIncrease,          KBDK_PAGEUP },
+  { Event::SettingIncrease,          KBDK_KP_9, KBDM_CTRL },
 
-  {Event::DecreaseAutoFire,         KBDK_A, KBDM_SHIFT | KBDM_CTRL},
-  {Event::IncreaseAutoFire,         KBDK_A, KBDM_CTRL },
-  {Event::HandleMouseControl,       KBDK_0, KBDM_CTRL},
-  {Event::ToggleGrabMouse,          KBDK_G, KBDM_CTRL},
-  {Event::ToggleSAPortOrder,        KBDK_1, KBDM_CTRL},
-  {Event::FormatDecrease,           KBDK_F, KBDM_SHIFT | KBDM_CTRL},
-  {Event::FormatIncrease,           KBDK_F, KBDM_CTRL},
+  { Event::ToggleInter,              KBDK_I, KBDM_CTRL },
+  { Event::DecreaseSpeed,            KBDK_S, KBDM_SHIFT | KBDM_CTRL },
+  { Event::IncreaseSpeed,            KBDK_S, KBDM_CTRL },
+  { Event::ToggleTurbo,              KBDK_T, KBDM_CTRL },
+  { Event::ToggleJitter,             KBDK_J, MOD3 },
+  { Event::ToggleFrameStats,         KBDK_L, MOD3 },
+  { Event::ToggleTimeMachine,        KBDK_T, MOD3 },
 
-  {Event::ToggleP0Collision,        KBDK_Z, KBDM_SHIFT | MOD3},
-  {Event::ToggleP0Bit,              KBDK_Z, MOD3},
-  {Event::ToggleP1Collision,        KBDK_X, KBDM_SHIFT | MOD3},
-  {Event::ToggleP1Bit,              KBDK_X, MOD3},
-  {Event::ToggleM0Collision,        KBDK_C, KBDM_SHIFT | MOD3},
-  {Event::ToggleM0Bit,              KBDK_C, MOD3},
-  {Event::ToggleM1Collision,        KBDK_V, KBDM_SHIFT | MOD3},
-  {Event::ToggleM1Bit,              KBDK_V, MOD3},
-  {Event::ToggleBLCollision,        KBDK_B, KBDM_SHIFT | MOD3},
-  {Event::ToggleBLBit,              KBDK_B, MOD3},
-  {Event::TogglePFCollision,        KBDK_N, KBDM_SHIFT | MOD3},
-  {Event::TogglePFBit,              KBDK_N, MOD3},
-  {Event::ToggleCollisions,         KBDK_COMMA, KBDM_SHIFT | MOD3},
-  {Event::ToggleBits,               KBDK_COMMA, MOD3},
-  {Event::ToggleFixedColors,        KBDK_PERIOD, MOD3},
+  #ifdef PNG_SUPPORT
+  { Event::ToggleContSnapshots,      KBDK_S, MOD3 | KBDM_CTRL },
+  { Event::ToggleContSnapshotsFrame, KBDK_S, KBDM_SHIFT | MOD3 | KBDM_CTRL },
+  #endif
 
-  {Event::RewindPause,              KBDK_LEFT, KBDM_SHIFT},
-  {Event::Rewind1Menu,              KBDK_LEFT, MOD3},
-  {Event::Rewind10Menu,             KBDK_LEFT, KBDM_SHIFT | MOD3},
-  {Event::RewindAllMenu,            KBDK_DOWN, MOD3},
-  {Event::UnwindPause,              KBDK_LEFT, KBDM_SHIFT},
-  {Event::Unwind1Menu,              KBDK_RIGHT, MOD3},
-  {Event::Unwind10Menu,             KBDK_RIGHT, KBDM_SHIFT | MOD3},
-  {Event::UnwindAllMenu,            KBDK_UP, MOD3},
-  {Event::TogglePlayBackMode,       KBDK_SPACE, KBDM_SHIFT},
+  { Event::DecreaseDeadzone,         KBDK_F1, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreaseDeadzone,         KBDK_F1, KBDM_CTRL },
+  { Event::DecAnalogSense,           KBDK_F2, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncAnalogSense,           KBDK_F2, KBDM_CTRL },
+  { Event::DecDejtterAveraging,      KBDK_F3, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncDejtterAveraging,      KBDK_F3, KBDM_CTRL },
+  { Event::DecDejtterReaction,       KBDK_F4, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncDejtterReaction,       KBDK_F4, KBDM_CTRL },
+  { Event::DecDigitalSense,          KBDK_F5, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncDigitalSense,          KBDK_F5, KBDM_CTRL },
+  { Event::DecreaseAutoFire,         KBDK_A, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreaseAutoFire,         KBDK_A, KBDM_CTRL },
+  { Event::ToggleFourDirections,     KBDK_F6, KBDM_CTRL },
+  { Event::ToggleKeyCombos,          KBDK_F7, KBDM_CTRL },
+  { Event::ToggleSAPortOrder,        KBDK_1, KBDM_CTRL },
 
-#if defined(RETRON77)
-  {Event::ConsoleColorToggle,       KBDK_F4},         // back ("COLOR","B/W")
-  {Event::ConsoleLeftDiffToggle,    KBDK_F6},         // front ("SKILL P1")
-  {Event::ConsoleRightDiffToggle,   KBDK_F8},         // front ("SKILL P2")
-  {Event::CmdMenuMode,              KBDK_F13},        // back ("4:3","16:9")
-  {Event::ExitMode,                 KBDK_BACKSPACE},  // back ("FRY")
-#else // defining duplicate keys must be avoided!
-  {Event::ConsoleBlackWhite,        KBDK_F4},
-  {Event::ConsoleLeftDiffB,         KBDK_F6},
-  {Event::ConsoleRightDiffB,        KBDK_F8},
-  {Event::Fry,                      KBDK_BACKSPACE},
+  { Event::PrevMouseAsController,    KBDK_F8, KBDM_CTRL | KBDM_SHIFT },
+  { Event::NextMouseAsController,    KBDK_F8, KBDM_CTRL },
+  { Event::DecMousePaddleSense,      KBDK_F9, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncMousePaddleSense,      KBDK_F9, KBDM_CTRL },
+  { Event::DecMouseTrackballSense,   KBDK_F10, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncMouseTrackballSense,   KBDK_F10, KBDM_CTRL },
+  { Event::DecreaseDrivingSense,     KBDK_F11, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreaseDrivingSense,     KBDK_F11, KBDM_CTRL },
+  { Event::PreviousCursorVisbility,  KBDK_F12, KBDM_CTRL | KBDM_SHIFT },
+  { Event::NextCursorVisbility,      KBDK_F12, KBDM_CTRL },
+  { Event::ToggleGrabMouse,          KBDK_G, KBDM_CTRL },
+
+  { Event::PreviousLeftPort,         KBDK_2, KBDM_CTRL | KBDM_SHIFT },
+  { Event::NextLeftPort,             KBDK_2, KBDM_CTRL },
+  { Event::PreviousRightPort,        KBDK_3, KBDM_CTRL | KBDM_SHIFT },
+  { Event::NextRightPort,            KBDK_3, KBDM_CTRL },
+  { Event::ToggleSwapPorts,          KBDK_4, KBDM_CTRL },
+  { Event::ToggleSwapPaddles,        KBDK_5, KBDM_CTRL },
+  { Event::DecreasePaddleCenterX,    KBDK_6, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreasePaddleCenterX,    KBDK_6, KBDM_CTRL },
+  { Event::DecreasePaddleCenterY,    KBDK_7, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreasePaddleCenterY,    KBDK_7, KBDM_CTRL },
+  { Event::PreviousMouseControl,     KBDK_0, KBDM_CTRL | KBDM_SHIFT },
+  { Event::NextMouseControl,         KBDK_0, KBDM_CTRL },
+  { Event::DecreaseMouseAxesRange,   KBDK_8, KBDM_CTRL | KBDM_SHIFT },
+  { Event::IncreaseMouseAxesRange,   KBDK_8, KBDM_CTRL },
+
+  { Event::ToggleP0Collision,        KBDK_Z, KBDM_SHIFT | MOD3 },
+  { Event::ToggleP0Bit,              KBDK_Z, MOD3 },
+  { Event::ToggleP1Collision,        KBDK_X, KBDM_SHIFT | MOD3 },
+  { Event::ToggleP1Bit,              KBDK_X, MOD3 },
+  { Event::ToggleM0Collision,        KBDK_C, KBDM_SHIFT | MOD3 },
+  { Event::ToggleM0Bit,              KBDK_C, MOD3 },
+  { Event::ToggleM1Collision,        KBDK_V, KBDM_SHIFT | MOD3 },
+  { Event::ToggleM1Bit,              KBDK_V, MOD3 },
+  { Event::ToggleBLCollision,        KBDK_B, KBDM_SHIFT | MOD3 },
+  { Event::ToggleBLBit,              KBDK_B, MOD3 },
+  { Event::TogglePFCollision,        KBDK_N, KBDM_SHIFT | MOD3 },
+  { Event::TogglePFBit,              KBDK_N, MOD3 },
+  { Event::ToggleCollisions,         KBDK_COMMA, KBDM_SHIFT | MOD3 },
+  { Event::ToggleBits,               KBDK_COMMA, MOD3 },
+  { Event::ToggleFixedColors,        KBDK_PERIOD, MOD3 },
+
+  { Event::RewindPause,              KBDK_LEFT, KBDM_SHIFT },
+  { Event::Rewind1Menu,              KBDK_LEFT, MOD3 },
+  { Event::Rewind10Menu,             KBDK_LEFT, KBDM_SHIFT | MOD3 },
+  { Event::RewindAllMenu,            KBDK_DOWN, MOD3 },
+  { Event::UnwindPause,              KBDK_LEFT, KBDM_SHIFT },
+  { Event::Unwind1Menu,              KBDK_RIGHT, MOD3 },
+  { Event::Unwind10Menu,             KBDK_RIGHT, KBDM_SHIFT | MOD3 },
+  { Event::UnwindAllMenu,            KBDK_UP, MOD3 },
+  { Event::HighScoresMenuMode,       KBDK_INSERT },
+  { Event::TogglePlayBackMode,       KBDK_SPACE, KBDM_SHIFT },
+
+  #if defined(RETRON77)
+  { Event::ConsoleColorToggle,       KBDK_F4 },         // back ("COLOR","B/W")
+  { Event::ConsoleLeftDiffToggle,    KBDK_F6 },         // front ("SKILL P1")
+  { Event::ConsoleRightDiffToggle,   KBDK_F8 },         // front ("SKILL P2")
+  { Event::CmdMenuMode,              KBDK_F13 },        // back ("4:3","16:9")
+  { Event::ExitMode,                 KBDK_BACKSPACE },  // back ("FRY")
+  #else // defining duplicate keys must be avoided!
+  { Event::ConsoleBlackWhite,        KBDK_F4 },
+  { Event::ConsoleLeftDiffB,         KBDK_F6 },
+  { Event::ConsoleRightDiffB,        KBDK_F8 },
+  { Event::Fry,                      KBDK_BACKSPACE },
 #endif
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultMenuMapping = {
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::DefaultMenuMapping = {
   {Event::UIUp,                     KBDK_UP},
   {Event::UIDown,                   KBDK_DOWN},
   {Event::UILeft,                   KBDK_LEFT},
   {Event::UIRight,                  KBDK_RIGHT},
   {Event::UISelect,                 KBDK_RETURN},
+  {Event::UISelect,                 KBDK_SPACE},
 
   {Event::UIHome,                   KBDK_HOME},
   {Event::UIEnd,                    KBDK_END},
@@ -640,6 +748,88 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultMenuM
 #endif
 };
 
+#ifdef GUI_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::FixedEditMapping = {
+  {Event::MoveLeftChar,             KBDK_LEFT},
+  {Event::MoveRightChar,            KBDK_RIGHT},
+  {Event::SelectLeftChar,           KBDK_LEFT, KBDM_SHIFT},
+  {Event::SelectRightChar,          KBDK_RIGHT, KBDM_SHIFT},
+#if defined(BSPF_MACOS) || defined(MACOS_KEYS)
+  {Event::MoveLeftWord,             KBDK_LEFT, OPTION},
+  {Event::MoveRightWord,            KBDK_RIGHT, OPTION},
+  {Event::MoveHome,                 KBDK_HOME},
+  {Event::MoveHome,                 KBDK_A, KBDM_CTRL},
+  {Event::MoveHome,                 KBDK_LEFT, CMD},
+  {Event::MoveEnd,                  KBDK_END},
+  {Event::MoveEnd,                  KBDK_E, KBDM_CTRL},
+  {Event::MoveEnd,                  KBDK_RIGHT, CMD},
+  {Event::SelectLeftWord,           KBDK_LEFT, KBDM_SHIFT | OPTION},
+  {Event::SelectRightWord,          KBDK_RIGHT, KBDM_SHIFT | OPTION},
+  {Event::SelectHome,               KBDK_HOME, KBDM_SHIFT},
+  {Event::SelectHome,               KBDK_LEFT, KBDM_SHIFT | CMD},
+  {Event::SelectHome,               KBDK_A, KBDM_CTRL | KBDM_SHIFT},
+  {Event::SelectEnd,                KBDK_E, KBDM_SHIFT | KBDM_CTRL},
+  {Event::SelectEnd,                KBDK_RIGHT, KBDM_SHIFT | CMD},
+  {Event::SelectEnd,                KBDK_END, KBDM_SHIFT},
+  {Event::SelectAll,                KBDK_A, CMD},
+  {Event::Delete,                   KBDK_DELETE},
+  {Event::Delete,                   KBDK_D, KBDM_CTRL},
+  {Event::DeleteLeftWord,           KBDK_W, KBDM_CTRL},
+  {Event::DeleteLeftWord,           KBDK_BACKSPACE, OPTION},
+  {Event::DeleteRightWord,          KBDK_DELETE, OPTION},
+  {Event::DeleteHome,               KBDK_U, KBDM_CTRL},
+  {Event::DeleteHome,               KBDK_BACKSPACE, CMD},
+  {Event::DeleteEnd,                KBDK_K, KBDM_CTRL},
+  {Event::Backspace,                KBDK_BACKSPACE},
+  {Event::Undo,                     KBDK_Z, CMD},
+  {Event::Redo,                     KBDK_Y, CMD},
+  {Event::Redo,                     KBDK_Z, KBDM_SHIFT | CMD},
+  {Event::Cut,                      KBDK_X, CMD},
+  {Event::Copy,                     KBDK_C, CMD},
+  {Event::Paste,                    KBDK_V, CMD},
+#else
+  {Event::MoveLeftWord,             KBDK_LEFT, KBDM_CTRL},
+  {Event::MoveRightWord,            KBDK_RIGHT, KBDM_CTRL},
+  {Event::MoveHome,                 KBDK_HOME},
+  {Event::MoveEnd,                  KBDK_END},
+  {Event::SelectLeftWord,           KBDK_LEFT, KBDM_SHIFT | KBDM_CTRL},
+  {Event::SelectRightWord,          KBDK_RIGHT, KBDM_SHIFT | KBDM_CTRL},
+  {Event::SelectHome,               KBDK_HOME, KBDM_SHIFT},
+  {Event::SelectEnd,                KBDK_END, KBDM_SHIFT},
+  {Event::SelectAll,                KBDK_A, KBDM_CTRL},
+  {Event::Delete,                   KBDK_DELETE},
+  {Event::Delete,                   KBDK_KP_PERIOD},
+  {Event::Delete,                   KBDK_D, KBDM_CTRL},
+  {Event::DeleteLeftWord,           KBDK_BACKSPACE, KBDM_CTRL},
+  {Event::DeleteLeftWord,           KBDK_W, KBDM_CTRL},
+  {Event::DeleteRightWord,          KBDK_DELETE, KBDM_CTRL},
+  {Event::DeleteRightWord,          KBDK_D, KBDM_ALT},
+  {Event::DeleteHome,               KBDK_HOME, KBDM_CTRL},
+  {Event::DeleteHome,               KBDK_U, KBDM_CTRL},
+  {Event::DeleteEnd,                KBDK_END, KBDM_CTRL},
+  {Event::DeleteEnd,                KBDK_K, KBDM_CTRL},
+  {Event::Backspace,                KBDK_BACKSPACE},
+  {Event::Undo,                     KBDK_Z, KBDM_CTRL},
+  {Event::Undo,                     KBDK_BACKSPACE, KBDM_ALT},
+  {Event::Redo,                     KBDK_Y, KBDM_CTRL},
+  {Event::Redo,                     KBDK_Z, KBDM_SHIFT | KBDM_CTRL},
+  {Event::Redo,                     KBDK_BACKSPACE, KBDM_SHIFT | KBDM_ALT},
+  {Event::Cut,                      KBDK_X, KBDM_CTRL},
+  {Event::Cut,                      KBDK_DELETE, KBDM_SHIFT},
+  {Event::Cut,                      KBDK_KP_PERIOD, KBDM_SHIFT},
+  {Event::Copy,                     KBDK_C, KBDM_CTRL},
+  {Event::Copy,                     KBDK_INSERT, KBDM_CTRL},
+  {Event::Paste,                    KBDK_V, KBDM_CTRL},
+  {Event::Paste,                    KBDK_INSERT, KBDM_SHIFT},
+#endif
+  {Event::EndEdit,                  KBDK_RETURN},
+  {Event::EndEdit,                  KBDK_KP_ENTER},
+  {Event::AbortEdit,                KBDK_ESCAPE},
+};
+#endif
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultJoystickMapping = {
   {Event::JoystickZeroUp,           KBDK_UP},
@@ -686,7 +876,8 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultJoyst
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultPaddleMapping = {
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::DefaultPaddleMapping = {
   {Event::PaddleZeroDecrease,       KBDK_RIGHT},
   {Event::PaddleZeroIncrease,       KBDK_LEFT},
   {Event::PaddleZeroFire,           KBDK_SPACE},
@@ -708,7 +899,8 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultPaddl
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultKeypadMapping = {
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::DefaultKeypadMapping = {
   {Event::KeyboardZero1,            KBDK_1},
   {Event::KeyboardZero2,            KBDK_2},
   {Event::KeyboardZero3,            KBDK_3},
@@ -737,7 +929,8 @@ PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::DefaultKeypa
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PhysicalKeyboardHandler::EventMappingArray PhysicalKeyboardHandler::CompuMateMapping = {
+PhysicalKeyboardHandler::EventMappingArray
+PhysicalKeyboardHandler::CompuMateMapping = {
   {Event::CompuMateShift,         KBDK_LSHIFT},
   {Event::CompuMateShift,         KBDK_RSHIFT},
   {Event::CompuMateFunc,          KBDK_LCTRL},
