@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,8 +22,9 @@
 
 #include "PaletteHandler.hxx"
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PaletteHandler::PaletteHandler(OSystem& system)
-  : myOSystem(system)
+  : myOSystem{system}
 {
   // Load user-defined palette for this ROM
   loadUserPalette();
@@ -69,7 +70,7 @@ void PaletteHandler::cyclePalette(int direction)
   const string palette = toPaletteName(PaletteType(type));
   const string message = MESSAGES[type] + " palette";
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 
   setPalette(palette);
 }
@@ -112,7 +113,7 @@ void PaletteHandler::showAdjustableMessage()
     const float value =
         myOSystem.console().timing() == ConsoleTiming::pal ? myPhasePAL : myPhaseNTSC;
     buf << std::fixed << std::setprecision(1) << value << DEGREE;
-    myOSystem.frameBuffer().showMessage(
+    myOSystem.frameBuffer().showGaugeMessage(
         "Palette phase shift", buf.str(), value,
         (isNTSC ? DEF_NTSC_SHIFT : DEF_PAL_SHIFT) - MAX_PHASE_SHIFT,
         (isNTSC ? DEF_NTSC_SHIFT : DEF_PAL_SHIFT) + MAX_PHASE_SHIFT);
@@ -122,7 +123,7 @@ void PaletteHandler::showAdjustableMessage()
     const float value = *myAdjustables[myCurrentAdjustable].value;
 
     buf << std::fixed << std::setprecision(1) << value << DEGREE;
-    myOSystem.frameBuffer().showMessage(
+    myOSystem.frameBuffer().showGaugeMessage(
       msg.str(), buf.str(), value, -MAX_RGB_SHIFT, +MAX_RGB_SHIFT);
   }
   else
@@ -131,7 +132,7 @@ void PaletteHandler::showAdjustableMessage()
       ? scaleRGBTo100(*myAdjustables[myCurrentAdjustable].value)
       : scaleTo100(*myAdjustables[myCurrentAdjustable].value);
     buf << value << "%";
-    myOSystem.frameBuffer().showMessage(
+    myOSystem.frameBuffer().showGaugeMessage(
       msg.str(), buf.str(), value);
   }
 }
@@ -441,7 +442,7 @@ void PaletteHandler::generateCustomPalette(ConsoleTiming timing)
   if(timing == ConsoleTiming::ntsc)
   {
     vector2d IQ[NUM_CHROMA];
-    // YIQ is YUV shifted by 33
+    // YIQ is YUV shifted by 33 degrees
     constexpr float offset = 33 * BSPF::PI_f / 180;
     const float shift = myPhaseNTSC * BSPF::PI_f / 180;
 
