@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -123,11 +123,11 @@ void parseCommandLine(int ac, char* av[],
 
 #if 0
   cout << "Global opts:" << endl;
-  for(const auto& x: globalOpts)
-    cout << " -> " << x.first << ": " << x.second << endl;
+  for(const auto& [key, value]: globalOpts)
+    cout << " -> " << key << ": " << value << endl;
   cout << "Local opts:" << endl;
-  for(const auto& x: localOpts)
-    cout << " -> " << x.first << ": " << x.second << endl;
+  for(const auto& [key, value]: localOpts)
+    cout << " -> " << key << ": " << value << endl;
 #endif
 }
 
@@ -203,12 +203,11 @@ int main(int ac, char* av[])
 
   // Create the parent OSystem object and initialize settings
   theOSystem = MediaFactory::createOSystem();
-  theOSystem->loadConfig(globalOpts);
 
   // Create the full OSystem after the settings, since settings are
   // probably needed for defaults
   Logger::debug("Creating the OSystem ...");
-  if(!theOSystem->create())
+  if(!theOSystem->initialize(globalOpts))
   {
     Logger::error("ERROR: Couldn't create OSystem");
     return Cleanup();
