@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,40 +18,19 @@
 #include "KeyValueRepositoryConfigfile.hxx"
 #include "Logger.hxx"
 
-namespace {
-  string trim(const string& str)
-  {
-    string::size_type first = str.find_first_not_of(' ');
-    return (first == string::npos) ? EmptyString :
-            str.substr(first, str.find_last_not_of(' ')-first+1);
-  }
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 KeyValueRepositoryConfigfile::KeyValueRepositoryConfigfile(const FilesystemNode& file)
-  : myFile(file)
-{
-}
+  : KeyValueRepositoryFile<KeyValueRepositoryConfigfile>(file)
+{}
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-std::map<string, Variant> KeyValueRepositoryConfigfile::load()
+std::map<string, Variant> KeyValueRepositoryConfigfile::load(istream& in)
 {
   std::map<string, Variant> values;
 
   string line, key, value;
   string::size_type equalPos, garbage;
 
-  stringstream in;
-  try
-  {
-    myFile.read(in);
-  }
-  catch(...)
-  {
-    Logger::error("ERROR: Couldn't load from settings file " + myFile.getShortPath());
-    return values;
-  }
-
   while(getline(in, line))
   {
     // Strip all whitespace and tabs from the line
@@ -67,8 +46,8 @@ std::map<string, Variant> KeyValueRepositoryConfigfile::load()
       continue;
 
     // Split the line into key/value pairs and trim any whitespace
-    key   = trim(line.substr(0, equalPos));
-    value = trim(line.substr(equalPos + 1, line.length() - key.length() - 1));
+    key   = BSPF::trim(line.substr(0, equalPos));
+    value = BSPF::trim(line.substr(equalPos + 1, line.length() - key.length() - 1));
 
     // Skip absent key
     if(key.length() == 0)
@@ -81,9 +60,8 @@ std::map<string, Variant> KeyValueRepositoryConfigfile::load()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void KeyValueRepositoryConfigfile::save(const std::map<string, Variant>& values)
+bool KeyValueRepositoryConfigfile::save(ostream& out, const std::map<string, Variant>& values)
 {
-  stringstream out;
   out << ";  Stella configuration file" << endl
       << ";" << endl
       << ";  Lines starting with ';' are comments and are ignored." << endl
@@ -100,15 +78,8 @@ void KeyValueRepositoryConfigfile::save(const std::map<string, Variant>& values)
       << ";" << endl;
 
   // Write out each of the key and value pairs
-  for(const auto& pair: values)
-    out << pair.first << " = " << pair.second << endl;
+  for(const auto& [key, value]: values)
+    out << key << " = " << value << endl;
 
-  try
-  {
-    myFile.write(out);
-  }
-  catch(...)
-  {
-    Logger::error("ERROR: Couldn't save to settings file " + myFile.getShortPath());
-  }
+  return true;
 }
