@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -77,19 +77,11 @@ void NTSCFilter::selectAdjustable(int direction,
 {
   if(direction == +1)
   {
-  #ifdef BLARGG_PALETTE
-    myCurrentAdjustable = (myCurrentAdjustable + 1) % 10;
-  #else
     myCurrentAdjustable = (myCurrentAdjustable + 1) % 5;
-  #endif
   }
   else if(direction == -1)
   {
-  #ifdef BLARGG_PALETTE
-    if(myCurrentAdjustable == 0) myCurrentAdjustable = 9;
-  #else
     if(myCurrentAdjustable == 0) myCurrentAdjustable = 4;
-  #endif
     else                         --myCurrentAdjustable;
   }
 
@@ -139,13 +131,6 @@ void NTSCFilter::loadConfig(const Settings& settings)
 {
 
   // Load adjustables for custom mode
-#ifdef BLARGG_PALETTE
-  myCustomSetup.hue = BSPF::clamp(settings.getFloat("tv.hue"), -1.0F, 1.0F);
-  myCustomSetup.saturation = BSPF::clamp(settings.getFloat("tv.saturation"), -1.0F, 1.0F);
-  myCustomSetup.contrast = BSPF::clamp(settings.getFloat("tv.contrast"), -1.0F, 1.0F);
-  myCustomSetup.brightness = BSPF::clamp(settings.getFloat("tv.brightness"), -1.0F, 1.0F);
-  myCustomSetup.gamma = BSPF::clamp(settings.getFloat("tv.gamma"), -1.0F, 1.0F);
-#endif
   myCustomSetup.sharpness = BSPF::clamp(settings.getFloat("tv.sharpness"), -1.0F, 1.0F);
   myCustomSetup.resolution = BSPF::clamp(settings.getFloat("tv.resolution"), -1.0F, 1.0F);
   myCustomSetup.artifacts = BSPF::clamp(settings.getFloat("tv.artifacts"), -1.0F, 1.0F);
@@ -157,13 +142,6 @@ void NTSCFilter::loadConfig(const Settings& settings)
 void NTSCFilter::saveConfig(Settings& settings) const
 {
   // Save adjustables for custom mode
-#ifdef BLARGG_PALETTE
-  settings.setValue("tv.hue", myCustomSetup.hue);
-  settings.setValue("tv.saturation", myCustomSetup.saturation);
-  settings.setValue("tv.contrast", myCustomSetup.contrast);
-  settings.setValue("tv.brightness", myCustomSetup.brightness);
-  settings.setValue("tv.gamma", myCustomSetup.gamma);
-#endif
   settings.setValue("tv.sharpness", myCustomSetup.sharpness);
   settings.setValue("tv.resolution", myCustomSetup.resolution);
   settings.setValue("tv.artifacts", myCustomSetup.artifacts);
@@ -192,15 +170,8 @@ void NTSCFilter::getAdjustables(Adjustable& adjustable, Preset preset) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void NTSCFilter::setCustomAdjustables(Adjustable& adjustable)
+void NTSCFilter::setCustomAdjustables(const Adjustable& adjustable)
 {
-#ifdef BLARGG_PALETTE
-  //myCustomSetup.hue = scaleFrom100(adjustable.hue);
-  //myCustomSetup.saturation = scaleFrom100(adjustable.saturation);
-  //myCustomSetup.contrast = scaleFrom100(adjustable.contrast);
-  //myCustomSetup.brightness = scaleFrom100(adjustable.brightness);
-  //myCustomSetup.gamma = scaleFrom100(adjustable.gamma);
-#endif
   myCustomSetup.sharpness = scaleFrom100(adjustable.sharpness);
   myCustomSetup.resolution = scaleFrom100(adjustable.resolution);
   myCustomSetup.artifacts = scaleFrom100(adjustable.artifacts);
@@ -212,13 +183,6 @@ void NTSCFilter::setCustomAdjustables(Adjustable& adjustable)
 void NTSCFilter::convertToAdjustable(Adjustable& adjustable,
                                      const AtariNTSC::Setup& setup) const
 {
-#ifdef BLARGG_PALETTE
-  //adjustable.hue         = scaleTo100(setup.hue);
-  //adjustable.saturation  = scaleTo100(setup.saturation);
-  //adjustable.contrast    = scaleTo100(setup.contrast);
-  //adjustable.brightness  = scaleTo100(setup.brightness);
-  //adjustable.gamma       = scaleTo100(setup.gamma);
-#endif
   adjustable.sharpness   = scaleTo100(setup.sharpness);
   adjustable.resolution  = scaleTo100(setup.resolution);
   adjustable.artifacts   = scaleTo100(setup.artifacts);
@@ -230,16 +194,7 @@ void NTSCFilter::convertToAdjustable(Adjustable& adjustable,
 AtariNTSC::Setup NTSCFilter::myCustomSetup = AtariNTSC::TV_Composite;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-#ifdef BLARGG_PALETTE
-const std::array<NTSCFilter::AdjustableTag, 10> NTSCFilter::ourCustomAdjustables = { {
-  { "contrast", &myCustomSetup.contrast },
-  { "brightness", &myCustomSetup.brightness },
-  { "hue", &myCustomSetup.hue },
-  { "saturation", &myCustomSetup.saturation },
-  { "gamma", &myCustomSetup.gamma },
-#else
 const std::array<NTSCFilter::AdjustableTag, int(NTSCFilter::Adjustables::NUM_ADJUSTABLES)> NTSCFilter::ourCustomAdjustables = { {
-#endif
   { "sharpness", &myCustomSetup.sharpness },
   { "resolution", &myCustomSetup.resolution },
   { "artifacts", &myCustomSetup.artifacts },
