@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -48,7 +48,7 @@ using std::right;
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
   : DebuggerSystem(dbg, console),
-    myOSystem(osystem)
+    myOSystem{osystem}
 {
   // Add case sensitive compare for user labels
   // TODO - should user labels be case insensitive too?
@@ -302,12 +302,16 @@ bool CartDebug::disassemble(int bank, uInt16 PC, bool force)
       AddressList::const_iterator i;
       for(i = addresses.cbegin(); i != addresses.cend(); ++i)
       {
-        if (PC == *i)  // already present
+        if(PC == *i)  // already present
           break;
       }
       // Otherwise, add the item at the end
-      if (i == addresses.end())
+      if(i == addresses.end())
+      {
         addresses.push_back(PC);
+        if(!DiStella::settings.resolveCode)
+          addDirective(Device::AccessType::CODE, PC, PC, bank);
+      }
     }
 
     // Always attempt to resolve code sections unless it's been
@@ -347,8 +351,8 @@ bool CartDebug::fillDisassemblyList(BankInfo& info, uInt16 search)
     const DisassemblyTag& tag = myDisassembly.list[i];
     const uInt16 address = tag.address & 0xFFF;
 
-    // Exclude 'Device::ROW'; they don't have a valid address
-    if(tag.type != Device::ROW)
+    // Exclude 'Device::ROW|NONE'; they don't have a valid address
+    if(tag.type != Device::ROW && tag.type != Device::NONE)
     {
       // Create a mapping from addresses to line numbers
       myAddrToLineList.emplace(address, i);
@@ -605,7 +609,8 @@ bool CartDebug::removeLabel(const string& label)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) const
+bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead,
+                         int places, bool isRam) const
 {
   switch(addressType(addr))
   {
@@ -662,21 +667,24 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
     {
       // RAM can use user-defined labels; otherwise we default to
       // standard mnemonics
-      auto iter = myUserLabels.find(addr);
-      if(iter != myUserLabels.end())
-      {
-        buf << iter->second;
-      }
-      else
-      {
-        uInt16 a = addr & 0xFF, offset = addr & 0xFF00;
-        if((iter = myUserLabels.find(a)) != myUserLabels.end())
+      AddrToLabel::const_iterator iter;
+      uInt16 a = addr & 0xFF, offset = addr & 0xFF00;
+      bool found = false;
+
+      // Search for nearest label
+      for(uInt16 i = a; i >= 0x80; --i)
+        if((iter = myUserLabels.find(i)) != myUserLabels.end())
+        {
           buf << iter->second;
-        else
-          buf << ourZPMnemonic[a - 0x80];
-        if(offset > 0)
-          buf << "|$" << Base::HEX2 << offset;
-      }
+          if(a != i)
+            buf << "+$" << Base::HEX1 << (a - i);
+          found = true;
+          break;
+        }
+      if(!found)
+        buf << ourZPMnemonic[a - 0x80];
+      if(offset > 0)
+        buf << "|$" << Base::HEX2 << offset;
 
       return true;
     }
@@ -684,11 +692,28 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
     case AddrType::ROM:
     {
       // These addresses can never be in the system labels list
-      const auto& iter = myUserLabels.find(addr);
-      if(iter != myUserLabels.end())
+      if(isRam) // cartridge RAM
       {
-        buf << iter->second;
-        return true;
+        AddrToLabel::const_iterator iter;
+
+        // Search for nearest label
+        for(uInt16 i = addr; i >= (addr & 0xf000); --i)
+          if((iter = myUserLabels.find(i)) != myUserLabels.end())
+          {
+            buf << iter->second;
+            if(addr != i)
+              buf << "+$" << Base::HEX1 << (addr - i);
+            return true;
+          }
+      }
+      else
+      {
+        const auto& iter = myUserLabels.find(addr);
+        if(iter != myUserLabels.end())
+        {
+          buf << iter->second;
+          return true;
+        }
       }
       break;
     }
@@ -713,10 +738,10 @@ bool CartDebug::getLabel(ostream& buf, uInt16 addr, bool isRead, int places) con
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartDebug::getLabel(uInt16 addr, bool isRead, int places) const
+string CartDebug::getLabel(uInt16 addr, bool isRead, int places, bool isRam) const
 {
   ostringstream buf;
-  getLabel(buf, addr, isRead, places);
+  getLabel(buf, addr, isRead, places, isRam);
   return buf.str();
 }
 
@@ -1020,7 +1045,7 @@ string CartDebug::saveConfigFile()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartDebug::saveDisassembly()
+string CartDebug::saveDisassembly(string path)
 {
   string NTSC_COLOR[16] = {
     "BLACK", "YELLOW", "BROWN", "ORANGE",
@@ -1329,9 +1354,16 @@ string CartDebug::saveDisassembly()
   // And finally, output the disassembly
   out << buf.str();
 
-  const string& propsname =
-    myConsole.properties().get(PropType::Cart_Name) + ".asm";
-  FilesystemNode node(myOSystem.defaultSaveDir().getPath() + propsname);
+
+  if(path.empty())
+    path = myOSystem.userDir().getPath()
+      + myConsole.properties().get(PropType::Cart_Name) + ".asm";
+  else
+    // Append default extension when missing
+    if(path.find_last_of('.') == string::npos)
+      path += ".asm";
+
+  FilesystemNode node(path);
   stringstream retVal;
   try
   {
@@ -1349,11 +1381,18 @@ string CartDebug::saveDisassembly()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartDebug::saveRom()
+string CartDebug::saveRom(string path)
 {
-  const string& rom = myConsole.properties().get(PropType::Cart_Name) + ".a26";
+  if(path.empty())
+    path = myOSystem.userDir().getPath()
+      + myConsole.properties().get(PropType::Cart_Name) + ".a26";
+  else
+    // Append default extension when missing
+    if(path.find_last_of('.') == string::npos)
+      path += ".a26";
+
+  FilesystemNode node(path);
 
-  FilesystemNode node(myOSystem.defaultSaveDir().getPath() + rom);
   if(myConsole.cartridge().saveROM(node))
     return "saved ROM as " + node.getShortPath();
   else
@@ -1361,7 +1400,7 @@ string CartDebug::saveRom()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartDebug::saveAccessFile()
+string CartDebug::saveAccessFile(string path)
 {
   stringstream out;
   out << myConsole.tia().getAccessCounters();
@@ -1370,8 +1409,15 @@ string CartDebug::saveAccessFile()
 
   try
   {
-    const string& rom = myConsole.properties().get(PropType::Cart_Name) + ".csv";
-    FilesystemNode node(myOSystem.defaultSaveDir().getPath() + rom);
+    if(path.empty())
+      path = myOSystem.userDir().getPath()
+        + myConsole.properties().get(PropType::Cart_Name) + ".csv";
+    else
+      // Append default extension when missing
+      if(path.find_last_of('.') == string::npos)
+        path += ".csv";
+
+    FilesystemNode node(path);
 
     node.write(out);
     return "saved access counters as " + node.getShortPath();
@@ -1495,7 +1541,7 @@ CartDebug::AddrType CartDebug::addressType(uInt16 addr) const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartDebug::getBankDirectives(ostream& buf, BankInfo& info) const
+void CartDebug::getBankDirectives(ostream& buf, const BankInfo& info) const
 {
   // Start with the offset for this bank
   buf << "ORG " << Base::HEX4 << info.offset << endl;
