@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -62,8 +62,8 @@ Debugger* Debugger::myStaticDebugger = nullptr;
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Debugger::Debugger(OSystem& osystem, Console& console)
   : DialogContainer(osystem),
-    myConsole(console),
-    mySystem(console.system())
+    myConsole{console},
+    mySystem{console.system()}
 {
   // Init parser
   myParser = make_unique<DebuggerParser>(*this, osystem.settings());
@@ -118,7 +118,8 @@ FBInitStatus Debugger::initializeVideo()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::start(const string& message, int address, bool read)
+bool Debugger::start(const string& message, int address, bool read,
+                     const string& toolTip)
 {
   if(myOSystem.eventHandler().enterDebugMode())
   {
@@ -129,6 +130,7 @@ bool Debugger::start(const string& message, int address, bool read)
     if(address > -1)
       buf << cartDebug().getLabel(address, read, 4);
     myDialog->message().setText(buf.str());
+    myDialog->message().setToolTip(toolTip);
     return true;
   }
   return false;
@@ -714,10 +716,8 @@ bool Debugger::addFunction(const string& name, const string& definition,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Debugger::isBuiltinFunction(const string& name)
 {
-  for(const auto& func: ourBuiltinFunctions)
-    if(name == func.name)
-      return true;
-  return false;
+  return std::any_of(ourBuiltinFunctions.cbegin(), ourBuiltinFunctions.cend(),
+      [&](const auto& func) { return name == func.name; });
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
