@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -35,6 +35,8 @@
 #include "PromptWidget.hxx"
 #include "RomWidget.hxx"
 #include "ProgressDialog.hxx"
+#include "BrowserDialog.hxx"
+#include "FrameBuffer.hxx"
 #include "TimerManager.hxx"
 #include "Vec.hxx"
 
@@ -57,8 +59,8 @@ using std::right;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DebuggerParser::DebuggerParser(Debugger& d, Settings& s)
-  : debugger(d),
-    settings(s)
+  : debugger{d},
+    settings{s}
 {
 }
 
@@ -564,6 +566,12 @@ string DebuggerParser::eval()
   return buf.str();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const string& DebuggerParser::cartName() const
+{
+  return debugger.myOSystem.console().properties().get(PropType::Cart_Name);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerParser::listTraps(bool listCond)
 {
@@ -629,15 +637,11 @@ string DebuggerParser::trapStatus(const Trap& trap)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string DebuggerParser::saveScriptFile(string file)
 {
-  // Append 'script' extension when necessary
-  if(file.find_last_of('.') == string::npos)
-    file += ".script";
-
   stringstream out;
   Debugger::FunctionDefMap funcs = debugger.getFunctionDefMap();
-  for(const auto& f: funcs)
-    if (!debugger.isBuiltinFunction(f.first))
-      out << "function " << f.first << " {" << f.second << "}" << endl;
+  for(const auto& [name, cmd]: funcs)
+    if (!debugger.isBuiltinFunction(name))
+      out << "function " << name << " {" << cmd << "}" << endl;
 
   for(const auto& w: myWatches)
     out << "watch " << w << endl;
@@ -674,7 +678,16 @@ string DebuggerParser::saveScriptFile(string file)
     out << endl;
   }
 
-  FilesystemNode node(debugger.myOSystem.defaultSaveDir().getPath() + file);
+  // Append 'script' extension when necessary
+  if(file.find_last_of('.') == string::npos)
+    file += ".script";
+
+  // Use user dir if no path is provided
+  if(file.find_first_of(FilesystemNode::PATH_SEPARATOR) == string::npos)
+    file = debugger.myOSystem.userDir().getPath() + file;
+
+  FilesystemNode node(file);
+
   try
   {
     node.write(out);
@@ -687,6 +700,21 @@ string DebuggerParser::saveScriptFile(string file)
   return "saved " + node.getShortPath() + " OK";
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DebuggerParser::saveDump(const FilesystemNode& node, const stringstream& out,
+                              ostringstream& result)
+{
+  try
+  {
+    node.write(out);
+    result << " to file " << node.getShortPath();
+  }
+  catch(...)
+  {
+    result.str(red("Unable to append dump to file " + node.getShortPath()));
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerParser::executeDirective(Device::AccessType type)
 {
@@ -1114,7 +1142,7 @@ void DebuggerParser::executeDump()
   };
 
   // Error checking
-  if( argCount == 0 || argCount > 3)
+  if(argCount == 0 || argCount > 4)
   {
     outputCommandError("wrong number of arguments", myCommand);
     return;
@@ -1131,33 +1159,33 @@ void DebuggerParser::executeDump()
     dump(commandResult, args[0], args[1]);
   else
   {
-    ostringstream file;
-    file << debugger.myOSystem.defaultSaveDir()
-         << debugger.myOSystem.console().properties().get(PropType::Cart_Name) << "_dbg_";
-    if(execDepth > 0)
+    if((args[2] & 0x07) == 0)
     {
-      file << execPrefix;
-    }
-    else
-    {
-      file << std::hex << std::setw(8) << std::setfill('0')
-           << uInt32(TimerManager::getTicks() / 1000);
-    }
-    file << ".dump";
-    FilesystemNode node(file.str());
-    // cout << "dump " << args[0] << "-" << args[1] << " to " << file.str() << endl;
-    std::ofstream ofs(node.getPath(), std::ofstream::out | std::ofstream::app);
-    if(!ofs.is_open())
-    {
-      outputCommandError("Unable to append dump to file " + node.getShortPath(), myCommand);
+      commandResult << red("dump flags must be 1..7");
       return;
     }
-    if((args[2] & 0x07) != 0)
-      commandResult << "dumped ";
+    if(argCount == 4 && argStrings[3] != "?")
+    {
+      commandResult << red("browser dialog parameter must be '?'");
+      return;
+    }
+
+    ostringstream path;
+    path << debugger.myOSystem.userDir() << cartName() << "_dbg_";
+    if(execDepth > 0)
+      path << execPrefix;
+    else
+      path << std::hex << std::setw(8) << std::setfill('0')
+           << uInt32(TimerManager::getTicks() / 1000);
+    path << ".dump";
+
+    commandResult << "dumped ";
+
+    stringstream out;
     if((args[2] & 0x01) != 0)
     {
       // dump memory
-      dump(ofs, args[0], args[1]);
+      dump(out, args[0], args[1]);
       commandResult << "bytes from $" << hex << args[0] << " to $" << hex << args[1];
       if((args[2] & 0x06) != 0)
         commandResult << ", ";
@@ -1166,8 +1194,8 @@ void DebuggerParser::executeDump()
     {
       // dump CPU state
       CpuDebug& cpu = debugger.cpuDebug();
-      ofs << "   <PC>PC SP  A  X  Y  -  -    N  V  B  D  I  Z  C  -\n";
-      ofs << "XC: "
+      out << "   <PC>PC SP  A  X  Y  -  -    N  V  B  D  I  Z  C  -\n";
+      out << "XC: "
         << Base::toString(cpu.pc() & 0xff) << " "    // PC lsb
         << Base::toString(cpu.pc() >> 8) << " "    // PC msb
         << Base::toString(cpu.sp()) << " "    // SP
@@ -1192,8 +1220,8 @@ void DebuggerParser::executeDump()
     if((args[2] & 0x04) != 0)
     {
       // dump SWCHx/INPTx state
-      ofs << "   SWA - SWB  - IT  -  -  -   I0 I1 I2 I3 I4 I5 -  -\n";
-      ofs << "XS: "
+      out << "   SWA - SWB  - IT  -  -  -   I0 I1 I2 I3 I4 I5 -  -\n";
+      out << "XS: "
         << Base::toString(debugger.peek(0x280)) << " "    // SWCHA
         << Base::toString(0) << " "    // unused
         << Base::toString(debugger.peek(0x282)) << " "    // SWCHB
@@ -1213,8 +1241,37 @@ void DebuggerParser::executeDump()
         << endl;
       commandResult << "switches and fire buttons";
     }
-    if((args[2] & 0x07) != 0)
-      commandResult << " to file " << node.getShortPath();
+
+    if(argCount == 4)
+    {
+      // FIXME: C++ doesn't currently allow capture of stringstreams
+      //        So we pass a copy of its contents, then re-create the
+      //        stream inside the lambda
+      //        Maybe this will change in a future version
+      const string outStr = out.str();
+      const string resultStr = commandResult.str();
+
+      DebuggerDialog* dlg = debugger.myDialog;
+      BrowserDialog::show(dlg, "Save Dump as", path.str(),
+                          BrowserDialog::Mode::FileSave,
+                          [this, dlg, outStr, resultStr]
+                          (bool OK, const FilesystemNode& node)
+      {
+        if(OK)
+        {
+          stringstream  localOut(outStr);
+          ostringstream localResult(resultStr, std::ios_base::app);
+
+          saveDump(node, localOut, localResult);
+          dlg->prompt().print(localResult.str() + '\n');
+        }
+        dlg->prompt().printPrompt();
+      });
+      // avoid printing a new prompt
+      commandResult.str("_NO_PROMPT");
+    }
+    else
+      saveDump(FilesystemNode(path.str()), out, commandResult);
   }
 }
 
@@ -1228,7 +1285,7 @@ void DebuggerParser::executeExec()
     file += ".script";
   FilesystemNode node(file);
   if (!node.exists())
-    node = FilesystemNode(debugger.myOSystem.defaultSaveDir().getPath() + file);
+    node = FilesystemNode(debugger.myOSystem.userDir().getPath() + file);
 
   if (argCount == 2) {
     execPrefix = argStrings[1];
@@ -1526,10 +1583,8 @@ void DebuggerParser::executeListfunctions()
   const Debugger::FunctionDefMap& functions = debugger.getFunctionDefMap();
 
   if(functions.size() > 0)
-  {
-    for(const auto& iter: functions)
-      commandResult << iter.first << " -> " << iter.second << endl;
-  }
+    for(const auto& [name, cmd]: functions)
+      commandResult << name << " -> " << cmd << endl;
   else
     commandResult << "no user-defined functions";
 }
@@ -1747,9 +1802,13 @@ void DebuggerParser::executeRunTo()
   // Create a progress dialog box to show the progress searching through the
   // disassembly, since this may be a time-consuming operation
   ostringstream buf;
-  buf << "RunTo searching through " << max_iterations << " disassembled instructions";
-  ProgressDialog progress(debugger.baseDialog(), debugger.lfont(), buf.str());
+  ProgressDialog progress(debugger.baseDialog(), debugger.lfont());
+
+  buf << "RunTo searching through " << max_iterations << " disassembled instructions"
+    << progress.ELLIPSIS;
+  progress.setMessage(buf.str());
   progress.setRange(0, max_iterations, 5);
+  progress.open();
 
   bool done = false;
   do {
@@ -1763,8 +1822,8 @@ void DebuggerParser::executeRunTo()
       done = (BSPF::findIgnoreCase(next, argStrings[0]) != string::npos);
     }
     // Update the progress bar
-    progress.setProgress(count);
-  } while(!done && ++count < max_iterations);
+    progress.incProgress();
+  } while(!done && ++count < max_iterations && !progress.isCancelled());
 
   progress.close();
 
@@ -1789,13 +1848,15 @@ void DebuggerParser::executeRunToPc()
 
   uInt32 count = 0;
   bool done = false;
-  constexpr uInt32 max_iterations = 1000000;
   // Create a progress dialog box to show the progress searching through the
   // disassembly, since this may be a time-consuming operation
   ostringstream buf;
-  buf << "RunTo PC searching through " << max_iterations << " instructions";
-  ProgressDialog progress(debugger.baseDialog(), debugger.lfont(), buf.str());
-  progress.setRange(0, max_iterations, 5);
+  ProgressDialog progress(debugger.baseDialog(), debugger.lfont());
+
+  buf << "        RunTo PC running" << progress.ELLIPSIS << "        ";
+  progress.setMessage(buf.str());
+  progress.setRange(0, 100000, 5);
+  progress.open();
 
   do {
     debugger.step(false);
@@ -1803,8 +1864,9 @@ void DebuggerParser::executeRunToPc()
     // Update romlist to point to current PC
     int pcline = cartdbg.addressToLine(debugger.cpuDebug().pc());
     done = (pcline >= 0) && (list[pcline].address == args[0]);
-    progress.setProgress(count);
-  } while(!done && ++count < max_iterations/*list.size()*/);
+    progress.incProgress();
+    ++count;
+  } while(!done && !progress.isCancelled());
   progress.close();
 
   if(done)
@@ -1828,14 +1890,48 @@ void DebuggerParser::executeS()
 // "save"
 void DebuggerParser::executeSave()
 {
-  commandResult << saveScriptFile(argStrings[0]);
+  if(argCount && argStrings[0] == "?")
+  {
+    DebuggerDialog* dlg = debugger.myDialog;
+
+    BrowserDialog::show(dlg, "Save Workbench as",
+                        dlg->instance().userDir().getPath() + cartName() + ".script",
+                        BrowserDialog::Mode::FileSave,
+                        [this, dlg](bool OK, const FilesystemNode& node)
+    {
+      if(OK)
+        dlg->prompt().print(saveScriptFile(node.getPath()) + '\n');
+      dlg->prompt().printPrompt();
+    });
+    // avoid printing a new prompt
+    commandResult.str("_NO_PROMPT");
+  }
+  else
+    commandResult << saveScriptFile(argStrings[0]);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "saveaccess"
 void DebuggerParser::executeSaveAccess()
 {
-  commandResult << debugger.cartDebug().saveAccessFile();
+  if(argCount && argStrings[0] == "?")
+  {
+    DebuggerDialog* dlg = debugger.myDialog;
+
+    BrowserDialog::show(dlg, "Save Access Counters as",
+                        dlg->instance().userDir().getPath() + cartName() + ".csv",
+                        BrowserDialog::Mode::FileSave,
+                        [this, dlg](bool OK, const FilesystemNode& node)
+    {
+      if(OK)
+        dlg->prompt().print(debugger.cartDebug().saveAccessFile(node.getPath()) + '\n');
+      dlg->prompt().printPrompt();
+    });
+    // avoid printing a new prompt
+    commandResult.str("_NO_PROMPT");
+  }
+  else
+    commandResult << debugger.cartDebug().saveAccessFile();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1849,14 +1945,48 @@ void DebuggerParser::executeSaveconfig()
 // "savedis"
 void DebuggerParser::executeSavedisassembly()
 {
-  commandResult << debugger.cartDebug().saveDisassembly();
+  if(argCount && argStrings[0] == "?")
+  {
+    DebuggerDialog* dlg = debugger.myDialog;
+
+    BrowserDialog::show(dlg, "Save Disassembly as",
+                        dlg->instance().userDir().getPath() + cartName() + ".asm",
+                        BrowserDialog::Mode::FileSave,
+                        [this, dlg](bool OK, const FilesystemNode& node)
+    {
+      if(OK)
+        dlg->prompt().print(debugger.cartDebug().saveDisassembly(node.getPath()) + '\n');
+      dlg->prompt().printPrompt();
+    });
+    // avoid printing a new prompt
+    commandResult.str("_NO_PROMPT");
+  }
+  else
+    commandResult << debugger.cartDebug().saveDisassembly();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // "saverom"
 void DebuggerParser::executeSaverom()
 {
-  commandResult << debugger.cartDebug().saveRom();
+  if(argCount && argStrings[0] == "?")
+  {
+    DebuggerDialog* dlg = debugger.myDialog;
+
+    BrowserDialog::show(dlg, "Save ROM as",
+                        dlg->instance().userDir().getPath() + cartName() + ".a26",
+                        BrowserDialog::Mode::FileSave,
+                        [this, dlg](bool OK, const FilesystemNode& node)
+    {
+      if(OK)
+        dlg->prompt().print(debugger.cartDebug().saveRom(node.getPath()) + '\n');
+      dlg->prompt().printPrompt();
+    });
+    // avoid printing a new prompt
+    commandResult.str("_NO_PROMPT");
+  }
+  else
+    commandResult << debugger.cartDebug().saveRom();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1865,13 +1995,35 @@ void DebuggerParser::executeSaveses()
 {
   ostringstream filename;
   auto timeinfo = BSPF::localTime();
-  filename << debugger.myOSystem.defaultSaveDir()
-           << std::put_time(&timeinfo, "session_%F_%H-%M-%S.txt");
-  FilesystemNode file(filename.str());
-  if(debugger.prompt().saveBuffer(file))
-    commandResult << "saved " + file.getShortPath() + " OK";
+  filename << std::put_time(&timeinfo, "session_%F_%H-%M-%S.txt");
+
+  if(argCount && argStrings[0] == "?")
+  {
+    DebuggerDialog* dlg = debugger.myDialog;
+
+    BrowserDialog::show(dlg, "Save Session as",
+                        dlg->instance().userDir().getPath() + filename.str(),
+                        BrowserDialog::Mode::FileSave,
+                        [this, dlg](bool OK, const FilesystemNode& node)
+    {
+      if(OK)
+        dlg->prompt().print(debugger.prompt().saveBuffer(node) + '\n');
+      dlg->prompt().printPrompt();
+    });
+    // avoid printing a new prompt
+    commandResult.str("_NO_PROMPT");
+  }
   else
-    commandResult << "unable to save session";
+  {
+    ostringstream path;
+
+    if(argCount)
+      path << argStrings[0];
+    else
+      path << debugger.myOSystem.userDir() << filename.str();
+
+    commandResult << debugger.prompt().saveBuffer(FilesystemNode(path.str()));
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1953,20 +2105,24 @@ void DebuggerParser::executeStepwhile()
   Expression* expr = YaccParser::getResult();
   int ncycles = 0;
   uInt32 count = 0;
-  constexpr uInt32 max_iterations = 1000000;
 
   // Create a progress dialog box to show the progress searching through the
   // disassembly, since this may be a time-consuming operation
   ostringstream buf;
-  buf << "stepwhile running through " << max_iterations << " disassembled instructions";
-  ProgressDialog progress(debugger.baseDialog(), debugger.lfont(), buf.str());
-  progress.setRange(0, max_iterations, 5);
+  ProgressDialog progress(debugger.baseDialog(), debugger.lfont());
+
+  buf << "stepwhile running through disassembled instructions"
+    << progress.ELLIPSIS;
+  progress.setMessage(buf.str());
+  progress.setRange(0, 100000, 5);
+  progress.open();
 
   do {
     ncycles += debugger.step(false);
 
-    progress.setProgress(count);
-  } while (expr->evaluate() && ++count < max_iterations);
+    progress.incProgress();
+    ++count;
+  } while (expr->evaluate() && !progress.isCancelled());
 
   progress.close();
   commandResult << "executed " << ncycles << " cycles";
@@ -2605,14 +2761,16 @@ std::array<DebuggerParser::Command, 100> DebuggerParser::commands = { {
 
   {
     "dump",
-    "Dump data at address <xx> [to yy] [1: memory; 2: CPU state; 4: input regs]",
+    "Dump data at address <xx> [to yy] [1: memory; 2: CPU state; 4: input regs] [?]",
     "Example:\n"
-    "  dump f000 - dumps 128 bytes @ f000\n"
+    "  dump f000 - dumps 128 bytes from f000\n"
     "  dump f000 f0ff - dumps all bytes from f000 to f0ff\n"
-    "  dump f000 f0ff 7 - dumps all bytes from f000 to f0ff, CPU state and input registers into a file",
+    "  dump f000 f0ff 7 - dumps all bytes from f000 to f0ff,\n"
+    "    CPU state and input registers into a file in user dir,\n"
+    "  dump f000 f0ff 7 ? - same, but with a browser dialog\n",
     true,
     false,
-    { Parameters::ARG_WORD, Parameters::ARG_MULTI_BYTE },
+    { Parameters::ARG_WORD, Parameters::ARG_WORD, Parameters::ARG_BYTE, Parameters::ARG_LABEL },
     std::mem_fn(&DebuggerParser::executeDump)
   },
 
@@ -3032,8 +3190,9 @@ std::array<DebuggerParser::Command, 100> DebuggerParser::commands = { {
 
   {
     "save",
-    "Save breaks, watches, traps and functions to file xx",
-    "Example: save commands.script",
+    "Save breaks, watches, traps and functions to file <xx or ?>",
+    "Example: save commands.script, save ?\n"
+    "NOTE: saves to user dir by default",
     true,
     false,
     { Parameters::ARG_FILE, Parameters::ARG_END_ARGS },
@@ -3042,11 +3201,12 @@ std::array<DebuggerParser::Command, 100> DebuggerParser::commands = { {
 
   {
     "saveaccess",
-    "Save the access counters to CSV file",
-    "Example: saveaccess (no parameters)",
+    "Save the access counters to CSV file [?]",
+    "Example: saveaccess, saveaccess ?\n"
+    "NOTE: saves to user dir by default",
       false,
       false,
-    { Parameters::ARG_END_ARGS },
+    { Parameters::ARG_LABEL, Parameters::ARG_END_ARGS },
       std::mem_fn(&DebuggerParser::executeSaveAccess)
   },
 
@@ -3062,34 +3222,34 @@ std::array<DebuggerParser::Command, 100> DebuggerParser::commands = { {
 
   {
     "savedis",
-    "Save Distella disassembly (with default name)",
-    "Example: savedis\n"
-    "NOTE: saves to default save location",
+    "Save Distella disassembly to file [?]",
+    "Example: savedis, savedis ?\n"
+    "NOTE: saves to user dir by default",
     false,
     false,
-    { Parameters::ARG_END_ARGS },
+    { Parameters::ARG_LABEL, Parameters::ARG_END_ARGS },
     std::mem_fn(&DebuggerParser::executeSavedisassembly)
   },
 
   {
     "saverom",
-    "Save (possibly patched) ROM (with default name)",
-    "Example: saverom\n"
-    "NOTE: saves to default save location",
+    "Save (possibly patched) ROM to file [?]",
+    "Example: saverom, saverom ?\n"
+    "NOTE: saves to user dir by default",
     false,
     false,
-    { Parameters::ARG_END_ARGS },
+    { Parameters::ARG_LABEL, Parameters::ARG_END_ARGS },
     std::mem_fn(&DebuggerParser::executeSaverom)
   },
 
   {
     "saveses",
-    "Save console session (with default name)",
-    "Example: saveses\n"
-    "NOTE: saves to default save location",
+    "Save console session to file [?]",
+    "Example: saveses, saveses ?\n"
+    "NOTE: saves to user dir by default",
     false,
     false,
-    { Parameters::ARG_END_ARGS },
+    { Parameters::ARG_LABEL, Parameters::ARG_END_ARGS },
     std::mem_fn(&DebuggerParser::executeSaveses)
   },
 
