@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -27,12 +27,12 @@ DiStella::DiStella(const CartDebug& dbg, CartDebug::DisassemblyList& list,
                    CartDebug::AddrTypeArray& labels,
                    CartDebug::AddrTypeArray& directives,
                    CartDebug::ReservedEquates& reserved)
-  : myDbg(dbg),
-    myList(list),
-    mySettings(s),
-    myReserved(reserved),
-    myLabels(labels),
-    myDirectives(directives)
+  : myDbg{dbg},
+    myList{list},
+    mySettings{s},
+    myReserved{reserved},
+    myLabels{labels},
+    myDirectives{directives}
 {
   bool resolve_code = mySettings.resolveCode;
   CartDebug::AddressList& debuggerAddresses = info.addressList;
