@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -117,7 +117,7 @@ void Cartridge3EWidget::loadConfig()
     myBankWidgets[1]->setSelectedIndex(bank - myCart.romBankCount(), oldBank != bank);
   }
 
-  CartDebugWidget::loadConfig();
+  CartDebugWidget::loadConfig();  // NOLINT : Intentionally calling grand-parent method
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
