@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,11 +29,11 @@ CartDebugWidget::CartDebugWidget(GuiObject* boss, const GUI::Font& lfont,
                                  int x, int y, int w, int h)
   : Widget(boss, lfont, x, y, w, h),
     CommandSender(boss),
-    _nfont(nfont),
-    myFontWidth(lfont.getMaxCharWidth()),
-    myFontHeight(lfont.getFontHeight()),
-    myLineHeight(lfont.getLineHeight()),
-    myButtonHeight(myLineHeight + 4)
+    _nfont{nfont},
+    myFontWidth{lfont.getMaxCharWidth()},
+    myFontHeight{lfont.getFontHeight()},
+    myLineHeight{lfont.getLineHeight()},
+    myButtonHeight{myLineHeight + 4}
 {
 }
 
