@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,31 +33,38 @@ DataGridOpsWidget::DataGridOpsWidget(GuiObject* boss, const GUI::Font& font,
   xpos = x;  ypos = y;
   _zeroButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                  "0", kDGZeroCmd);
+  _zeroButton->setToolTip("Zero currently selected value");
 
   ypos += bheight + space;
   _invButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                 "Inv", kDGInvertCmd);
+  _invButton->setToolTip("Invert currently selected value");
 
   ypos += bheight + space;
   _incButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                 "++", kDGIncCmd);
+  _incButton->setToolTip("Increase currently selected value.");
 
   ypos += bheight + space;
   _shiftLeftButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                       "<<", kDGShiftLCmd);
+  _shiftLeftButton->setToolTip("Shift currently selected value left");
 
   // Move to next column, skip a row
   xpos = x + bwidth + space;  ypos = y + bheight + space;
   _negButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                 "Neg", kDGNegateCmd);
+  _negButton->setToolTip("Negate currently selected value");
 
   ypos += bheight + space;
   _decButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                 "--", kDGDecCmd);
+  _decButton->setToolTip("Decrease currently selected value");
 
   ypos += bheight + space;
   _shiftRightButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
                                        ">>", kDGShiftRCmd);
+  _shiftRightButton->setToolTip("Shift currently selected value right");
 
   // Calculate real dimensions
   _w = 2 * (bwidth+space);
