@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -50,7 +50,11 @@ void DelayQueueWidget::loadConfig() {
   using Common::Base;
   for (auto&& line : myLines) {
     if (!delayQueueIterator->isValid()) {
-      line = "";
+      if(line != "")
+      {
+        setDirty();
+        line = "";
+      }
       continue;
     }
 
@@ -81,7 +85,11 @@ void DelayQueueWidget::loadConfig() {
         break;
     }
 
-    line = ss.str();
+    if(line != ss.str())
+    {
+      setDirty();
+      line = ss.str();
+    }
     delayQueueIterator->next();
   }
 }
@@ -90,7 +98,6 @@ void DelayQueueWidget::loadConfig() {
 void DelayQueueWidget::drawWidget(bool hilite)
 {
   FBSurface& surface = _boss->dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
 
   int y = _y,
       x = _x,
@@ -102,14 +109,14 @@ void DelayQueueWidget::drawWidget(bool hilite)
   y += 1;
   x += 1;
   w -= 1;
-  surface.fillRect(x, y, w - 1, _h - 2, onTop ? kDlgColor : _bgcolorlo);
+  surface.fillRect(x, y, w - 1, _h - 2, kDlgColor);
 
   y += 2;
   x += 2;
   w -= 3;
 
   for (const auto& line : myLines) {
-    surface.drawString(_font, line, x, y, w, onTop ? _textcolor : kColor);
+    surface.drawString(_font, line, x, y, w, _textcolor);
     y += lineHeight;
   }
 }
