@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -41,12 +41,12 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h)
   : Widget(boss, font, x, y, w - ScrollBarWidget::scrollBarWidth(font), h),
     CommandSender(boss),
-    _historySize(0),
-    _historyIndex(0),
-    _historyLine(0),
-    _makeDirty(false),
-    _firstTime(true),
-    _exitedEarly(false)
+    _historySize{0},
+    _historyIndex{0},
+    _historyLine{0},
+    _makeDirty{false},
+    _firstTime{true},
+    _exitedEarly{false}
 {
   _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS |
            Widget::FLAG_WANTS_TAB | Widget::FLAG_WANTS_RAWDATA;
@@ -81,9 +81,7 @@ void PromptWidget::drawWidget(bool hilite)
 {
 //cerr << "PromptWidget::drawWidget\n";
   ColorId fgcolor, bgcolor;
-
   FBSurface& s = _boss->dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
 
   // Draw text
   int start = _scrollLine - _linesPerPage + 1;
@@ -104,7 +102,7 @@ void PromptWidget::drawWidget(bool hilite)
       else
         fgcolor = ColorId(c >> 8);
 
-      s.drawChar(_font, c & 0x7f, x, y, onTop ? fgcolor : kColor);
+      s.drawChar(_font, c & 0x7f, x, y, fgcolor);
       x += _kConsoleCharWidth;
     }
     y += _kConsoleLineHeight;
@@ -193,6 +191,8 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
           _exitedEarly = true;
           return true;
         }
+        else if(result == "_NO_PROMPT")
+          return true;
         else if(result != "")
           print(result + "\n");
       }
@@ -591,7 +591,7 @@ void PromptWidget::specialKeys(StellaKey key)
       killLine(-1);
       break;
     case KBDK_W:
-      killLastWord();
+      killWord();
       break;
     case KBDK_A:
       textSelectAll();
@@ -666,7 +666,7 @@ void PromptWidget::killLine(int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::killLastWord()
+void PromptWidget::killWord()
 {
   int cnt = 0;
   bool space = true;
@@ -718,7 +718,7 @@ void PromptWidget::textCut()
 #if defined(PSEUDO_CUT_COPY_PASTE)
   string text = getLine();
 
-  instance().eventHandler().cutText(text);
+  instance().eventHandler().copyText(text);
 
   // Remove the current line
   _currentPos = _promptStartPos;
@@ -938,8 +938,6 @@ void PromptWidget::drawCaret()
 {
 //cerr << "PromptWidget::drawCaret()\n";
   FBSurface& s = _boss->dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
-
   int line = _currentPos / _lineWidth;
 
   // Don't draw the cursor if it's not in the current view
@@ -951,7 +949,7 @@ void PromptWidget::drawCaret()
   int y = _y + displayLine * _kConsoleLineHeight;
 
   char c = buffer(_currentPos); //FIXME: int to char??
-  s.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, onTop ? kTextColor : kColor);
+  s.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, kTextColor);
   s.drawChar(_font, c, x, y + 2, kBGColor);
 }
 
@@ -972,7 +970,7 @@ void PromptWidget::scrollToCurrent()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PromptWidget::saveBuffer(const FilesystemNode& file)
+string PromptWidget::saveBuffer(const FilesystemNode& file)
 {
   stringstream out;
   for(int start = 0; start < _promptStartPos; start += _lineWidth)
@@ -992,8 +990,13 @@ bool PromptWidget::saveBuffer(const FilesystemNode& file)
     out << endl;
   }
 
-  try { return file.write(out) > 0; }
-  catch(...) { return false; }
+  try {
+    if(file.write(out) > 0)
+      return "saved " + file.getShortPath() + " OK";
+  }
+  catch(...) { }
+
+  return "unable to save session";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
