@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,7 +25,7 @@
 RiotRamWidget::RiotRamWidget(GuiObject* boss, const GUI::Font& lfont,
         const GUI::Font& nfont, int x, int y, int w)
   : RamWidget(boss, lfont, nfont, x, y, w, 0, 128, 8, 128),
-    myDbg(instance().debugger().cartDebug())
+    myDbg{instance().debugger().cartDebug()}
 {
 }
 
