@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -75,6 +75,7 @@ void RomWidget::loadConfig()
 
   // Update romlist to point to current PC (if it has changed)
   int pcline = cart.addressToLine(dbg.cpuDebug().pc());
+
   if(pcline >= 0 && pcline != myRomList->getHighlighted())
     myRomList->setHighlighted(pcline);
 
@@ -90,10 +91,6 @@ void RomWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
     case RomListWidget::kBPointChangedCmd:
       // 'data' is the line in the disassemblylist to be accessed
       toggleBreak(data);
-      // Refresh the romlist, since the breakpoint may not have
-      // actually changed
-      myRomList->setDirty();
-      myRomList->draw();
       break;
 
     case RomListWidget::kRomChangedCmd:
@@ -199,7 +196,7 @@ void RomWidget::runtoPC(int disasm_line)
     ostringstream command;
     command << "runtopc #" << address;
     string msg = instance().debugger().run(command.str());
-    instance().frameBuffer().showMessage(msg);
+    instance().frameBuffer().showTextMessage(msg);
   }
 }
 
