@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -563,7 +563,7 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& lfont,
   new StaticTextWidget(boss, lfont, xpos, ypos+2, 2*fontWidth, fontHeight,
                        "PF", TextAlign::Left);
   xpos += 2*fontWidth + 5;
-  myPF[0] = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 4, 1);
+  myPF[0] = new TogglePixelWidget(boss, nfont, xpos, ypos+1, 4, 1, 4);
   myPF[0]->setTarget(this);
   myPF[0]->setID(kPF0ID);
   addFocusWidget(myPF[0]);
@@ -1043,8 +1043,8 @@ void TiaWidget::loadConfig()
     myGRP0Old->setColor(kBGColorLo);
     myGRP0Old->setCrossed(true);
   }
-  myGRP0->setIntState(state.gr[TiaState::P0], false);
-  myGRP0Old->setIntState(state.gr[TiaState::P0+2], false);
+  myGRP0->setIntState(state.gr[TiaState::P0], state.ref[TiaState::P0]);
+  myGRP0Old->setIntState(state.gr[TiaState::P0+2], state.ref[TiaState::P0]);
 
   // posP0
   myPosP0->setList(0, state.pos[TiaState::P0],
@@ -1079,8 +1079,8 @@ void TiaWidget::loadConfig()
     myGRP1Old->setColor(kBGColorLo);
     myGRP1Old->setCrossed(true);
   }
-  myGRP1->setIntState(state.gr[TiaState::P1], false);
-  myGRP1Old->setIntState(state.gr[TiaState::P1+2], false);
+  myGRP1->setIntState(state.gr[TiaState::P1], state.ref[TiaState::P1]);
+  myGRP1Old->setIntState(state.gr[TiaState::P1+2], state.ref[TiaState::P1]);
 
   // posP1
   myPosP1->setList(0, state.pos[TiaState::P1],
