@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,8 +25,10 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ToggleBitWidget::ToggleBitWidget(GuiObject* boss, const GUI::Font& font,
-                                 int x, int y, int cols, int rows, int colchars)
-  : ToggleWidget(boss, font, x, y, cols, rows, 1)
+                                 int x, int y, int cols, int rows, int colchars,
+                                 const StringList& labels)
+  : ToggleWidget(boss, font, x, y, cols, rows),
+    _labelList{labels}
 {
   _rowHeight = font.getLineHeight();
   _colWidth  = colchars * font.getMaxCharWidth() + 8;
@@ -47,6 +49,13 @@ ToggleBitWidget::ToggleBitWidget(GuiObject* boss, const GUI::Font& font,
   _h = _rowHeight * rows + 1;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ToggleBitWidget::ToggleBitWidget(GuiObject* boss, const GUI::Font& font,
+                                 int x, int y, int cols, int rows, int colchars)
+  : ToggleBitWidget(boss, font, x, y, cols, rows, colchars, StringList())
+{
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ToggleBitWidget::setList(const StringList& off, const StringList& on)
 {
@@ -61,12 +70,35 @@ void ToggleBitWidget::setList(const StringList& off, const StringList& on)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ToggleBitWidget::setState(const BoolArray& state, const BoolArray& changed)
 {
+  if(!std::equal(_changedList.begin(), _changedList.end(),
+     changed.begin(), changed.end()))
+    setDirty();
+
   _stateList.clear();
   _stateList = state;
   _changedList.clear();
   _changedList = changed;
+}
 
-  setDirty();
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string ToggleBitWidget::getToolTip(const Common::Point& pos) const
+{
+  const Common::Point& idx = getToolTipIndex(pos);
+
+  if(idx.y < 0)
+    return EmptyString;
+
+  string tip = ToggleWidget::getToolTip(pos);
+
+  if(idx.x < static_cast<int>(_labelList.size()))
+  {
+    const string label = _labelList[idx.x];
+
+    if(!label.empty())
+      return tip + "\n" + label;
+  }
+  return tip;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -74,7 +106,6 @@ void ToggleBitWidget::drawWidget(bool hilite)
 {
 //cerr << "ToggleBitWidget::drawWidget\n";
   FBSurface& s = dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
   int row, col;
   string buffer;
 
@@ -116,18 +147,16 @@ void ToggleBitWidget::drawWidget(bool hilite)
         // Highlight changes
         if(_changedList[pos])
         {
-          s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1,
-                     onTop ? kDbgChangedColor : _bgcolorlo);
-          s.drawString(_font, buffer, x, y, _colWidth, onTop ? kDbgChangedTextColor : kColor);
+          s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
+          s.drawString(_font, buffer, x, y, _colWidth, kDbgChangedTextColor);
         }
         else
-          s.drawString(_font, buffer, x, y, _colWidth,
-                       onTop ? textColor : kColor);
+          s.drawString(_font, buffer, x, y, _colWidth, textColor);
       }
       else
       {
-        s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, onTop ? kBGColorHi : kDlgColor);
-        s.drawString(_font, buffer, x, y, _colWidth, onTop ? kTextColor : kColor);
+        s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kBGColorHi);
+        s.drawString(_font, buffer, x, y, _colWidth, kTextColor);
       }
     }
   }
