@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -16,44 +16,26 @@
 //============================================================================
 
 #include "OSystem.hxx"
+#include "Base.hxx"
 #include "StellaKeys.hxx"
 #include "Widget.hxx"
+#include "Dialog.hxx"
+#include "ToolTip.hxx"
 #include "ToggleWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ToggleWidget::ToggleWidget(GuiObject* boss, const GUI::Font& font,
-                           int x, int y, int cols, int rows,
-                           int clicksToChange)
+                           int x, int y, int cols, int rows, int shiftBits)
   : Widget(boss, font, x, y, 16, 16),
     CommandSender(boss),
-    _rows(rows),
-    _cols(cols),
-    _currentRow(0),
-    _currentCol(0),
-    _rowHeight(0),
-    _colWidth(0),
-    _selectedItem(0),
-    _clicksToChange(clicksToChange),
-    _editable(true)
+    _rows{rows},
+    _cols{cols},
+    _shiftBits{shiftBits}
 {
   _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS |
            Widget::FLAG_WANTS_RAWDATA;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ToggleWidget::handleMouseEntered()
-{
-  setFlags(Widget::FLAG_HILITED);
-  setDirty();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ToggleWidget::handleMouseLeft()
-{
-  clearFlags(Widget::FLAG_HILITED);
-  setDirty();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ToggleWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
@@ -71,6 +53,7 @@ void ToggleWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
     _selectedItem = newSelectedItem;
     _currentRow = _selectedItem / _cols;
     _currentCol = _selectedItem - (_currentRow * _cols);
+    dialog().tooltip().hide();
     setDirty();
   }
 }
@@ -83,7 +66,7 @@ void ToggleWidget::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 
   // If this was a double click and the mouse is still over the selected item,
   // send the double click command
-  if (clickCount == _clicksToChange && (_selectedItem == findItem(x, y)))
+  if (clickCount == 1 && (_selectedItem == findItem(x, y)))
   {
     _stateList[_selectedItem] = !_stateList[_selectedItem];
     _changedList[_selectedItem] = !_changedList[_selectedItem];
@@ -202,6 +185,7 @@ bool ToggleWidget::handleKeyDown(StellaKey key, StellaMod mod)
       _stateList[_selectedItem] = !_stateList[_selectedItem];
       _changedList[_selectedItem] = !_changedList[_selectedItem];
       sendCommand(ToggleWidget::kItemDataChangedCmd, _selectedItem, _id);
+      dialog().tooltip().hide();
     }
 
     setDirty();
@@ -223,3 +207,56 @@ void ToggleWidget::handleCommand(CommandSender* sender, int cmd,
     }
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Common::Point ToggleWidget::getToolTipIndex(const Common::Point& pos) const
+{
+  const int col = (pos.x - getAbsX()) / _colWidth;
+  const int row = (pos.y - getAbsY()) / _rowHeight;
+
+  if(row >= 0 && row < _rows && col >= 0 && col < _cols)
+    return Common::Point(col, row);
+  else
+    return Common::Point(-1, -1);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string ToggleWidget::getToolTip(const Common::Point& pos) const
+{
+  const int idx = getToolTipIndex(pos).y * _cols;
+
+  if(idx < 0)
+    return EmptyString;
+
+  Int32 val = 0;
+  ostringstream buf;
+
+  if(_swapBits)
+    for(int col = _cols - 1; col >= 0; --col)
+    {
+      val <<= 1;
+      val += _stateList[idx + col];
+    }
+  else
+    for(int col = 0; col < _cols; ++col)
+    {
+      val <<= 1;
+      val += _stateList[idx + col];
+    }
+  val <<= _shiftBits;
+
+  buf << _toolTipText
+    << "$" << Common::Base::toString(val, Common::Base::Fmt::_16)
+    << " = #" << val;
+  if(val < 0x100)
+    buf << " = %" << Common::Base::toString(val, Common::Base::Fmt::_2);
+
+  return buf.str();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool ToggleWidget::changedToolTip(const Common::Point& oldPos,
+                                  const Common::Point& newPos) const
+{
+  return getToolTipIndex(oldPos) != getToolTipIndex(newPos);
+}
