@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -124,8 +124,8 @@ Bankswitch::BSList = {{
   { "DPC"     , "DPC (Pitfall II)"            },
   { "DPC+"    , "DPC+ (Enhanced DPC)"         },
   { "E0"      , "E0 (8K Parker Bros)"         },
-  { "E7"      , "E7 (16K M-network)"          },
-  { "E78K"    , "E78K (8K M-network)"         },
+  { "E7"      , "E7 (16K M Network)"          },
+  { "E78K"    , "E78K (8K M Network)"         },
   { "EF"      , "EF (64K H. Runner)"          },
   { "EFSC"    , "EFSC (64K H. Runner + RAM)"  },
   { "F0"      , "F0 (Dynacom Megaboy)"        },
