@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,8 +29,8 @@
 CartridgeDPCPlus::CartridgeDPCPlus(const ByteBuffer& image, size_t size,
                                    const string& md5, const Settings& settings)
   : Cartridge(settings, md5),
-    myImage(make_unique<uInt8[]>(32_KB)),
-    mySize(std::min(size, 32_KB))
+    myImage{make_unique<uInt8[]>(32_KB)},
+    mySize{std::min(size, 32_KB)}
 {
   // Image is always 32K, but in the case of ROM < 32K, the image is
   // copied to the end of the buffer
@@ -650,6 +650,15 @@ const ByteBuffer& CartridgeDPCPlus::getImage(size_t& size) const
   return myImage;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8 CartridgeDPCPlus::internalRamGetValue(uInt16 addr) const
+{
+  if(addr < internalRamSize())
+    return myDPCRAM[addr];
+  else
+    return 0;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeDPCPlus::save(Serializer& out) const
 {
