@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -23,8 +23,8 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CompuMate::CompuMate(const Console& console, const Event& event,
                      const System& system)
-  : myConsole(console),
-    myEvent(event)
+  : myConsole{console},
+    myEvent{event}
 {
   // These controller pointers will be retrieved by the Console, which will
   // also take ownership of them
