@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -116,11 +116,11 @@ namespace {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Console::Console(OSystem& osystem, unique_ptr<Cartridge>& cart,
                  const Properties& props, AudioSettings& audioSettings)
-  : myOSystem(osystem),
-    myEvent(osystem.eventHandler().event()),
-    myProperties(props),
-    myCart(std::move(cart)),
-    myAudioSettings(audioSettings)
+  : myOSystem{osystem},
+    myEvent{osystem.eventHandler().event()},
+    myProperties{props},
+    myCart{std::move(cart)},
+    myAudioSettings{audioSettings}
 {
   // Create subsystems for the console
   my6502 = make_unique<M6502>(myOSystem.settings());
@@ -316,13 +316,13 @@ string Console::formatFromFilename() const
     { R"([ _\-(\[<]+NTSC[ _-]?50)",     "NTSC50"  },
     { R"([ _\-(\[<]+PAL[ _-]?60)",      "PAL60"   },
     { R"([ _\-(\[<]+SECAM[ _-]?60)",    "SECAM60" },
-    { R"([ _\-(\[<]+NTSC[ _\-)\]>]?)",  "NTSC"    },
-    { R"([ _\-(\[<]+PAL[ _\-)\]>]?)",   "PAL"     },
-    { R"([ _\-(\[<]+SECAM[ _\-)\]>]?)", "SECAM"   }
+    { R"([ _\-(\[<]+NTSC[ _\-)\]>.])",  "NTSC"    },
+    { R"([ _\-(\[<]+PAL[ _\-)\]>.])",   "PAL"     },
+    { R"([ _\-(\[<]+SECAM[ _\-)\]>.])", "SECAM"   }
   }};
 
-  // Get filename *without* extension, and search using regex's above
-  const string& filename = myOSystem.romFile().getNameWithExt("");
+  // Get filename, and search using regex's above
+  const string& filename = myOSystem.romFile().getName();
   for(size_t i = 0; i < Pattern.size(); ++i)
   {
     try
@@ -476,7 +476,7 @@ void Console::setFormat(uInt32 format, bool force)
   initializeAudio(); // ensure that audio synthesis is set up to match emulation rate
   myOSystem.resetFps(); // Reset FPS measurement
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 
   // Let the other devices know about the console change
   mySystem->consoleChanged(myConsoleTiming);
@@ -493,10 +493,10 @@ void Console::toggleColorLoss(bool toggle)
 
     string message = string("PAL color-loss ") +
                      (colorloss ? "enabled" : "disabled");
-    myOSystem.frameBuffer().showMessage(message);
+    myOSystem.frameBuffer().showTextMessage(message);
   }
   else
-    myOSystem.frameBuffer().showMessage(
+    myOSystem.frameBuffer().showTextMessage(
       "PAL color-loss not available in non PAL modes");
 }
 
@@ -521,7 +521,7 @@ void Console::toggleInter(bool toggle)
   ostringstream ss;
 
   ss << "Interpolation " << (enabled ? "enabled" : "disabled");
-  myOSystem.frameBuffer().showMessage(ss.str());
+  myOSystem.frameBuffer().showTextMessage(ss.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -539,7 +539,7 @@ void Console::toggleTurbo()
 
   ostringstream ss;
   ss << "Turbo mode " << (!enabled ? "enabled" : "disabled");
-  myOSystem.frameBuffer().showMessage(ss.str());
+  myOSystem.frameBuffer().showTextMessage(ss.str());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -564,7 +564,7 @@ void Console::changeSpeed(int direction)
   ostringstream val;
 
   val << formatSpeed(speed) << "%";
-  myOSystem.frameBuffer().showMessage("Emulation speed", val.str(), speed, MIN_SPEED, MAX_SPEED);
+  myOSystem.frameBuffer().showGaugeMessage("Emulation speed", val.str(), speed, MIN_SPEED, MAX_SPEED);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -574,13 +574,13 @@ void Console::togglePhosphor()
   {
     myProperties.set(PropType::Display_Phosphor, "NO");
     myOSystem.frameBuffer().tiaSurface().enablePhosphor(false);
-    myOSystem.frameBuffer().showMessage("Phosphor effect disabled");
+    myOSystem.frameBuffer().showTextMessage("Phosphor effect disabled");
   }
   else
   {
     myProperties.set(PropType::Display_Phosphor, "YES");
     myOSystem.frameBuffer().tiaSurface().enablePhosphor(true);
-    myOSystem.frameBuffer().showMessage("Phosphor effect enabled");
+    myOSystem.frameBuffer().showTextMessage("Phosphor effect enabled");
   }
 }
 
@@ -605,7 +605,7 @@ void Console::changePhosphor(int direction)
     val.str("");
     val << "Off";
   }
-  myOSystem.frameBuffer().showMessage("Phosphor blend", val.str(), blend);
+  myOSystem.frameBuffer().showGaugeMessage("Phosphor blend", val.str(), blend);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -699,7 +699,7 @@ void Console::changeVerticalCenter(int direction)
   if (vcenter != myTIA->vcenter()) myTIA->setVcenter(vcenter);
 
   val << (vcenter ? vcenter > 0 ? "+" : "" : " ") << vcenter << "px";
-  myOSystem.frameBuffer().showMessage("V-Center", val.str(), vcenter,
+  myOSystem.frameBuffer().showGaugeMessage("V-Center", val.str(), vcenter,
                                       myTIA->minVcenter(), myTIA->maxVcenter());
 }
 
@@ -729,7 +729,7 @@ void Console::changeVSizeAdjust(int direction)
 
   val << (newAdjustVSize ? newAdjustVSize > 0 ? "+" : "" : " ")
       << newAdjustVSize << "%";
-  myOSystem.frameBuffer().showMessage("V-Size", val.str(), newAdjustVSize, -5, 5);
+  myOSystem.frameBuffer().showGaugeMessage("V-Size", val.str(), newAdjustVSize, -5, 5);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -746,7 +746,7 @@ void Console::toggleCorrectAspectRatio(bool toggle)
   const string& message = string("Correct aspect ratio ") +
       (enabled ? "enabled" : "disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -858,6 +858,40 @@ void Console::setControllers(const string& romMd5)
   myOSystem.eventHandler().setMouseControllerMode(myOSystem.settings().getString("usemouse"));
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changeLeftController(int direction)
+{
+  int type = int(Controller::getType(myProperties.get(PropType::Controller_Left)));
+  if(!type)
+    type = int(Controller::getType(leftController().name()));
+  type = BSPF::clampw(type + direction,
+                      1, int(Controller::Type::LastType) - 1);
+
+  myProperties.set(PropType::Controller_Left, Controller::getPropName(Controller::Type(type)));
+  setControllers(myProperties.get(PropType::Cart_MD5));
+
+  ostringstream msg;
+  msg << "Left controller " << Controller::getName(Controller::Type(type));
+  myOSystem.frameBuffer().showTextMessage(msg.str());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changeRightController(int direction)
+{
+  int type = int(Controller::getType(myProperties.get(PropType::Controller_Right)));
+  if(!type)
+    type = int(Controller::getType(rightController().name()));
+  type = BSPF::clampw(type + direction,
+                      1, int(Controller::Type::LastType) - 1);
+
+  myProperties.set(PropType::Controller_Right, Controller::getPropName(Controller::Type(type)));
+  setControllers(myProperties.get(PropType::Cart_MD5));
+
+  ostringstream msg;
+  msg << "Right controller " << Controller::getName(Controller::Type(type));
+  myOSystem.frameBuffer().showTextMessage(msg.str());
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
                                                   const Controller::Jack port, const string& romMd5)
@@ -920,7 +954,7 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
       Controller::onMessageCallback callback = [&os = myOSystem](const string& msg) {
         bool devSettings = os.settings().getBool("dev.settings");
         if(os.settings().getBool(devSettings ? "dev.eepromaccess" : "plr.eepromaccess"))
-          os.frameBuffer().showMessage(msg);
+          os.frameBuffer().showTextMessage(msg);
       };
       controller = make_unique<AtariVox>(port, myEvent, *mySystem,
           myOSystem.settings().getString("avoxport"), nvramfile, callback);
@@ -933,7 +967,7 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
       Controller::onMessageCallback callback = [&os = myOSystem](const string& msg) {
         bool devSettings = os.settings().getBool("dev.settings");
         if(os.settings().getBool(devSettings ? "dev.eepromaccess" : "plr.eepromaccess"))
-          os.frameBuffer().showMessage(msg);
+          os.frameBuffer().showTextMessage(msg);
       };
       controller = make_unique<SaveKey>(port, myEvent, *mySystem, nvramfile, callback);
       break;
@@ -967,6 +1001,99 @@ unique_ptr<Controller> Console::getControllerPort(const Controller::Type type,
   return controller;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleSwapPorts(bool toggle)
+{
+  bool swapped = myProperties.get(PropType::Console_SwapPorts) == "YES";
+
+  if(toggle)
+  {
+    swapped = !swapped;
+    myProperties.set(PropType::Console_SwapPorts, (swapped ? "YES" : "NO"));
+    //myOSystem.propSet().insert(myProperties);
+    setControllers(myProperties.get(PropType::Cart_MD5));
+  }
+
+  ostringstream msg;
+  msg << "Swap ports " << (swapped ? "enabled" : "disabled");
+  myOSystem.frameBuffer().showTextMessage(msg.str());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::toggleSwapPaddles(bool toggle)
+{
+  bool swapped = myProperties.get(PropType::Controller_SwapPaddles) == "YES";
+
+  if(toggle)
+  {
+    swapped = !swapped;
+    myProperties.set(PropType::Controller_SwapPaddles, (swapped ? "YES" : "NO"));
+    //myOSystem.propSet().insert(myProperties);
+    setControllers(myProperties.get(PropType::Cart_MD5));
+  }
+
+  ostringstream msg;
+  msg << "Swap paddles " << (swapped ? "enabled" : "disabled");
+  myOSystem.frameBuffer().showTextMessage(msg.str());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changePaddleCenterX(int direction)
+{
+  int center =
+    BSPF::clamp(BSPF::stringToInt(myProperties.get(PropType::Controller_PaddlesXCenter)) + direction,
+                Paddles::MIN_ANALOG_CENTER, Paddles::MAX_ANALOG_CENTER);
+  myProperties.set(PropType::Controller_PaddlesXCenter, std::to_string(center));
+  Paddles::setAnalogXCenter(center);
+
+  ostringstream val;
+  val << (center ? center > 0 ? "+" : "" : " ") << center * 5 << "px";
+  myOSystem.frameBuffer().showGaugeMessage("Paddles x-center ", val.str(), center,
+                                           Paddles::MIN_ANALOG_CENTER, Paddles::MAX_ANALOG_CENTER);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changePaddleCenterY(int direction)
+{
+  int center =
+    BSPF::clamp(BSPF::stringToInt(myProperties.get(PropType::Controller_PaddlesYCenter)) + direction,
+                Paddles::MIN_ANALOG_CENTER, Paddles::MAX_ANALOG_CENTER);
+  myProperties.set(PropType::Controller_PaddlesYCenter, std::to_string(center));
+  Paddles::setAnalogYCenter(center);
+
+  ostringstream val;
+  val << (center ? center > 0 ? "+" : "" : " ") << center * 5 << "px";
+  myOSystem.frameBuffer().showGaugeMessage("Paddles y-center ", val.str(), center,
+                                           Paddles::MIN_ANALOG_CENTER, Paddles::MAX_ANALOG_CENTER);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Console::changePaddleAxesRange(int direction)
+{
+  istringstream m_axis(myProperties.get(PropType::Controller_MouseAxis));
+  string mode = "AUTO";
+  int range;
+
+  m_axis >> mode;
+  if(!(m_axis >> range))
+    range = Paddles::MAX_MOUSE_RANGE;
+
+  range = BSPF::clamp(range + direction,
+                      Paddles::MIN_MOUSE_RANGE, Paddles::MAX_MOUSE_RANGE);
+
+  ostringstream control;
+  control << mode;
+  if(range != 100)
+    control << " " << std::to_string(range);
+  myProperties.set(PropType::Controller_MouseAxis, control.str());
+
+  Paddles::setDigitalPaddleRange(range);
+
+  ostringstream val;
+  val << range << "%";
+  myOSystem.frameBuffer().showGaugeMessage("Mouse axes range", val.str(), range);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Console::changeAutoFireRate(int direction)
 {
@@ -987,7 +1114,7 @@ void Console::changeAutoFireRate(int direction)
   else
     val << "Off";
 
-  myOSystem.frameBuffer().showMessage("Autofire rate", val.str(), rate, 0, isNTSC ? 30 : 25);
+  myOSystem.frameBuffer().showGaugeMessage("Autofire rate", val.str(), rate, 0, isNTSC ? 30 : 25);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1012,7 +1139,7 @@ void Console::toggleTIABit(TIABit bit, const string& bitname, bool show, bool to
   bool result = myTIA->toggleBit(bit, toggle ? 2 : 3);
   const string message = bitname + (result ? " enabled" : " disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1021,7 +1148,7 @@ void Console::toggleBits(bool toggle) const
   bool enabled = myTIA->toggleBits(toggle);
   const string message = string("TIA bits ") + (enabled ? "enabled" : "disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1030,7 +1157,7 @@ void Console::toggleTIACollision(TIABit bit, const string& bitname, bool show, b
   bool result = myTIA->toggleCollision(bit, toggle ? 2 : 3);
   const string message = bitname + (result ? " collision enabled" : " collision disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1039,7 +1166,7 @@ void Console::toggleCollisions(bool toggle) const
   bool enabled = myTIA->toggleCollisions(toggle);
   const string message = string("TIA collisions ") + (enabled ? "enabled" : "disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1048,7 +1175,7 @@ void Console::toggleFixedColors(bool toggle) const
   bool enabled = toggle ? myTIA->toggleFixedColors() : myTIA->usingFixedColors();
   const string message = string("Fixed debug colors ") + (enabled ? "enabled" : "disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1057,7 +1184,7 @@ void Console::toggleJitter(bool toggle) const
   bool enabled = myTIA->toggleJitter(toggle ? 2 : 3);
   const string message = string("TV scanline jitter ") + (enabled ? "enabled" : "disabled");
 
-  myOSystem.frameBuffer().showMessage(message);
+  myOSystem.frameBuffer().showTextMessage(message);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
