@@ -15,9 +15,6 @@
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include "Event.hxx"
-#include "System.hxx"
-
 #include "Driving.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -38,8 +35,8 @@ Driving::Driving(Jack jack, const Event& event, const System& system, bool altma
       myCWEvent   = Event::JoystickTwoRight;
       myFireEvent = Event::JoystickTwoFire;
     }
-    myXAxisValue = Event::PaddleZeroAnalog;
-    myYAxisValue = Event::PaddleOneAnalog;
+    myXAxisValue = Event::SALeftAxis0Value; // joystick input
+    myYAxisValue = Event::SALeftAxis1Value; // driving controller input
   }
   else
   {
@@ -55,8 +52,8 @@ Driving::Driving(Jack jack, const Event& event, const System& system, bool altma
       myCWEvent   = Event::JoystickThreeRight;
       myFireEvent = Event::JoystickThreeFire;
     }
-    myXAxisValue = Event::PaddleTwoAnalog;
-    myYAxisValue = Event::PaddleThreeAnalog;
+    myXAxisValue = Event::SARightAxis0Value; // joystick input
+    myYAxisValue = Event::SARightAxis1Value; // driving controller input
   }
 
   // Digital pins 3 and 4 are not connected
@@ -67,70 +64,11 @@ Driving::Driving(Jack jack, const Event& event, const System& system, bool altma
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Driving::update()
 {
-  // Digital events (from keyboard or joystick hats & buttons)
-  bool firePressed = myEvent.get(myFireEvent) != 0;
+  updateButtons();
 
-  int d_axis = myEvent.get(myXAxisValue);
-  if(myEvent.get(myCCWEvent) != 0 || d_axis < -16384)     --myCounter;
-  else if(myEvent.get(myCWEvent) != 0 || d_axis > 16384)  ++myCounter;
-
-  // Mouse motion and button events
-  if(myControlID > -1)
-  {
-    int m_axis = myEvent.get(Event::MouseAxisXMove);
-    if(m_axis < -2)     --myCounter;
-    else if(m_axis > 2) ++myCounter;
-    firePressed = firePressed
-      || myEvent.get(Event::MouseButtonLeftValue)
-      || myEvent.get(Event::MouseButtonRightValue);
-  }
-  else
-  {
-    // Test for 'untied' mouse axis mode, where each axis is potentially
-    // mapped to a separate driving controller
-    if(myControlIDX > -1)
-    {
-      int m_axis = myEvent.get(Event::MouseAxisXMove);
-      if(m_axis < -2)     --myCounter;
-      else if(m_axis > 2) ++myCounter;
-      firePressed = firePressed
-        || myEvent.get(Event::MouseButtonLeftValue);
-    }
-    if(myControlIDY > -1)
-    {
-      int m_axis = myEvent.get(Event::MouseAxisYMove);
-      if(m_axis < -2)     --myCounter;
-      else if(m_axis > 2) ++myCounter;
-      firePressed = firePressed
-        || myEvent.get(Event::MouseButtonRightValue);
-    }
-  }
-  setPin(DigitalPin::Six, !getAutoFireState(firePressed));
-
-  // Only consider the lower-most bits (corresponding to pins 1 & 2)
-  myGrayIndex = Int32(myCounter * SENSITIVITY / 4.0F) & 0b11;
-
-  // Stelladaptor is the only controller that should set this
-  int yaxis = myEvent.get(myYAxisValue);
-
-  // Only overwrite gray code when Stelladaptor input has changed
-  // (that means real changes, not just analog signal jitter)
-  if((yaxis < (myLastYaxis - 1024)) || (yaxis > (myLastYaxis + 1024)))
-  {
-    myLastYaxis = yaxis;
-    if(yaxis <= -16384-4096)
-      myGrayIndex = 3; // up
-    else if(yaxis > 16384+4096)
-      myGrayIndex = 1; // down
-    else if(yaxis >= 16384-4096)
-      myGrayIndex = 2; // up + down
-    else /* if(yaxis < 16384-4096) */
-      myGrayIndex = 0; // no movement
-
-    // Make sure direct gray codes from Stelladaptor stay in sync with
-    // simulated gray codes generated by PC keyboard or PC joystick
-    myCounter = myGrayIndex / SENSITIVITY * 4.0F;
-  }
+  updateDigitalAxes();
+  updateMouseAxes();
+  updateStelladaptorAxes();
 
   // Gray codes for rotation
   static constexpr std::array<uInt8, 4> graytable = { 0x03, 0x01, 0x00, 0x02 };
@@ -141,6 +79,113 @@ void Driving::update()
   setPin(DigitalPin::Two, (gray & 0x2) != 0);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Driving::updateButtons()
+{
+  bool firePressed = myEvent.get(myFireEvent) != 0;
+
+  // The joystick uses both mouse buttons for the single joystick button
+  updateMouseButtons(firePressed);
+
+  setPin(DigitalPin::Six, !getAutoFireState(firePressed));
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Driving::updateMouseButtons(bool& firePressed)
+{
+  if(myControlID > -1)
+    firePressed |= (myEvent.get(Event::MouseButtonLeftValue) != 0
+      || myEvent.get(Event::MouseButtonRightValue) != 0);
+  else
+  {
+    // Test for 'untied' mouse axis mode, where each axis is potentially
+    // mapped to a separate driving controller
+    if(myControlIDX > -1)
+      firePressed |= (myEvent.get(Event::MouseButtonLeftValue) != 0);
+    if(myControlIDY > -1)
+      firePressed |= (myEvent.get(Event::MouseButtonRightValue) != 0);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Driving::updateDigitalAxes()
+{
+  // Digital events (from keyboard or joystick hats & buttons)
+  int d_axis = myEvent.get(myXAxisValue);
+
+  if(myEvent.get(myCCWEvent) != 0 || d_axis < -16384)
+    --myCounter;
+  else if(myEvent.get(myCWEvent) != 0 || d_axis > 16384)
+    ++myCounter;
+
+  // Only consider the lower-most bits (corresponding to pins 1 & 2)
+  myGrayIndex = Int32(myCounter * SENSITIVITY / 4.0F) & 0b11;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Driving::updateMouseAxes()
+{
+  #define MJ_Threshold 2
+  // Mouse motion and button events
+  if(myControlID > -1)
+  {
+    int m_axis = myEvent.get(Event::MouseAxisXMove);
+    if(m_axis < -MJ_Threshold)
+      --myCounter;
+    else if(m_axis > MJ_Threshold)
+      ++myCounter;
+  }
+  else
+  {
+    // Test for 'untied' mouse axis mode, where each axis is potentially
+    // mapped to a separate driving controller
+    if(myControlIDX > -1)
+    {
+      int m_axis = myEvent.get(Event::MouseAxisXMove);
+      if(m_axis < -MJ_Threshold)
+        --myCounter;
+      else if(m_axis > MJ_Threshold)
+        ++myCounter;
+    }
+    if(myControlIDY > -1)
+    {
+      int m_axis = myEvent.get(Event::MouseAxisYMove);
+      if(m_axis < -MJ_Threshold)
+        --myCounter;
+      else if(m_axis > MJ_Threshold)
+        ++myCounter;
+    }
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Driving::updateStelladaptorAxes()
+{
+  #define SA_Threshold 4096
+  // Stelladaptor is the only controller that should set this
+  int yaxis = myEvent.get(myYAxisValue);
+
+  // Only overwrite gray code when Stelladaptor input has changed
+  // (that means real changes, not just analog signal jitter)
+  if((yaxis < (myLastYaxis - 1024)) || (yaxis > (myLastYaxis + 1024)))
+  {
+    myLastYaxis = yaxis;
+    if(yaxis <= -16384 - SA_Threshold)
+      myGrayIndex = 3; // up
+    else if(yaxis > 16384 + SA_Threshold)
+      myGrayIndex = 1; // down
+    else if(yaxis >= 16384 - SA_Threshold)
+      myGrayIndex = 2; // up + down
+    else /* if(yaxis < 16384 - SA_Threshold) */
+      myGrayIndex = 0; // no movement
+
+    // Make sure direct gray codes from Stelladaptor stay in sync with
+    // simulated gray codes generated by PC keyboard or PC joystick
+    // Must be rounded into the middle of the myCounter interval!
+    myCounter = (myGrayIndex + 0.5F) * 4.0F / SENSITIVITY;
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Driving::setMouseControl(
     Controller::Type xtype, int xid, Controller::Type ytype, int yid)
