@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,15 +33,12 @@
 #ifdef GUI_SUPPORT
   #include "Menu.hxx"
   #include "CommandMenu.hxx"
+  #include "HighScoresMenu.hxx"
   #include "MessageMenu.hxx"
   #include "Launcher.hxx"
   #include "TimeMachine.hxx"
   #include "Widget.hxx"
 #endif
-#ifdef SQLITE_SUPPORT
-  #include "KeyValueRepositorySqlite.hxx"
-  #include "SettingsDb.hxx"
-#endif
 
 #include "FSNode.hxx"
 #include "MD5.hxx"
@@ -49,7 +46,6 @@
 #include "CartCreator.hxx"
 #include "FrameBuffer.hxx"
 #include "TIASurface.hxx"
-#include "PaletteHandler.hxx"
 #include "TIAConstants.hxx"
 #include "Settings.hxx"
 #include "PropsSet.hxx"
@@ -59,13 +55,16 @@
 #include "Random.hxx"
 #include "StateManager.hxx"
 #include "TimerManager.hxx"
+#ifdef GUI_SUPPORT
+#include "HighScoresManager.hxx"
+#endif
 #include "Version.hxx"
 #include "TIA.hxx"
 #include "DispatchResult.hxx"
 #include "EmulationWorker.hxx"
 #include "AudioSettings.hxx"
 #include "repository/KeyValueRepositoryNoop.hxx"
-#include "repository/KeyValueRepositoryConfigfile.hxx"
+#include "repository/CompositeKeyValueRepositoryNoop.hxx"
 #include "M6532.hxx"
 
 #include "OSystem.hxx"
@@ -114,8 +113,10 @@ OSystem::~OSystem()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool OSystem::create()
+bool OSystem::initialize(const Settings::Options& options)
 {
+  loadConfig(options);
+
   ostringstream buf;
   buf << "Stella " << STELLA_VERSION << endl
       << "  Features: " << myFeatures << endl
@@ -125,14 +126,9 @@ bool OSystem::create()
       << "State directory:    '"
       << myStateDir.getShortPath() << "'" << endl
       << "NVRam directory:    '"
-      << myNVRamDir.getShortPath() << "'" << endl;
-
-  if(myConfigFile.getPath() != EmptyString)
-    buf << "Configuration file: '"
-        << myConfigFile.getShortPath() << "'" << endl;
-
-  buf << "Game properties:    '"
-      << myPropertiesFile.getShortPath() << "'" << endl
+      << myNVRamDir.getShortPath() << "'" << endl
+      << "Persistence:        '"
+      << describePresistence() << "'" << endl
       << "Cheat file:         '"
       << myCheatFile.getShortPath() << "'" << endl
       << "Palette file:       '"
@@ -148,8 +144,9 @@ bool OSystem::create()
     myFrameBuffer = make_unique<FrameBuffer>(*this);
     myFrameBuffer->initialize();
   }
-  catch(...)
+  catch(const runtime_error& e)
   {
+    Logger::error(e.what());
     return false;
   }
 
@@ -159,6 +156,7 @@ bool OSystem::create()
 
   myStateManager = make_unique<StateManager>(*this);
   myTimerManager = make_unique<TimerManager>();
+
   myAudioSettings = make_unique<AudioSettings>(*mySettings);
 
   // Create the sound object; the sound subsystem isn't actually
@@ -178,9 +176,13 @@ bool OSystem::create()
   // Create various subsystems (menu and launcher GUI objects, etc)
   myMenu = make_unique<Menu>(*this);
   myCommandMenu = make_unique<CommandMenu>(*this);
+  myHighScoresManager = make_unique<HighScoresManager>(*this);
+  myHighScoresMenu = make_unique<HighScoresMenu>(*this);
   myMessageMenu = make_unique<MessageMenu>(*this);
   myTimeMachine = make_unique<TimeMachine>(*this);
   myLauncher = make_unique<Launcher>(*this);
+
+  myHighScoresManager->setRepository(getHighscoreRepository());
 #endif
 
 #ifdef PNG_SUPPORT
@@ -188,8 +190,6 @@ bool OSystem::create()
   myPNGLib = make_unique<PNGLibrary>(*this);
 #endif
 
-  myPropSet->load(myPropertiesFile);
-
   // Detect serial port for AtariVox-USB
   // If a previously set port is defined, use it;
   // otherwise use the first one found (if any)
@@ -207,35 +207,34 @@ void OSystem::loadConfig(const Settings::Options& options)
 {
   // Get base directory and config file from derived class
   // It will decide whether it can override its default location
-  string baseDir, cfgFile, defSaveDir, defLoadDir;
-  getBaseDirAndConfig(baseDir, cfgFile, defSaveDir, defLoadDir,
-                      ourOverrideBaseDirWithApp, ourOverrideBaseDir);
+  string baseDir, homeDir;
+  getBaseDirectories(baseDir, homeDir,
+                     ourOverrideBaseDirWithApp, ourOverrideBaseDir);
 
   // Get fully-qualified pathnames, and make directories when needed
   myBaseDir = FilesystemNode(baseDir);
   if(!myBaseDir.isDirectory())
     myBaseDir.makeDir();
-  if(!cfgFile.empty())
-    myConfigFile = FilesystemNode(cfgFile);
 
-  myDefaultSaveDir = FilesystemNode(defSaveDir);
-  if(!myDefaultSaveDir.isDirectory())
-    myDefaultSaveDir.makeDir();
+  myHomeDir = FilesystemNode(homeDir);
+  if(!myHomeDir.isDirectory())
+    myHomeDir.makeDir();
 
-  myDefaultLoadDir = FilesystemNode(defLoadDir);
-  if(!myDefaultLoadDir.isDirectory())
-    myDefaultLoadDir.makeDir();
+  initPersistence(myBaseDir);
 
-#ifdef SQLITE_SUPPORT
-  mySettingsDb = make_shared<SettingsDb>(myBaseDir.getPath(), "settings");
-  if(!mySettingsDb->initialize())
-    mySettingsDb.reset();
-#endif
-
-  mySettings->setRepository(createSettingsRepository());
+  mySettings->setRepository(getSettingsRepository());
+  myPropSet->setRepository(getPropertyRepository());
 
   mySettings->load(options);
 
+  // userDir is NOT affected by '-baseDir'and '-basedirinapp' params
+  string userDir = mySettings->getString("userdir");
+  if(userDir.empty())
+    userDir = homeDir;
+  myUserDir = FilesystemNode(userDir);
+  if(!myUserDir.isDirectory())
+    myUserDir.makeDir();
+
   Logger::instance().setLogParameters(mySettings->getInt("loglevel"),
                                       mySettings->getBool("logtoconsole"));
   Logger::debug("Loading config options ...");
@@ -248,22 +247,14 @@ void OSystem::loadConfig(const Settings::Options& options)
 void OSystem::saveConfig()
 {
   // Ask all subsystems to save their settings
-  if(myFrameBuffer)
+  if(myFrameBuffer && mySettings)
+    myFrameBuffer->saveConfig(settings());
+
+  if(mySettings)
   {
-    // Save the last windowed position and display on system shutdown
-    myFrameBuffer->saveCurrentWindowPosition();
-
-    Logger::debug("Saving TV effects options ...");
-    myFrameBuffer->tiaSurface().ntsc().saveConfig(settings());
-    Logger::debug("Saving palette settings...");
-    myFrameBuffer->tiaSurface().paletteHandler().saveConfig(settings());
+    Logger::debug("Saving config options ...");
+    mySettings->save();
   }
-
-  Logger::debug("Saving config options ...");
-  mySettings->save();
-
-  if(myPropSet && myPropSet->save(myPropertiesFile))
-    Logger::debug("Saving properties set ...");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -290,7 +281,7 @@ void OSystem::setConfigPaths()
 #ifdef PNG_SUPPORT
   const string& ssSaveDir = mySettings->getString("snapsavedir");
   if(ssSaveDir == EmptyString)
-    mySnapshotSaveDir = defaultSaveDir();
+    mySnapshotSaveDir = userDir();
   else
     mySnapshotSaveDir = FilesystemNode(ssSaveDir);
   if(!mySnapshotSaveDir.isDirectory())
@@ -298,7 +289,7 @@ void OSystem::setConfigPaths()
 
   const string& ssLoadDir = mySettings->getString("snaploaddir");
   if(ssLoadDir == EmptyString)
-    mySnapshotLoadDir = defaultLoadDir();
+    mySnapshotLoadDir = userDir();
   else
     mySnapshotLoadDir = FilesystemNode(ssLoadDir);
   if(!mySnapshotLoadDir.isDirectory())
@@ -307,7 +298,6 @@ void OSystem::setConfigPaths()
 
   myCheatFile = myBaseDir;  myCheatFile /= "stella.cht";
   myPaletteFile = myBaseDir;  myPaletteFile /= "stella.pal";
-  myPropertiesFile = myBaseDir;  myPropertiesFile /= "stella.pro";
 
 #if 0
   // Debug code
@@ -323,11 +313,17 @@ void OSystem::setConfigPaths()
   dbgPath("sload dir ", mySnapshotLoadDir);
   dbgPath("cheat file", myCheatFile);
   dbgPath("pal file  ", myPaletteFile);
-  dbgPath("pro file  ", myPropertiesFile);
-  dbgPath("INI file  ", myConfigFile);
 #endif
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void OSystem::setUserDir(const string& path)
+{
+  mySettings->setValue("userdir", path);
+
+  myUserDir = FilesystemNode(path);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool OSystem::checkUserPalette(bool outputError) const
 {
@@ -475,9 +471,9 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
     {
       const string& id = myConsole->cartridge().multiCartID();
       if(id == "")
-        myFrameBuffer->showMessage("New console created");
+        myFrameBuffer->showTextMessage("New console created");
       else
-        myFrameBuffer->showMessage("Multicart " +
+        myFrameBuffer->showTextMessage("Multicart " +
           myConsole->cartridge().detectedType() + ", loading ROM" + id);
     }
     buf << "Game console created:" << endl
@@ -506,7 +502,7 @@ string OSystem::createConsole(const FilesystemNode& rom, const string& md5sum,
       msg << myConsole->leftController().name() << "/" << myConsole->rightController().name()
         << " - " << myConsole->cartridge().detectedType()
         << " - " << myConsole->getFormatString();
-      myFrameBuffer->showMessage(msg.str());
+      myFrameBuffer->showTextMessage(msg.str());
     }
   }
 
@@ -736,7 +732,7 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
   if (!myConsole) return 0.;
 
   TIA& tia(myConsole->tia());
-  EmulationTiming& timing(myConsole->emulationTiming());
+  const EmulationTiming& timing = myConsole->emulationTiming();
   DispatchResult dispatchResult;
 
   // Check whether we have a frame pending for rendering...
@@ -748,8 +744,8 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
     tia.renderToFrameBuffer();
   }
 
-  // Start emulation on a dedicated thread. It will do its own scheduling to sync 6507 and real time
-  // and will run until we stop the worker.
+  // Start emulation on a dedicated thread. It will do its own scheduling to
+  // sync 6507 and real time and will run until we stop the worker.
   emulationWorker.start(
     timing.cyclesPerSecond(),
     timing.maxCyclesPerTimeslice(),
@@ -758,8 +754,8 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
     &tia
   );
 
-  // Render the frame. This may block, but emulation will continue to run on the worker, so the
-  // audio pipeline is kept fed :)
+  // Render the frame. This may block, but emulation will continue to run on
+  // the worker, so the audio pipeline is kept fed :)
   if (framePending) myFrameBuffer->updateInEmulationMode(myFpsMeter.fps());
 
   // Stop the worker and wait until it has finished
@@ -775,7 +771,8 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
        myDebugger->start(
           dispatchResult.getMessage(),
           dispatchResult.getAddress(),
-          dispatchResult.wasReadTrap()
+          dispatchResult.wasReadTrap(),
+          dispatchResult.getToolTip()
         );
       #endif
 
@@ -794,11 +791,13 @@ double OSystem::dispatchEmulation(EmulationWorker& emulationWorker)
   }
 
   // Handle frying
-  if (dispatchResult.getStatus() == DispatchResult::Status::ok && myEventHandler->frying())
+  if (dispatchResult.getStatus() == DispatchResult::Status::ok &&
+      myEventHandler->frying())
     myConsole->fry();
 
   // Return the 6507 time used in seconds
-  return static_cast<double>(totalCycles) / static_cast<double>(timing.cyclesPerSecond());
+  return static_cast<double>(totalCycles) /
+      static_cast<double>(timing.cyclesPerSecond());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -828,7 +827,15 @@ void OSystem::mainLoop()
     if (myEventHandler->state() == EventHandlerState::EMULATION)
       // Dispatch emulation and render frame (if applicable)
       timesliceSeconds = dispatchEmulation(emulationWorker);
-    else {
+    else if(myEventHandler->state() == EventHandlerState::PLAYBACK)
+    {
+      // Playback at emulation speed
+      timesliceSeconds = static_cast<double>(myConsole->tia().scanlinesLastFrame() * 76) /
+        static_cast<double>(myConsole->emulationTiming().cyclesPerSecond());
+      myFrameBuffer->update();
+    }
+    else
+    {
       // Render the GUI with 60 Hz in all other modes
       timesliceSeconds = 1. / 60.;
       myFrameBuffer->update();
@@ -864,21 +871,6 @@ void OSystem::mainLoop()
 #endif
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-shared_ptr<KeyValueRepository> OSystem::createSettingsRepository()
-{
-  #ifdef SQLITE_SUPPORT
-    return mySettingsDb
-      ? shared_ptr<KeyValueRepository>(mySettingsDb, &mySettingsDb->settingsRepository())
-      : make_shared<KeyValueRepositoryNoop>();
-  #else
-    if (myConfigFile.getPath() == EmptyString)
-      return make_shared<KeyValueRepositoryNoop>();
-
-    return make_shared<KeyValueRepositoryConfigfile>(myConfigFile);
-  #endif
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string OSystem::ourOverrideBaseDir = "";
 bool OSystem::ourOverrideBaseDirWithApp = false;
