@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //============================================================================
 
-#include <sstream>
+#include <map>
 
-#include "bspf.hxx"
 #include "Props.hxx"
+#include "Variant.hxx"
+#include "bspf.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Properties::Properties()
@@ -32,6 +33,33 @@ Properties::Properties(const Properties& properties)
   copy(properties);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Properties::load(KeyValueRepository& repo)
+{
+  setDefaults();
+
+  const auto props = repo.load();
+
+  for (const auto& [key, value]: props)
+    set(getPropType(key), value.toString());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Properties::save(KeyValueRepository& repo) const
+{
+  std::map<string, Variant> props;
+
+  for (size_t i = 0; i < static_cast<size_t>(PropType::NumTypes); i++) {
+    if (myProperties[i] == ourDefaultProperties[i]) {
+      if (repo.atomic()) repo.atomic()->remove(ourPropertyNames[i]);
+    } else {
+      props[ourPropertyNames[i]] = myProperties[i];
+    }
+  }
+
+  return repo.save(props);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::set(PropType key, const string& value)
 {
@@ -79,120 +107,6 @@ void Properties::set(PropType key, const string& value)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-istream& operator>>(istream& is, Properties& p)
-{
-  p.setDefaults();
-
-  // Loop reading properties
-  string key, value;
-  for(;;)
-  {
-    // Get the key associated with this property
-    key = p.readQuotedString(is);
-
-    // Make sure the stream is still okay
-    if(!is)
-      return is;
-
-    // A null key signifies the end of the property list
-    if(key == "")
-      break;
-
-    // Get the value associated with this property
-    value = p.readQuotedString(is);
-
-    // Make sure the stream is still okay
-    if(!is)
-      return is;
-
-    // Set the property
-    PropType type = Properties::getPropType(key);
-    p.set(type, value);
-  }
-
-  return is;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-ostream& operator<<(ostream& os, const Properties& p)
-{
-  // Write out each of the key and value pairs
-  bool changed = false;
-  for(size_t i = 0; i < static_cast<size_t>(PropType::NumTypes); ++i)
-  {
-    // Try to save some space by only saving the items that differ from default
-    if(p.myProperties[i] != Properties::ourDefaultProperties[i])
-    {
-      p.writeQuotedString(os, Properties::ourPropertyNames[i]);
-      os.put(' ');
-      p.writeQuotedString(os, p.myProperties[i]);
-      os.put('\n');
-      changed = true;
-    }
-  }
-
-  if(changed)
-  {
-    // Put a trailing null string so we know when to stop reading
-    p.writeQuotedString(os, "");
-    os.put('\n');
-    os.put('\n');
-  }
-
-  return os;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Properties::readQuotedString(istream& in)
-{
-  // Read characters until we see a quote
-  char c;
-  while(in.get(c))
-    if(c == '"')
-      break;
-
-  // Read characters until we see the close quote
-  string s;
-  while(in.get(c))
-  {
-    if((c == '\\') && (in.peek() == '"'))
-      in.get(c);
-    else if((c == '\\') && (in.peek() == '\\'))
-      in.get(c);
-    else if(c == '"')
-      break;
-    else if(c == '\r')
-      continue;
-
-    s += c;
-  }
-
-  return s;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Properties::writeQuotedString(ostream& out, const string& s)
-{
-  out.put('"');
-  for(uInt32 i = 0; i < s.length(); ++i)
-  {
-    if(s[i] == '\\')
-    {
-      out.put('\\');
-      out.put('\\');
-    }
-    else if(s[i] == '\"')
-    {
-      out.put('\\');
-      out.put('"');
-    }
-    else
-      out.put(s[i]);
-  }
-  out.put('"');
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Properties::operator==(const Properties& properties) const
 {
@@ -265,10 +179,19 @@ void Properties::print() const
        << get(PropType::Display_Format)         << "|"
        << get(PropType::Display_VCenter)        << "|"
        << get(PropType::Display_Phosphor)       << "|"
-       << get(PropType::Display_PPBlend)
+       << get(PropType::Display_PPBlend)        << "|"
+       << get(PropType::Cart_Highscore)
        << endl;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Properties::reset(PropType key)
+{
+  size_t pos = static_cast<size_t>(key);
+
+  myProperties[pos] = ourDefaultProperties[pos];
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Properties::setDefaults()
 {
@@ -316,7 +239,8 @@ void Properties::printHeader()
        << "Display_Format|"
        << "Display_VCenter|"
        << "Display_Phosphor|"
-       << "Display_PPBlend"
+       << "Display_PPBlend|"
+       << "Cart_Highscore"
        << endl;
 }
 
@@ -349,7 +273,8 @@ std::array<string, Properties::NUM_PROPS> Properties::ourDefaultProperties =
   "AUTO",   // Display.Format
   "0",      // Display.VCenter
   "NO",     // Display.Phosphor
-  "0"       // Display.PPBlend
+  "0",      // Display.PPBlend
+  ""        // Cart.Highscore
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -381,5 +306,6 @@ std::array<string, Properties::NUM_PROPS> Properties::ourPropertyNames =
   "Display.Format",
   "Display.VCenter",
   "Display.Phosphor",
-  "Display.PPBlend"
+  "Display.PPBlend",
+  "Cart.Highscore"
 };
