@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -25,7 +25,7 @@ SaveKey::SaveKey(Jack jack, const Event& event, const System& system,
                  const FilesystemNode& eepromfile, const onMessageCallback& callback,
                  Type type)
   : Controller(jack, event, system, type),
-    myEEPROM(make_unique<MT24LC256>(eepromfile, system, callback))
+    myEEPROM{make_unique<MT24LC256>(eepromfile, system, callback)}
 {
   setPin(DigitalPin::One, true);
   setPin(DigitalPin::Two, true);
