@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -29,15 +29,14 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font)
   : Dialog(osystem, parent, font, "About Stella")
 {
-  const int lineHeight   = font.getLineHeight(),
-        fontWidth    = font.getMaxCharWidth(),
-        fontHeight   = font.getFontHeight(),
-        buttonWidth  = font.getStringWidth("Previous") + fontWidth * 2.5,
-        buttonHeight = font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int VGAP = fontHeight / 4;
-
+  const int lineHeight   = Dialog::lineHeight(),
+            fontHeight   = Dialog::fontHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            buttonWidth  = Dialog::buttonWidth("Previous"),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int xpos, ypos;
   WidgetArray wid;
 
@@ -67,11 +66,12 @@ AboutDialog::AboutDialog(OSystem& osystem, DialogContainer& parent,
   addCancelWidget(b);
 
   xpos = HBORDER;  ypos = _th + VBORDER + (buttonHeight - fontHeight) / 2;
-  myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - xpos * 2, fontHeight,
-                                 "", TextAlign::Center);
+  int bwidth = font.getStringWidth("What's New" + ELLIPSIS) + fontWidth * 2.5;
+
+  myTitle = new StaticTextWidget(this, font, xpos + bwidth, ypos, _w - (xpos + bwidth) * 2,
+                                 fontHeight, "", TextAlign::Center);
   myTitle->setTextColor(kTextColorEm);
 
-  int bwidth = font.getStringWidth("What's New" + ELLIPSIS) + fontWidth * 2.5;
   myWhatsNewButton =
     new ButtonWidget(this, font, _w - HBORDER - bwidth, ypos - (buttonHeight - fontHeight) / 2,
                      bwidth, buttonHeight, "What's New" + ELLIPSIS, kWhatsNew);
@@ -118,7 +118,7 @@ void AboutDialog::updateStrings(int page, int lines, string& title)
       ADD_ATEXT(string("\\C\\c2Features: ") + instance().features());
       ADD_ATEXT(string("\\C\\c2") + instance().buildInfo());
       ADD_ALINE();
-      ADD_ATEXT("\\CCopyright (c) 1995-2020 The Stella Team");
+      ADD_ATEXT("\\CCopyright (c) 1995-2021 The Stella Team");
       ADD_ATEXT("\\C(https://stella-emu.github.io)");
       ADD_ALINE();
       ADD_ATEXT("\\CStella is now DonationWare!");
@@ -171,7 +171,7 @@ void AboutDialog::updateStrings(int page, int lines, string& title)
       return;
   }
 
-  while(i < lines)
+  while(i < lines)  // NOLINT : i changes in lambda above
     ADD_ALINE();
 }
 
@@ -273,7 +273,7 @@ void AboutDialog::handleCommand(CommandSender* sender, int cmd, int data, int id
 
     case kWhatsNew:
       if(myWhatsNewDialog == nullptr)
-        myWhatsNewDialog = make_unique<WhatsNewDialog>(instance(), parent(), _font,
+        myWhatsNewDialog = make_unique<WhatsNewDialog>(instance(), parent(),
                                                        640 * 0.95, 480 * 0.95);
       myWhatsNewDialog->open();
       break;
