@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,8 +28,8 @@ ColorWidget::ColorWidget(GuiObject* boss, const GUI::Font& font,
                          int x, int y, int w, int h, int cmd, bool framed)
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss),
-    _framed(framed),
-    _cmd(cmd)
+    _framed{framed},
+    _cmd{cmd}
 {
   _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
 }
@@ -37,15 +37,27 @@ ColorWidget::ColorWidget(GuiObject* boss, const GUI::Font& font,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ColorWidget::setColor(ColorId color)
 {
-  _color = color;
-  setDirty();
+  if(_color != color)
+  {
+    _color = color;
+    setDirty();
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ColorWidget::setCrossed(bool enable)
+{
+  if(_crossGrid != enable)
+  {
+    _crossGrid = enable;
+    setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ColorWidget::drawWidget(bool hilite)
 {
   FBSurface& s = dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
 
   if(_framed)
   {
@@ -53,11 +65,11 @@ void ColorWidget::drawWidget(bool hilite)
     s.frameRect(_x, _y, _w, _h + 1, kColor);
 
     // Show the currently selected color
-    s.fillRect(_x + 1, _y + 1, _w - 2, _h - 1, onTop ? isEnabled() ? _color : kWidColor : kBGColorLo);
+    s.fillRect(_x + 1, _y + 1, _w - 2, _h - 1, isEnabled() ? _color : kWidColor);
   }
   else
   {
-    s.fillRect(_x, _y, _w, _h, onTop ? isEnabled() ? _color : kWidColor : kBGColorLo);
+    s.fillRect(_x, _y, _w, _h, isEnabled() ? _color : kWidColor);
   }
 
   // Cross out the grid?
