@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -17,6 +17,7 @@
 
 #include "OSystem.hxx"
 #include "Dialog.hxx"
+#include "ToolTip.hxx"
 #include "Stack.hxx"
 #include "EventHandler.hxx"
 #include "FrameBuffer.hxx"
@@ -27,7 +28,7 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DialogContainer::DialogContainer(OSystem& osystem)
-  : myOSystem(osystem)
+  : myOSystem{osystem}
 {
   _DOUBLE_CLICK_DELAY = osystem.settings().getInt("mdouble");
   _REPEAT_INITIAL_DELAY = osystem.settings().getInt("ctrldelay");
@@ -89,31 +90,55 @@ void DialogContainer::updateTime(uInt64 time)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool DialogContainer::draw(bool full)
+void DialogContainer::draw(bool full)
 {
   if(myDialogStack.empty())
-    return false;
+    return;
+#ifdef DEBUG_BUILD
+  //cerr << "draw " << full << " " << typeid(*this).name() << endl;
+#endif
 
-  // Make the top dialog dirty if a full redraw is requested
-  if(full)
-    myDialogStack.top()->setDirty();
-
-  // If the top dialog is dirty, then all below it must be redrawn too
-  const bool dirty = needsRedraw();
-
-  myDialogStack.applyAll([&](Dialog*& d){
-    if(dirty)
-      d->setDirty();
-    full |= d->render();
+  // Draw and render all dirty dialogs
+  myDialogStack.applyAll([&](Dialog*& d) {
+    if(full || d->needsRedraw())
+      d->redraw(full);
   });
+  // Always render all surfaces, bottom to top
+  render();
+}
 
-  return full;
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DialogContainer::tick()
+{
+  if(!myDialogStack.empty())
+    myDialogStack.top()->tick();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DialogContainer::render()
+{
+  if(myDialogStack.empty())
+    return;
+#ifdef DEBUG_BUILD
+  //cerr << "full re-render " << typeid(*this).name() << endl;
+#endif
+
+  // Make sure we start in a clean state (with zero'ed buffers)
+  if(!myOSystem.eventHandler().inTIAMode())
+    myOSystem.frameBuffer().clear();
+
+  // Render all dialogs
+  myDialogStack.applyAll([&](Dialog*& d) {
+    d->render();
+  });
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool DialogContainer::needsRedraw() const
 {
-  return !myDialogStack.empty() ? myDialogStack.top()->isDirty() : false;
+  return !myDialogStack.empty()
+    ? myDialogStack.top()->needsRedraw()
+    : false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -129,11 +154,16 @@ int DialogContainer::addDialog(Dialog* d)
   const uInt32 scale = myOSystem.frameBuffer().hidpiScaleFactor();
 
   if(uInt32(d->getWidth() * scale) > r.w() || uInt32(d->getHeight() * scale) > r.h())
-    myOSystem.frameBuffer().showMessage(
-        "Unable to show dialog box; FIX THE CODE");
+    myOSystem.frameBuffer().showTextMessage(
+      "Unable to show dialog box; FIX THE CODE", MessagePosition::BottomCenter, true);
   else
   {
+    // Close all open tooltips
+    if(!myDialogStack.empty())
+      myDialogStack.top()->tooltip().hide();
+
     d->setDirty();
+    d->resetSurfaces();
     myDialogStack.push(d);
   }
   return myDialogStack.size();
@@ -144,9 +174,13 @@ void DialogContainer::removeDialog()
 {
   if(!myDialogStack.empty())
   {
+  #ifdef DEBUG_BUILD
+    //cerr << "remove dialog " << typeid(*myDialogStack.top()).name() << endl;
+  #endif
     myDialogStack.pop();
-    if(!myDialogStack.empty())
-      myDialogStack.top()->setDirty();
+
+    // Inform the frame buffer that it has to render all surfaces
+    myOSystem.frameBuffer().setPendingRender();
   }
 }
 
@@ -157,12 +191,23 @@ void DialogContainer::reStack()
   while(!myDialogStack.empty())
     myDialogStack.top()->close();
 
+  // Make sure that all surfaces are cleared
+  myOSystem.frameBuffer().clear();
+
   baseDialog()->open();
 
   // Reset all continuous events
   reset();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DialogContainer::resetSurfaces()
+{
+  myDialogStack.applyAll([&](Dialog*& d) {
+    d->resetSurfaces();
+  });
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::handleTextEvent(char text)
 {
