@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -18,6 +18,7 @@
 #include "OSystem.hxx"
 #include "FBSurface.hxx"
 #include "Dialog.hxx"
+#include "ToolTip.hxx"
 #include "Font.hxx"
 #include "EditTextWidget.hxx"
 
@@ -26,7 +27,8 @@ EditTextWidget::EditTextWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int w, int h, const string& text)
   : EditableWidget(boss, font, x, y, w, h + 2, text)
 {
-  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG
+    | Widget::FLAG_RETAIN_FOCUS | Widget::FLAG_TRACK_MOUSE;
 
   EditableWidget::startEditMode();  // We're always in edit mode
 
@@ -45,56 +47,38 @@ void EditTextWidget::setText(const string& str, bool changed)
 {
   EditableWidget::setText(str, changed);
   _backupString = str;
-  _changed = changed;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EditTextWidget::handleMouseEntered()
-{
-  setFlags(Widget::FLAG_HILITED);
-  setDirty();
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void EditTextWidget::handleMouseLeft()
-{
-  clearFlags(Widget::FLAG_HILITED);
-  setDirty();
+  if(_changed != changed)
+  {
+    _changed = changed;
+    setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
-  if(!isEditable())
-    return;
-
-  x += _editScrollOffset;
-
-  int width = 0;
-  uInt32 i;
-
-  for (i = 0; i < editString().size(); ++i)
+  if(b == MouseButton::LEFT)
   {
-    width += _font.getCharWidth(editString()[i]);
-    if (width >= x)
-      break;
-  }
+    if(!isEditable())
+      return;
 
-  if (setCaretPos(i))
-    setDirty();
+    resetSelection();
+    if(setCaretPos(toCaretPos(x)))
+      setDirty();
+  }
+  EditableWidget::handleMouseDown(x, y, b, clickCount);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::drawWidget(bool hilite)
 {
   FBSurface& s = _boss->dialog().surface();
-  bool onTop = _boss->dialog().isOnTop();
 
   // Highlight changes
-  if(_changed && onTop)
+  if(_changed)
     s.fillRect(_x, _y, _w, _h, kDbgChangedColor);
   else if(!isEditable() || !isEnabled())
-    s.fillRect(_x, _y, _w, _h, onTop ? kDlgColor : kBGColorLo);
+    s.fillRect(_x, _y, _w, _h, kDlgColor);
 
   // Draw a thin frame around us.
   s.frameRect(_x, _y, _w, _h, hilite && isEditable() && isEnabled() ? kWidColorHi : kColor);
@@ -102,13 +86,13 @@ void EditTextWidget::drawWidget(bool hilite)
   // Draw the text
   adjustOffset();
   s.drawString(_font, editString(), _x + _textOfs, _y + 2, getEditRect().w(), getEditRect().h(),
-               _changed && onTop && isEnabled()
+               _changed && isEnabled()
                ? kDbgChangedTextColor
-               : onTop && isEnabled() ? _textcolor : kColor,
-               TextAlign::Left, isEditable() ? -_editScrollOffset : 0, !isEditable());
+               : isEnabled() ? _textcolor : kColor,
+               TextAlign::Left, scrollOffset(), !isEditable());
 
-  // Draw the caret
-  drawCaret();
+  // Draw the caret and selection
+  drawCaretSelection();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -120,6 +104,7 @@ Common::Rect EditTextWidget::getEditRect() const
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::lostFocusWidget()
 {
+  EditableWidget::lostFocusWidget();
   // If we loose focus, 'commit' the user changes
   _backupString = editString();
 }
