@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,15 +28,14 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
                        const GUI::Font& font)
   : Dialog(osystem, parent, font, "Help")
 {
-  const int lineHeight   = font.getLineHeight(),
-            fontWidth    = font.getMaxCharWidth(),
-            fontHeight   = font.getFontHeight(),
-            buttonWidth  = font.getStringWidth("Previous") + fontWidth * 2.5,
-            buttonHeight = font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int VGAP = fontHeight / 4;
-
+  const int lineHeight   = Dialog::lineHeight(),
+            fontHeight   = Dialog::fontHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            buttonWidth  = Dialog::buttonWidth("Previous"),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int xpos, ypos;
   WidgetArray wid;
 
@@ -76,10 +75,10 @@ HelpDialog::HelpDialog(OSystem& osystem, DialogContainer& parent,
   {
     myKey[i] =
       new StaticTextWidget(this, font, xpos, ypos, lwidth,
-                           fontHeight, "", TextAlign::Left);
+                           fontHeight);
     myDesc[i] =
       new StaticTextWidget(this, font, xpos+lwidth, ypos, _w - xpos - lwidth - HBORDER,
-                           fontHeight, "", TextAlign::Left);
+                           fontHeight);
     ypos += fontHeight;
   }
 
@@ -125,7 +124,7 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
       ADD_EVENT(Event::TogglePhosphor,      "Toggle 'phosphor' effect");
       ADD_LINE();
       ADD_EVENT(Event::ToggleGrabMouse,     "Grab mouse (keep in window)");
-      ADD_EVENT(Event::HandleMouseControl,  "Toggle controller for mouse");
+      ADD_EVENT(Event::NextMouseControl,    "Toggle controller for mouse");
       ADD_EVENT(Event::ToggleSAPortOrder,   "Toggle Stelladaptor left/right");
       ADD_LINE();
       ADD_EVENT(Event::VolumeIncrease,      "Increase volume by 2%");
@@ -134,16 +133,16 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
 
     case 3:
       title = "TV effects";
-      ADD_EVENT(Event::VidmodeStd,          "Disable TV effects");
-      ADD_EVENT(Event::VidmodeRGB,          "Enable 'RGB' mode");
-      ADD_EVENT(Event::VidmodeSVideo,       "Enable 'S-Video' mode");
-      ADD_EVENT(Event::VidModeComposite,    "Enable 'Composite' mode");
-      ADD_EVENT(Event::VidModeBad,          "Enable 'Badly adjusted' mode");
-      ADD_EVENT(Event::VidModeCustom,       "Enable 'Custom' mode");
-      ADD_EVENT(Event::NextAttribute,       "Select 'Custom' attribute");
-      ADD_EVENT(Event::IncreaseAttribute,   "Modify 'Custom' attribute");
-      ADD_EVENT(Event::PhosphorIncrease,    "Adjust phosphor blend");
-      ADD_EVENT(Event::ScanlinesIncrease,   "Adjust scanline intensity");
+      ADD_EVENT(Event::NextVideoMode,       "Select next TV effect mode");
+      ADD_EVENT(Event::PreviousVideoMode,   "Select previous TV effect mode");
+      ADD_EVENT(Event::NextAttribute,       "Select next 'Custom' attribute");
+      ADD_EVENT(Event::PreviousAttribute,   "Select previous 'Custom' attr.");
+      ADD_EVENT(Event::IncreaseAttribute,   "Increase 'Custom' attribute");
+      ADD_EVENT(Event::DecreaseAttribute,   "Decrease 'Custom' attribute");
+      ADD_EVENT(Event::PhosphorIncrease,    "Increase phosphor blend");
+      ADD_EVENT(Event::PhosphorDecrease,    "Decrease phosphor blend");
+      ADD_EVENT(Event::ScanlinesIncrease,   "Increase scanline intensity");
+      ADD_EVENT(Event::ScanlinesDecrease,   "Decrease scanline intensity");
       break;
 
     case 4:
@@ -174,7 +173,7 @@ void HelpDialog::updateStrings(uInt8 page, uInt8 lines, string& title)
       break;
   }
 
-  while(i < lines)
+  while(i < lines)  // NOLINT : i changes in lambda above
     ADD_LINE();
 }
 
