@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -41,16 +41,15 @@
 InputDialog::InputDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent, font, "Input settings"),
-    myMaxWidth(max_w),
-    myMaxHeight(max_h)
+    myMaxWidth{max_w},
+    myMaxHeight{max_h}
 {
-  const int lineHeight   = _font.getLineHeight(),
-            fontWidth    = _font.getMaxCharWidth(),
-            fontHeight   = _font.getFontHeight(),
-            buttonHeight = _font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int VGAP = fontHeight / 4;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight   = Dialog::lineHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int xpos, ypos, tabID;
 
   // Set real dimensions
@@ -107,13 +106,12 @@ InputDialog::~InputDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputDialog::addDevicePortTab()
 {
-  const int lineHeight = _font.getLineHeight(),
-            fontWidth  = _font.getMaxCharWidth(),
-            fontHeight = _font.getFontHeight(),
-            buttonHeight = _font.getLineHeight() * 1.25;
-  const int VGAP = fontHeight / 4;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight   = Dialog::lineHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int xpos, ypos, lwidth, tabID;
   WidgetArray wid;
 
@@ -126,7 +124,8 @@ void InputDialog::addDevicePortTab()
   // Add joystick deadzone setting
   myDeadzone = new SliderWidget(myTab, _font, HBORDER, ypos - 1, 13 * fontWidth, lineHeight,
                                 "Joystick deadzone size", lwidth, kDeadzoneChanged, 5 * fontWidth);
-  myDeadzone->setMinValue(0); myDeadzone->setMaxValue(29);
+  myDeadzone->setMinValue(Joystick::DEAD_ZONE_MIN);
+  myDeadzone->setMaxValue(Joystick::DEAD_ZONE_MAX);
   myDeadzone->setTickmarkIntervals(4);
   wid.push_back(myDeadzone);
 
@@ -152,6 +151,8 @@ void InputDialog::addDevicePortTab()
   myDejitterBase->setMinValue(Paddles::MIN_DEJITTER);
   myDejitterBase->setMaxValue(Paddles::MAX_DEJITTER);
   myDejitterBase->setTickmarkIntervals(5);
+  myDejitterBase->setToolTip("Adjust paddle input averaging.\n"
+                             "Note: Already implemented in 2600-daptor");
   //xpos += myDejitterBase->getWidth() + fontWidth - 4;
   wid.push_back(myDejitterBase);
 
@@ -162,6 +163,7 @@ void InputDialog::addDevicePortTab()
   myDejitterDiff->setMinValue(Paddles::MIN_DEJITTER);
   myDejitterDiff->setMaxValue(Paddles::MAX_DEJITTER);
   myDejitterDiff->setTickmarkIntervals(5);
+  myDejitterDiff->setToolTip("Adjust paddle reaction to fast movements.");
   wid.push_back(myDejitterDiff);
 
   // Add paddle speed (digital emulation)
@@ -209,7 +211,7 @@ void InputDialog::addDevicePortTab()
 
   // Show joystick database
   ypos += lineHeight;
-  lwidth = _font.getStringWidth("Joystick Database" + ELLIPSIS) + fontWidth * 2.5;
+  lwidth = Dialog::buttonWidth("Joystick Database" + ELLIPSIS);
   myJoyDlgButton = new ButtonWidget(myTab, _font, HBORDER, ypos, lwidth, buttonHeight,
                                     "Joystick Database" + ELLIPSIS, kDBButtonPressed);
   wid.push_back(myJoyDlgButton);
@@ -236,13 +238,12 @@ void InputDialog::addDevicePortTab()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputDialog::addMouseTab()
 {
-  const int lineHeight = _font.getLineHeight(),
-            fontWidth  = _font.getMaxCharWidth(),
-            fontHeight = _font.getFontHeight();
-  const int VGAP = fontHeight / 4;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int INDENT = fontWidth * 2;
+  const int lineHeight = Dialog::lineHeight(),
+            fontWidth  = Dialog::fontWidth(),
+            VBORDER    = Dialog::vBorder(),
+            HBORDER    = Dialog::hBorder(),
+            VGAP       = Dialog::vGap(),
+            INDENT     = Dialog::indent();
   int xpos = HBORDER, ypos, lwidth, pwidth, tabID;
   WidgetArray wid;
   VariantList items;
@@ -291,6 +292,7 @@ void InputDialog::addMouseTab()
                                     lwidth, kDCSpeedChanged, 4 * fontWidth, "%");
   myDrivingSpeed->setMinValue(1); myDrivingSpeed->setMaxValue(20);
   myDrivingSpeed->setTickmarkIntervals(4);
+  myDrivingSpeed->setToolTip("Adjust driving controller sensitivity for digital and mouse input.");
   wid.push_back(myDrivingSpeed);
 
   // Mouse cursor state
@@ -365,7 +367,7 @@ void InputDialog::loadConfig()
   VariantList items;
 
   for(const auto& port: ports)
-    VarList::push_back(items, port);
+    VarList::push_back(items, port, port);
   if(avoxport != EmptyString && !BSPF::contains(ports, avoxport))
     VarList::push_back(items, avoxport, avoxport);
   if(items.size() == 0)
@@ -403,14 +405,7 @@ void InputDialog::saveConfig()
 {
   Settings& settings = instance().settings();
 
-  // Left & right ports
-  instance().eventHandler().mapStelladaptors(mySAPort->getState() ? "rl": "lr");
-
-  // Use mouse as a controller
-  const string& usemouse = myMouseControl->getSelectedTag().toString();
-  settings.setValue("usemouse", usemouse);
-  instance().eventHandler().setMouseControllerMode(usemouse);
-
+  // *** Device & Ports ***
   // Joystick deadzone
   int deadzone = myDeadzone->getValue();
   settings.setValue("joydeadzone", deadzone);
@@ -433,6 +428,31 @@ void InputDialog::saveConfig()
   settings.setValue("dsense", sensitivity);
   Paddles::setDigitalSensitivity(sensitivity);
 
+  // Autofire rate
+  int rate = myAutoFireRate->getValue();
+  settings.setValue("autofirerate", rate);
+  Controller::setAutoFireRate(rate);
+
+  // Allow all 4 joystick directions
+  bool allowall4 = myAllowAll4->getState();
+  settings.setValue("joyallow4", allowall4);
+  instance().eventHandler().allowAllDirections(allowall4);
+
+  // Enable/disable modifier key-combos
+  settings.setValue("modcombo", myModCombo->getState());
+
+  // Left & right ports
+  instance().eventHandler().mapStelladaptors(mySAPort->getState() ? "rl" : "lr");
+
+  // AtariVox serial port
+  settings.setValue("avoxport", myAVoxPort->getText());
+
+  // *** Mouse ***
+  // Use mouse as a controller
+  const string& usemouse = myMouseControl->getSelectedTag().toString();
+  settings.setValue("usemouse", usemouse);
+  instance().eventHandler().setMouseControllerMode(usemouse);
+
   sensitivity = myMPaddleSpeed->getValue();
   settings.setValue("msense", sensitivity);
   Paddles::setMouseSensitivity(sensitivity);
@@ -447,22 +467,10 @@ void InputDialog::saveConfig()
   settings.setValue("dcsense", sensitivity);
   Driving::setSensitivity(sensitivity);
 
-  // Autofire rate
-  int rate = myAutoFireRate->getValue();
-  settings.setValue("autofirerate", rate);
-  Controller::setAutoFireRate(rate);
-
-  // AtariVox serial port
-  settings.setValue("avoxport", myAVoxPort->getText());
-
-  // Allow all 4 joystick directions
-  bool allowall4 = myAllowAll4->getState();
-  settings.setValue("joyallow4", allowall4);
-  instance().eventHandler().allowAllDirections(allowall4);
-
   // Grab mouse and hide cursor
   const string& cursor = myCursorState->getSelectedTag().toString();
   settings.setValue("cursor", cursor);
+
   // only allow grab mouse if cursor is hidden in emulation
   int state = myCursorState->getSelected();
   bool enableGrab = state != 1 && state != 3;
@@ -470,9 +478,6 @@ void InputDialog::saveConfig()
   settings.setValue("grabmouse", grab);
   instance().frameBuffer().enableGrabMouse(grab);
 
-  // Enable/disable modifier key-combos
-  settings.setValue("modcombo", myModCombo->getState());
-
   instance().eventHandler().saveKeyMapping();
   instance().eventHandler().saveJoyMapping();
 //  instance().saveConfig();
@@ -668,11 +673,12 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kDeadzoneChanged:
-      myDeadzone->setValueLabel(3200 + 1000 * myDeadzone->getValue());
+      myDeadzone->setValueLabel(Joystick::deadZoneValue(myDeadzone->getValue()));
       break;
 
     case kPSpeedChanged:
-      myPaddleSpeed->setValueLabel(Paddles::setAnalogSensitivity(myPaddleSpeed->getValue()) * 100.F + 0.5F);
+      myPaddleSpeed->setValueLabel(std::round(Paddles::setAnalogSensitivity(
+            myPaddleSpeed->getValue()) * 100.F));
       break;
 
     case kDejitterAvChanged:
@@ -704,7 +710,7 @@ void InputDialog::handleCommand(CommandSender* sender, int cmd,
       {
         const GUI::Font& font = instance().frameBuffer().font();
         myJoyDialog = make_unique<JoystickDialog>
-          (this, font, font.getMaxCharWidth() * 56 + 20, font.getFontHeight() * 18 + 20);
+          (this, font, fontWidth() * 56 + 20, fontHeight() * 18 + 20);
       }
       myJoyDialog->show();
       break;
@@ -780,10 +786,8 @@ void InputDialog::handleMouseControlState()
 {
   bool enable = myMouseControl->getSelected() != 2;
 
-  myMouseSensitivity->setEnabled(enable);
   myMPaddleSpeed->setEnabled(enable);
   myTrackBallSpeed->setEnabled(enable);
-  myDrivingSpeed->setEnabled(enable);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
