@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -50,14 +50,13 @@ InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& lfont,
 void InputTextDialog::initialize(const GUI::Font& lfont, const GUI::Font& nfont,
                                  const StringList& labels)
 {
-  const int lineHeight   = lfont.getLineHeight(),
-            fontWidth    = lfont.getMaxCharWidth(),
-            fontHeight   = lfont.getFontHeight(),
-            buttonHeight = lfont.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int VGAP = fontHeight / 4;
-
+  const int lineHeight   = Dialog::lineHeight(),
+            fontHeight   = Dialog::fontHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   uInt32 xpos, ypos, i, lwidth = 0, maxIdx = 0;
   WidgetArray wid;
 
@@ -81,9 +80,10 @@ void InputTextDialog::initialize(const GUI::Font& lfont, const GUI::Font& nfont,
   for(i = 0; i < labels.size(); ++i)
   {
     xpos = HBORDER;
-    new StaticTextWidget(this, lfont, xpos, ypos + 2,
-                         lwidth, fontHeight,
-                         labels[i], TextAlign::Left);
+    StaticTextWidget* s = new StaticTextWidget(this, lfont, xpos, ypos + 2,
+                                               lwidth, fontHeight,
+                                               labels[i]);
+    myLabel.push_back(s);
 
     xpos += lwidth + fontWidth;
     EditTextWidget* w = new EditTextWidget(this, nfont, xpos, ypos,
@@ -130,7 +130,7 @@ void InputTextDialog::show(uInt32 x, uInt32 y, const Common::Rect& bossRect)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void InputTextDialog::center()
+void InputTextDialog::setPosition()
 {
   if(!myEnableCenter)
   {
@@ -144,7 +144,7 @@ void InputTextDialog::center()
       surface().setDstPos(myXOrig, myYOrig);
   }
   else
-    Dialog::center();
+    Dialog::setPosition();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -177,6 +177,13 @@ void InputTextDialog::setTextFilter(const EditableWidget::TextFilter& f, int idx
     myInput[idx]->setTextFilter(f);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void InputTextDialog::setToolTip(const string& str, int idx)
+{
+  if(uInt32(idx) < myLabel.size())
+    myLabel[idx]->setToolTip(str);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void InputTextDialog::setFocus(int idx)
 {
