@@ -295,14 +295,6 @@ LauncherDialog::LauncherDialog(OSystem& osystem, DialogContainer& parent,
 
   addToFocusList(wid);
 
-  // Create (empty) context menu for ROM list options
-  myMenu = make_unique<ContextMenu>(this, _font, EmptyVarList);
-
-  // Create global props dialog, which is used to temporarily override
-  // ROM properties
-  myGlobalProps = make_unique<GlobalPropsDialog>(this,
-    myUseMinimalUI ? _font : osystem.frameBuffer().font());
-
   // since we cannot know how many files there are, use are really high value here
   myList->progress().setRange(0, 50000, 5);
   myList->progress().setMessage("        Filtering files" + ELLIPSIS + "        ");
@@ -358,6 +350,15 @@ void LauncherDialog::reload()
   myPendingReload = false;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::resetSurfaces()
+{
+  if(myRomInfoWidget)
+    myRomInfoWidget->resetSurfaces();
+
+  Dialog::resetSurfaces();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::tick()
 {
@@ -613,16 +614,27 @@ void LauncherDialog::loadRomInfo()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::handleContextMenu()
 {
-  const string& cmd = myMenu->getSelectedTag().toString();
+  const string& cmd = menu().getSelectedTag().toString();
 
   if(cmd == "override")
-    myGlobalProps->open();
+    openGlobalProps();
   else if(cmd == "reload")
     reload();
   else if(cmd == "highscores")
     openHighScores();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ContextMenu& LauncherDialog::menu()
+{
+  if(myMenu == nullptr)
+    // Create (empty) context menu for ROM list options
+    myMenu = make_unique<ContextMenu>(this, _font, EmptyVarList);
+
+
+  return *myMenu;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::showOnlyROMs(bool state)
 {
@@ -641,7 +653,7 @@ void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
     switch(key)
     {
       case KBDK_P:
-        myGlobalProps->open();
+        openGlobalProps();
         break;
 
       case KBDK_H:
@@ -664,7 +676,7 @@ void LauncherDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
     {
       case KBDK_F8: // front  ("Skill P2")
         if (!currentNode().isDirectory() && Bankswitch::isValidRomName(currentNode()))
-          myGlobalProps->open();
+          openGlobalProps();
         break;
       case KBDK_F4: // back ("COLOR", "B/W")
         openSettings();
@@ -705,7 +717,7 @@ void LauncherDialog::handleJoyUp(int stick, int button)
 
   if (button == 1 && (e == Event::UIOK || e == Event::NoType) &&
       !currentNode().isDirectory() && Bankswitch::isValidRomName(currentNode()))
-    myGlobalProps->open();
+    openGlobalProps();
   if (button == 3 && (e == Event::Event::UITabPrev || e == Event::NoType))
     openSettings();
   else if (!myEventHandled)
@@ -757,10 +769,10 @@ void LauncherDialog::handleMouseDown(int x, int y, MouseButton b, int clickCount
     if(instance().highScores().enabled())
       VarList::push_back(items, " High scores" + ELLIPSIS + "        Ctrl+H", "highscores");
     VarList::push_back(items, " Reload listing      Ctrl+R ", "reload");
-    myMenu->addItems(items);
+    menu().addItems(items);
 
     // Add menu at current x,y mouse location
-    myMenu->show(x + getAbsX(), y + getAbsY(), surface().dstRect());
+    menu().show(x + getAbsX(), y + getAbsY(), surface().dstRect());
   }
   else
     Dialog::handleMouseDown(x, y, b, clickCount);
@@ -802,7 +814,7 @@ void LauncherDialog::handleCommand(CommandSender* sender, int cmd,
 
     case ListWidget::kLongButtonPressCmd:
       if (!currentNode().isDirectory() && Bankswitch::isValidRomName(currentNode()))
-        myGlobalProps->open();
+        openGlobalProps();
       myEventHandled = true;
       break;
 
@@ -878,6 +890,17 @@ void LauncherDialog::setDefaultDir()
   instance().settings().setValue("romdir", myList->currentDir().getShortPath());
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void LauncherDialog::openGlobalProps()
+{
+  // Create global props dialog, which is used to temporarily override
+  // ROM properties
+  myDialog = make_unique<GlobalPropsDialog>(this, myUseMinimalUI
+                                            ? _font
+                                            : instance().frameBuffer().font());
+  myDialog->open();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::openSettings()
 {
@@ -885,37 +908,26 @@ void LauncherDialog::openSettings()
 
   // Create an options dialog, similar to the in-game one
   if (instance().settings().getBool("basic_settings"))
-  {
-    if (myStellaSettingsDialog == nullptr)
-      myStellaSettingsDialog = make_unique<StellaSettingsDialog>(instance(), parent(),
-        _w, _h, Menu::AppMode::launcher);
-    myStellaSettingsDialog->open();
-  }
+    myDialog = make_unique<StellaSettingsDialog>(instance(), parent(),
+                                                 _w, _h, Menu::AppMode::launcher);
   else
-  {
-    if (myOptionsDialog == nullptr)
-      myOptionsDialog = make_unique<OptionsDialog>(instance(), parent(), this, _w, _h,
-        Menu::AppMode::launcher);
-    myOptionsDialog->open();
-  }
+    myDialog = make_unique<OptionsDialog>(instance(), parent(), this, _w, _h,
+                                          Menu::AppMode::launcher);
+  myDialog->open();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::openHighScores()
 {
-  // Create an options dialog, similar to the in-game one
-  if(myHighScoresDialog == nullptr)
-    myHighScoresDialog = make_unique<HighScoresDialog>(instance(), parent(), _w, _h,
-                                                       Menu::AppMode::launcher);
-
-  myHighScoresDialog->open();
+  // Create an high scores dialog, similar to the in-game one
+  myDialog = make_unique<HighScoresDialog>(instance(), parent(), _w, _h,
+                                           Menu::AppMode::launcher);
+  myDialog->open();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void LauncherDialog::openWhatsNew()
 {
-  if(myWhatsNewDialog == nullptr)
-    myWhatsNewDialog = make_unique<WhatsNewDialog>(instance(), parent(), _w, _h);
-  myWhatsNewDialog->open();
-
+  myDialog = make_unique<WhatsNewDialog>(instance(), parent(), _w, _h);
+  myDialog->open();
 }
