@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -28,7 +28,7 @@
 ListWidget::ListWidget(GuiObject* boss, const GUI::Font& font,
                        int x, int y, int w, int h, bool useScrollbar)
   : EditableWidget(boss, font, x, y, 16, 16),
-    _useScrollbar(useScrollbar)
+    _useScrollbar{useScrollbar}
 {
   _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
   _bgcolor = kWidColor;
@@ -36,6 +36,8 @@ ListWidget::ListWidget(GuiObject* boss, const GUI::Font& font,
   _textcolor = kTextColor;
   _textcolorhi = kTextColor;
 
+  _editMode = false;
+
   _cols = w / _fontWidth;
   _rows = h / _lineHeight;
 
@@ -54,6 +56,17 @@ ListWidget::ListWidget(GuiObject* boss, const GUI::Font& font,
     _w = w - 1;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ListWidget::setHeight(int h)
+{
+  Widget::setHeight(h);
+  if(_useScrollbar)
+    _scrollBar->setHeight(h);
+
+  _rows = (h - 2) / _lineHeight;
+  recalc();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::setSelected(int item)
 {
@@ -159,7 +172,7 @@ void ListWidget::recalc()
 {
   int size = int(_list.size());
 
-  if(_currentPos >= size)
+  if(_currentPos >= size - _rows)
   {
     if(size <= _rows)
       _currentPos = 0;
@@ -169,7 +182,9 @@ void ListWidget::recalc()
   if (_currentPos < 0)
     _currentPos = 0;
 
-  if(_selectedItem < 0 || _selectedItem >= size)
+  if(_selectedItem >= size)
+    _selectedItem = size - 1;
+  if(_selectedItem < 0)
     _selectedItem = 0;
 
   _editMode = false;
@@ -205,6 +220,7 @@ void ListWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
   if (!isEnabled())
     return;
 
+  resetSelection();
   // First check whether the selection changed
   int newSelectedItem;
   newSelectedItem = findItem(x, y);
