@@ -94,7 +94,11 @@ void RomInfoWidget::parseProperties(const FilesystemNode& node)
         myAvail.w, myAvail.h, ScalingInterpolation::blur);
     mySurface->applyAttributes();
 
-    dialog().addSurface(mySurface);
+    dialog().addRenderCallback([this]() {
+      if(mySurfaceIsValid)
+        mySurface->render();
+      }
+    );
   }
 
   // Initialize to empty properties entry
@@ -169,6 +173,13 @@ void RomInfoWidget::parseProperties(const FilesystemNode& node)
   setDirty();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void RomInfoWidget::resetSurfaces()
+{
+  if(mySurface)
+    mySurface->reload();
+}
+
 #ifdef PNG_SUPPORT
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool RomInfoWidget::loadPng(const string& filename)
