@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -33,9 +33,11 @@ TimeLineWidget::TimeLineWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int w, int h,
                                const string& label, uInt32 labelWidth, int cmd)
   : ButtonWidget(boss, font, x, y, w, h, label, cmd),
-    _labelWidth(labelWidth)
+    _labelWidth{labelWidth}
 {
-  _flags = Widget::FLAG_ENABLED | Widget::FLAG_TRACK_MOUSE;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_TRACK_MOUSE
+    | Widget::FLAG_CLEARBG | Widget::FLAG_NOBG;
+
   _bgcolor = kDlgColor;
   _bgcolorhi = kDlgColor;
 
@@ -84,7 +86,7 @@ void TimeLineWidget::setStepValues(const IntArray& steps)
     if(steps.size() > _stepValue.capacity())
       _stepValue.reserve(2 * steps.size());
 
-    double scale = (_w - _labelWidth - 2 - HANDLE_W*0) / double(steps.back());
+    double scale = (_w - _labelWidth - 2 - HANDLE_W) / double(steps.back());
 
     // Skip the very last value; we take care of it outside the end of the loop
     for(uInt32 i = 0; i < steps.size() - 1; ++i)
@@ -92,7 +94,7 @@ void TimeLineWidget::setStepValues(const IntArray& steps)
 
     // Due to integer <-> double conversion, the last value is sometimes
     // slightly less than the maximum value; we assign it manually to fix this
-    _stepValue.push_back(_w - _labelWidth - 2 - HANDLE_W*0);
+    _stepValue.push_back(_w - _labelWidth - 2 - HANDLE_W);
   }
   else
     _stepValue.push_back(0);
@@ -146,12 +148,11 @@ void TimeLineWidget::drawWidget(bool hilite)
     s.drawString(_font, _label, _x, _y + 2, _labelWidth,
                  isEnabled() ? kTextColor : kColor, TextAlign::Left);
 
-  int p = valueToPos(_value),
-    x = _x + _labelWidth,
-    w = _w - _labelWidth;
-
   // Frame the handle
   const int HANDLE_W2 = (HANDLE_W + 1) / 2;
+  int p = valueToPos(_value),
+    x = _x + _labelWidth + HANDLE_W2,
+    w = _w - _labelWidth - HANDLE_W;
   s.hLine(x + p - HANDLE_W2, _y + 0, x + p - HANDLE_W2 + HANDLE_W, kColorInfo);
   s.vLine(x + p - HANDLE_W2, _y + 1, _y + _h - 2, kColorInfo);
   s.hLine(x + p - HANDLE_W2 + 1, _y + _h - 1, x + p - HANDLE_W2 + 1 + HANDLE_W, kBGColor);
@@ -202,13 +203,13 @@ void TimeLineWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 TimeLineWidget::valueToPos(uInt32 value)
+uInt32 TimeLineWidget::valueToPos(uInt32 value) const
 {
   return _stepValue[BSPF::clamp(value, _valueMin, _valueMax)];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 TimeLineWidget::posToValue(uInt32 pos)
+uInt32 TimeLineWidget::posToValue(uInt32 pos) const
 {
   // Find the interval in which 'pos' falls, and then the endpoint which
   // it is closest to
