@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -218,6 +218,7 @@ TimeMachineDialog::TimeMachineDialog(OSystem& osystem, DialogContainer& parent,
 
   this->clearFlags(Widget::FLAG_CLEARBG); // does only work combined with blending (0..100)!
   this->clearFlags(Widget::FLAG_BORDER);
+  this->setFlags(Widget::FLAG_NOBG);
 
   xpos = H_BORDER;
   ypos = V_BORDER;
@@ -225,8 +226,10 @@ TimeMachineDialog::TimeMachineDialog(OSystem& osystem, DialogContainer& parent,
   // Add index info
   myCurrentIdxWidget = new StaticTextWidget(this, font, xpos, ypos, "1000", TextAlign::Left, kBGColor);
   myCurrentIdxWidget->setTextColor(kColorInfo);
+  myCurrentIdxWidget->setFlags(Widget::FLAG_CLEARBG | Widget::FLAG_NOBG);
   myLastIdxWidget = new StaticTextWidget(this, font, _w - H_BORDER - font.getStringWidth("1000"), ypos,
                                          "1000", TextAlign::Right, kBGColor);
+  myLastIdxWidget->setFlags(Widget::FLAG_CLEARBG | Widget::FLAG_NOBG);
   myLastIdxWidget->setTextColor(kColorInfo);
 
   // Add timeline
@@ -241,19 +244,23 @@ TimeMachineDialog::TimeMachineDialog(OSystem& osystem, DialogContainer& parent,
   // Add time info
   int ypos_s = ypos + (buttonHeight - font.getFontHeight() + 1) / 2; // align to button vertical center
   myCurrentTimeWidget = new StaticTextWidget(this, font, xpos, ypos_s, "00:00.00", TextAlign::Left, kBGColor);
+  myCurrentTimeWidget->setFlags(Widget::FLAG_CLEARBG | Widget::FLAG_NOBG);
   myCurrentTimeWidget->setTextColor(kColorInfo);
   myLastTimeWidget = new StaticTextWidget(this, font, _w - H_BORDER - font.getStringWidth("00:00.00"), ypos_s,
                                           "00:00.00", TextAlign::Right, kBGColor);
+  myLastTimeWidget->setFlags(Widget::FLAG_CLEARBG | Widget::FLAG_NOBG);
   myLastTimeWidget->setTextColor(kColorInfo);
   xpos = myCurrentTimeWidget->getRight() + BUTTON_GAP * 4;
 
   // Add buttons
   myToggleWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                                     STOP.data(), BUTTON_W, BUTTON_H, kToggle);
+  myToggleWidget->setToolTip("Toogle Time Machine mode.");
   xpos += buttonWidth + BUTTON_GAP;
 
   myExitWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                                   EXIT.data(), BUTTON_W, BUTTON_H, kExit);
+  myExitWidget->setToolTip("Exit Time Machine dialog.");
   xpos += buttonWidth + BUTTON_GAP * 4;
 
   myRewindAllWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
@@ -266,6 +273,7 @@ TimeMachineDialog::TimeMachineDialog(OSystem& osystem, DialogContainer& parent,
 
   myPlayBackWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                                       PLAYBACK.data(), BUTTON_W, BUTTON_H, kPlayBack);
+  myPlayBackWidget->setToolTip("Start playback of Time Machine states.");
   xpos += buttonWidth + BUTTON_GAP;
 
   myUnwind1Widget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
@@ -278,20 +286,27 @@ TimeMachineDialog::TimeMachineDialog(OSystem& osystem, DialogContainer& parent,
 
   mySaveAllWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                                      SAVE_ALL.data(), BUTTON_W, BUTTON_H, kSaveAll);
+  mySaveAllWidget->setToolTip("Save all Time Machine states.");
   xpos = mySaveAllWidget->getRight() + BUTTON_GAP;
 
   myLoadAllWidget = new ButtonWidget(this, font, xpos, ypos, buttonWidth, buttonHeight,
                                      LOAD_ALL.data(), BUTTON_W, BUTTON_H, kLoadAll);
+  myLoadAllWidget->setToolTip("Load all Time Machine states.");
   xpos = myLoadAllWidget->getRight() + BUTTON_GAP * 4;
 
   // Add message
+  int mWidth = (myLastTimeWidget->getLeft() - xpos) / font.getMaxCharWidth();
+  const string blanks = "                                             ";
+
   myMessageWidget = new StaticTextWidget(this, font, xpos, ypos_s,
-      "                                             ", TextAlign::Left, kBGColor);
+                                         blanks.substr(0, mWidth),
+                                         TextAlign::Left, kBGColor);
+  myMessageWidget->setFlags(Widget::FLAG_CLEARBG | Widget::FLAG_NOBG);
   myMessageWidget->setTextColor(kColorInfo);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void TimeMachineDialog::center()
+void TimeMachineDialog::setPosition()
 {
   // Place on the bottom of the screen, centered horizontally
   const Common::Size& screen = instance().frameBuffer().screenSize();
@@ -317,11 +332,14 @@ void TimeMachineDialog::loadConfig()
 void TimeMachineDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
   // The following shortcuts duplicate the shortcuts in EventHandler
-
   Event::Type event = instance().eventHandler().eventForKey(EventMode::kEmulationMode, key, mod);
 
-  switch (event)
+  switch(event)
   {
+    case Event::ExitMode:
+      handleCommand(nullptr, kExit, 0, 0);
+      break;
+
     case Event::Rewind1Menu:
       handleCommand(nullptr, kRewind1, 0, 0);
       break;
@@ -346,13 +364,24 @@ void TimeMachineDialog::handleKeyDown(StellaKey key, StellaMod mod, bool repeate
       handleCommand(nullptr, kUnwindAll, 0, 0);
       break;
 
-    case Event::TakeSnapshot:
-      if (!repeated)
-        handleCommand(nullptr, kSnapShot, 0, 0);
+    case Event::LoadAllStates:
+      if(!repeated)
+        handleCommand(nullptr, kLoadAll, 0, 0);
       break;
 
-    case Event::ExitMode:
-      handleCommand(nullptr, kExit, 0, 0);
+    case Event::SaveAllStates:
+      if(!repeated)
+        handleCommand(nullptr, kSaveAll, 0, 0);
+      break;
+
+    // Hotkey only commands (no button available)
+    case Event::SaveState:
+    case Event::PreviousState:
+    case Event::NextState:
+    case Event::LoadState:
+    case Event::TakeSnapshot:
+      if(!repeated)
+        handleCommand(nullptr, event, 0, 0);
       break;
 
     default:
@@ -368,18 +397,10 @@ void TimeMachineDialog::handleKeyUp(StellaKey key, StellaMod mod)
 
   Event::Type event = instance().eventHandler().eventForKey(EventMode::kEmulationMode, key, mod);
 
-  switch (event)
-  {
-    case Event::TogglePlayBackMode:
-      handleCommand(nullptr, kPlayBack, 0, 0);
-      break;
-
-    default:
-      if (key == KBDK_SPACE)
-        handleCommand(nullptr, kPlayBack, 0, 0);
-      else
-        Dialog::handleKeyUp(key, mod);
-  }
+  if(event == Event::TogglePlayBackMode || key == KBDK_SPACE)
+    handleCommand(nullptr, kPlayBack, 0, 0);
+  else
+    Dialog::handleKeyUp(key, mod);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -434,16 +455,25 @@ void TimeMachineDialog::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kSaveAll:
-      instance().frameBuffer().showMessage(instance().state().rewindManager().saveAllStates());
+      instance().eventHandler().handleEvent(Event::SaveAllStates);
       break;
 
     case kLoadAll:
-      instance().frameBuffer().showMessage(instance().state().rewindManager().loadAllStates());
+      instance().eventHandler().handleEvent(Event::LoadAllStates);
       initBar();
       break;
 
-    case kSnapShot:
-      instance().frameBuffer().tiaSurface().saveSnapShot();
+    // Hotkey only commands (no button available)
+    case Event::SaveState:
+    case Event::PreviousState:
+    case Event::NextState:
+    case Event::LoadState:
+      instance().eventHandler().handleEvent(Event::Type(cmd));
+      break;
+
+    case Event::TakeSnapshot:
+      instance().eventHandler().handleEvent(Event::TakeSnapshot);
+      instance().frameBuffer().setPendingRender();
       break;
 
     default:
@@ -470,7 +500,7 @@ void TimeMachineDialog::initBar()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string TimeMachineDialog::getTimeString(uInt64 cycles)
+string TimeMachineDialog::getTimeString(uInt64 cycles) const
 {
   const Int32 scanlines = std::max<Int32>(instance().console().tia().scanlinesLastFrame(), 240);
   const bool isNTSC = scanlines <= 287;
