@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -42,18 +42,17 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
                    const GUI::Font& font, GuiObject* boss, int max_w, int max_h)
   : Dialog(osystem, parent, font, "User interface settings"),
     CommandSender(boss),
-    myFont(font),
-    myIsGlobal(boss != nullptr)
+    myIsGlobal{boss != nullptr}
 {
   const GUI::Font& ifont = instance().frameBuffer().infoFont();
-  const int lineHeight   = font.getLineHeight(),
-            fontWidth    = font.getMaxCharWidth(),
-            fontHeight   = font.getFontHeight(),
-            buttonHeight = font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int INDENT = fontWidth * 2;
-  const int VGAP = fontHeight / 4;
+  const int lineHeight   = Dialog::lineHeight(),
+            fontHeight   = Dialog::fontHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap(),
+            INDENT       = Dialog::indent();
   int xpos, ypos, tabID;
   int lwidth, pwidth, bwidth;
   WidgetArray wid;
@@ -91,7 +90,6 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   wid.push_back(myPalettePopup);
   ypos += lineHeight + VGAP;
 
-
   // Dialog font
   items.clear();
   VarList::push_back(items, "Small", "small");            //  8x13
@@ -109,6 +107,8 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   xpos = myDialogFontPopup->getRight() + fontWidth * 5;
   myHidpiWidget = new CheckboxWidget(myTab, font, xpos,
                                      ypos + 1, "HiDPI mode (*)");
+  myHidpiWidget->setToolTip("Scale the UI by a factor of two when enabled.");
+
   wid.push_back(myHidpiWidget);
 
   // Dialog position
@@ -126,6 +126,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   // Center window (in windowed mode)
   xpos = myHidpiWidget->getLeft();
   myCenter = new CheckboxWidget(myTab, _font, xpos, ypos + 1, "Center windows");
+  myCenter->setToolTip("Check to center all windows, else remember last position.");
   wid.push_back(myCenter);
 
   // Delay between quick-selecting characters in ListWidget
@@ -138,6 +139,8 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   myListDelaySlider->setMaxValue(1000);
   myListDelaySlider->setStepValue(50);
   myListDelaySlider->setTickmarkIntervals(5);
+  myListDelaySlider->setToolTip("Set delay between key presses in file lists"
+                                " before a search string resets.");
   wid.push_back(myListDelaySlider);
   ypos += lineHeight + VGAP;
 
@@ -214,6 +217,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   xpos = _w - HBORDER - font.getStringWidth("Follow Launcher path") - CheckboxWidget::prefixSize(font) - 1;
   ypos += lineHeight + VGAP * 2;
   myFollowLauncherWidget = new CheckboxWidget(myTab, font, xpos, ypos, "Follow Launcher path");
+  myFollowLauncherWidget->setToolTip("The ROM path is updated during Launcher navigation.");
   wid.push_back(myFollowLauncherWidget);
 
   xpos = HBORDER;
@@ -267,6 +271,7 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
   bwidth = font.getStringWidth("Image path" + ELLIPSIS) + fontWidth * 2 + 1;
   myOpenBrowserButton = new ButtonWidget(myTab, font, xpos, ypos, bwidth, buttonHeight,
                                          "Image path" + ELLIPSIS, kChooseSnapLoadDirCmd);
+  myOpenBrowserButton->setToolTip("Select path for snapshot images used in Launcher.");
   wid.push_back(myOpenBrowserButton);
 
   mySnapLoadPath = new EditTextWidget(myTab, font, HBORDER + lwidth,
@@ -311,11 +316,6 @@ UIDialog::UIDialog(OSystem& osystem, DialogContainer& parent,
 #endif
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-UIDialog::~UIDialog()
-{
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void UIDialog::loadConfig()
 {
@@ -441,6 +441,7 @@ void UIDialog::saveConfig()
   settings.setValue("uipalette",
     myPalettePopup->getSelectedTag().toString());
   instance().frameBuffer().setUIPalette();
+  instance().frameBuffer().update(FrameBuffer::UpdateMode::REDRAW);
 
   // Dialog font
   settings.setValue("dialogfont",
@@ -507,7 +508,7 @@ void UIDialog::setDefaults()
       myLauncherHeightSlider->setValue(h);
       myLauncherFontPopup->setSelected("medium", "");
       myRomViewerSize->setValue(35);
-      mySnapLoadPath->setText(instance().defaultLoadDir().getShortPath());
+      mySnapLoadPath->setText(instance().userDir().getShortPath());
       myLauncherExitWidget->setState(false);
       break;
     }
@@ -576,15 +577,12 @@ void UIDialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kChooseRomDirCmd:
-      // This dialog is resizable under certain conditions, so we need
-      // to re-create it as necessary
-      createBrowser("Select ROM directory");
-      myBrowser->show(myRomPath->getText(),
-                      BrowserDialog::Directories, LauncherDialog::kRomDirChosenCmd);
-      break;
-
-    case LauncherDialog::kRomDirChosenCmd:
-      myRomPath->setText(myBrowser->getResult().getShortPath());
+      BrowserDialog::show(this, _font, "Select ROM Directory",
+                          myRomPath->getText(),
+                          BrowserDialog::Mode::Directories,
+                          [this](bool OK, const FilesystemNode& node) {
+                            if(OK) myRomPath->setText(node.getShortPath());
+                          });
       break;
 
     case kRomViewer:
@@ -592,15 +590,12 @@ void UIDialog::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kChooseSnapLoadDirCmd:
-      // This dialog is resizable under certain conditions, so we need
-      // to re-create it as necessary
-      createBrowser("Select snapshot load directory");
-      myBrowser->show(mySnapLoadPath->getText(),
-                      BrowserDialog::Directories, kSnapLoadDirChosenCmd);
-      break;
-
-    case kSnapLoadDirChosenCmd:
-      mySnapLoadPath->setText(myBrowser->getResult().getShortPath());
+      BrowserDialog::show(this, _font, "Select ROM Info Viewer Image Directory",
+                          mySnapLoadPath->getText(),
+                          BrowserDialog::Mode::Directories,
+                          [this](bool OK, const FilesystemNode& node) {
+                            if(OK) mySnapLoadPath->setText(node.getShortPath());
+                          });
       break;
 
     default:
@@ -692,17 +687,3 @@ void UIDialog::handleRomViewer()
   myOpenBrowserButton->setEnabled(enable);
   mySnapLoadPath->setEnabled(enable);
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void UIDialog::createBrowser(const string& title)
-{
-  uInt32 w = 0, h = 0;
-  getDynamicBounds(w, h);
-
-  // Create file browser dialog
-  if(!myBrowser || uInt32(myBrowser->getWidth()) != w ||
-     uInt32(myBrowser->getHeight()) != h)
-    myBrowser = make_unique<BrowserDialog>(this, myFont, w, h, title);
-  else
-    myBrowser->setTitle(title);
-}
