@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -57,13 +57,12 @@ VideoAudioDialog::VideoAudioDialog(OSystem& osystem, DialogContainer& parent,
                          const GUI::Font& font, int max_w, int max_h)
   : Dialog(osystem, parent, font, "Video & Audio settings")
 {
-  const int lineHeight   = _font.getLineHeight(),
-            fontHeight   = _font.getFontHeight(),
-            fontWidth    = _font.getMaxCharWidth(),
-            buttonHeight = _font.getLineHeight() * 1.25;
-  const int VGAP = fontHeight / 4;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight   = Dialog::lineHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   int xpos, ypos;
 
   // Set real dimensions
@@ -103,12 +102,12 @@ VideoAudioDialog::VideoAudioDialog(OSystem& osystem, DialogContainer& parent,
 void VideoAudioDialog::addDisplayTab()
 {
   const GUI::Font& ifont = instance().frameBuffer().infoFont();
-  const int lineHeight   = _font.getLineHeight(),
-            fontHeight   = _font.getFontHeight(),
-            fontWidth    = _font.getMaxCharWidth();
-  const int VGAP = fontHeight / 4;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight = Dialog::lineHeight(),
+            fontHeight = Dialog::fontHeight(),
+            fontWidth  = Dialog::fontWidth(),
+            VBORDER    = Dialog::vBorder(),
+            HBORDER    = Dialog::hBorder(),
+            VGAP       = Dialog::vGap();
   const int INDENT = CheckboxWidget::prefixSize(_font);
   const int lwidth = _font.getStringWidth("V-Size adjust "),
             pwidth = _font.getStringWidth("OpenGLES2");
@@ -122,12 +121,14 @@ void VideoAudioDialog::addDisplayTab()
   myRenderer = new PopUpWidget(myTab, _font, xpos, ypos, pwidth, lineHeight,
                                instance().frameBuffer().supportedRenderers(),
                                "Renderer ", lwidth);
+  myRenderer->setToolTip("Select renderer used for displaying screen.");
   wid.push_back(myRenderer);
   const int swidth = myRenderer->getWidth() - lwidth;
   ypos += lineHeight + VGAP;
 
   // TIA interpolation
   myTIAInterpolate = new CheckboxWidget(myTab, _font, xpos, ypos + 1, "Interpolation ");
+  myTIAInterpolate->setToolTip("Blur emulated display.");
   wid.push_back(myTIAInterpolate);
 
   ypos += lineHeight + VGAP * 4;
@@ -145,12 +146,14 @@ void VideoAudioDialog::addDisplayTab()
 
   // FS stretch
   myUseStretch = new CheckboxWidget(myTab, _font, xpos + INDENT, ypos + 1, "Stretch");
+  myUseStretch->setToolTip("Stretch emulated display to fill whole screen.");
   wid.push_back(myUseStretch);
 
 #ifdef ADAPTABLE_REFRESH_SUPPORT
   // Adapt refresh rate
   ypos += lineHeight + VGAP;
   myRefreshAdapt = new CheckboxWidget(myTab, _font, xpos + INDENT, ypos + 1, "Adapt display refresh rate");
+  myRefreshAdapt->setToolTip("Select optimal display refresh rate for each ROM.");
   wid.push_back(myRefreshAdapt);
 #else
   myRefreshAdapt = nullptr;
@@ -167,7 +170,8 @@ void VideoAudioDialog::addDisplayTab()
   // Aspect ratio correction
   ypos += lineHeight + VGAP * 4;
   myCorrectAspect = new CheckboxWidget(myTab, _font, xpos, ypos + 1, "Correct aspect ratio (*)");
-  wid.push_back(myUseStretch);
+  myCorrectAspect->setToolTip("Uncheck to disable real world aspect ratio correction.");
+  wid.push_back(myCorrectAspect);
 
   // Vertical size
   ypos += lineHeight + VGAP;
@@ -176,6 +180,7 @@ void VideoAudioDialog::addDisplayTab()
                      "V-Size adjust", lwidth, kVSizeChanged, fontWidth * 7, "%", 0, true);
   myVSizeAdjust->setMinValue(-5); myVSizeAdjust->setMaxValue(5);
   myVSizeAdjust->setTickmarkIntervals(2);
+  myVSizeAdjust->setToolTip("Adjust vertical size to match emulated TV display.");
   wid.push_back(myVSizeAdjust);
 
 
@@ -193,13 +198,12 @@ void VideoAudioDialog::addDisplayTab()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoAudioDialog::addPaletteTab()
 {
-  const int lineHeight = _font.getLineHeight(),
-            fontHeight = _font.getFontHeight(),
-            fontWidth = _font.getMaxCharWidth();
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
-  const int INDENT = fontWidth * 2;
-  const int VGAP = fontHeight / 4;
+  const int lineHeight = Dialog::lineHeight(),
+            fontWidth  = Dialog::fontWidth(),
+            VBORDER    = Dialog::vBorder(),
+            HBORDER    = Dialog::hBorder(),
+            VGAP       = Dialog::vGap(),
+            INDENT     = Dialog::indent();
   const int lwidth = _font.getStringWidth("  NTSC phase ");
   const int pwidth = _font.getStringWidth("Standard");
   int xpos = HBORDER,
@@ -231,6 +235,7 @@ void VideoAudioDialog::addPaletteTab()
   myPhaseShiftNtsc->setMinValue((PaletteHandler::DEF_NTSC_SHIFT - PaletteHandler::MAX_PHASE_SHIFT) * 10);
   myPhaseShiftNtsc->setMaxValue((PaletteHandler::DEF_NTSC_SHIFT + PaletteHandler::MAX_PHASE_SHIFT) * 10);
   myPhaseShiftNtsc->setTickmarkIntervals(4);
+  myPhaseShiftNtsc->setToolTip("Adjust NTSC phase shift of 'Custom' palette.");
   wid.push_back(myPhaseShiftNtsc);
   ypos += lineHeight + VGAP;
 
@@ -240,6 +245,7 @@ void VideoAudioDialog::addPaletteTab()
   myPhaseShiftPal->setMinValue((PaletteHandler::DEF_PAL_SHIFT - PaletteHandler::MAX_PHASE_SHIFT) * 10);
   myPhaseShiftPal->setMaxValue((PaletteHandler::DEF_PAL_SHIFT + PaletteHandler::MAX_PHASE_SHIFT) * 10);
   myPhaseShiftPal->setTickmarkIntervals(4);
+  myPhaseShiftPal->setToolTip("Adjust PAL phase shift of 'Custom' palette.");
   wid.push_back(myPhaseShiftPal);
   ypos += lineHeight + VGAP;
 
@@ -252,6 +258,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVRedScale->setMinValue(0);
   myTVRedScale->setMaxValue(100);
   myTVRedScale->setTickmarkIntervals(2);
+  myTVRedScale->setToolTip("Adjust red saturation of 'Custom' palette.");
   wid.push_back(myTVRedScale);
 
   const int xposr = myTIAPalette->getRight() - rgbsWidth;
@@ -261,6 +268,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVRedShift->setMinValue((PaletteHandler::DEF_RGB_SHIFT - PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVRedShift->setMaxValue((PaletteHandler::DEF_RGB_SHIFT + PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVRedShift->setTickmarkIntervals(2);
+  myTVRedShift->setToolTip("Adjust red shift of 'Custom' palette.");
   wid.push_back(myTVRedShift);
   ypos += lineHeight + VGAP;
 
@@ -270,6 +278,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVGreenScale->setMinValue(0);
   myTVGreenScale->setMaxValue(100);
   myTVGreenScale->setTickmarkIntervals(2);
+  myTVGreenScale->setToolTip("Adjust green saturation of 'Custom' palette.");
   wid.push_back(myTVGreenScale);
 
   myTVGreenShift =
@@ -278,6 +287,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVGreenShift->setMinValue((PaletteHandler::DEF_RGB_SHIFT - PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVGreenShift->setMaxValue((PaletteHandler::DEF_RGB_SHIFT + PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVGreenShift->setTickmarkIntervals(2);
+  myTVGreenShift->setToolTip("Adjust green shift of 'Custom' palette.");
   wid.push_back(myTVGreenShift);
   ypos += lineHeight + VGAP;
 
@@ -287,6 +297,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVBlueScale->setMinValue(0);
   myTVBlueScale->setMaxValue(100);
   myTVBlueScale->setTickmarkIntervals(2);
+  myTVBlueScale->setToolTip("Adjust blue saturation of 'Custom' palette.");
   wid.push_back(myTVBlueScale);
 
   myTVBlueShift =
@@ -295,6 +306,7 @@ void VideoAudioDialog::addPaletteTab()
   myTVBlueShift->setMinValue((PaletteHandler::DEF_RGB_SHIFT - PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVBlueShift->setMaxValue((PaletteHandler::DEF_RGB_SHIFT + PaletteHandler::MAX_RGB_SHIFT) * 10);
   myTVBlueShift->setTickmarkIntervals(2);
+  myTVBlueShift->setToolTip("Adjust blue shift of 'Custom' palette.");
   wid.push_back(myTVBlueShift);
   ypos += lineHeight + VGAP;
   xpos -= INDENT;
@@ -317,14 +329,14 @@ void VideoAudioDialog::addPaletteTab()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoAudioDialog::addTVEffectsTab()
 {
-  const int lineHeight = _font.getLineHeight(),
-            fontHeight = _font.getFontHeight(),
-            fontWidth = _font.getMaxCharWidth(),
-            buttonHeight = _font.getLineHeight() * 1.25;
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight   = Dialog::lineHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            buttonWidth  = Dialog::buttonWidth("Clone Bad Adjust"),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
   const int INDENT = CheckboxWidget::prefixSize(_font);// fontWidth * 2;
-  const int VGAP = fontHeight / 4;
   int xpos = HBORDER,
       ypos = VBORDER;
   const int lwidth = _font.getStringWidth("Saturation ");
@@ -378,14 +390,13 @@ void VideoAudioDialog::addTVEffectsTab()
   CREATE_CUSTOM_SLIDERS(ScanIntense, "Intensity", kScanlinesChanged)
 
   // Create buttons in 2nd column
-  int cloneWidth = _font.getStringWidth("Clone Bad Adjust") + fontWidth * 2.5;
-  xpos = _w - HBORDER - 2 * 2 - cloneWidth;
+  xpos = _w - HBORDER - 2 * 2 - buttonWidth;
   ypos = VBORDER - VGAP / 2;
 
   // Adjustable presets
 #define CREATE_CLONE_BUTTON(obj, desc)                                 \
   myClone ## obj =                                                     \
-    new ButtonWidget(myTab, _font, xpos, ypos, cloneWidth, buttonHeight,\
+    new ButtonWidget(myTab, _font, xpos, ypos, buttonWidth, buttonHeight,\
                      desc, kClone ## obj ##Cmd);                       \
   wid.push_back(myClone ## obj);                                       \
   ypos += buttonHeight + VGAP;
@@ -404,14 +415,12 @@ void VideoAudioDialog::addTVEffectsTab()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoAudioDialog::addAudioTab()
 {
-  const int lineHeight   = _font.getLineHeight(),
-            fontHeight   = _font.getFontHeight(),
-            fontWidth    = _font.getMaxCharWidth();
-  const int VBORDER = fontHeight / 2;
-  const int HBORDER = fontWidth * 1.25;
+  const int lineHeight = Dialog::lineHeight(),
+            fontWidth  = Dialog::fontWidth(),
+            VBORDER    = Dialog::vBorder(),
+            HBORDER    = Dialog::hBorder(),
+            VGAP       = Dialog::vGap();
   const int INDENT = CheckboxWidget::prefixSize(_font);
-  const int VGAP = fontHeight / 4;
-
   int xpos, ypos;
   int lwidth = _font.getStringWidth("Volume "),
     pwidth;
@@ -631,7 +640,7 @@ void VideoAudioDialog::loadConfig()
   myVolumeSlider->setValue(audioSettings.volume());
 
   // Device
-  uInt32 deviceId = BSPF::clamp(audioSettings.device(), 0u,
+  uInt32 deviceId = BSPF::clamp(audioSettings.device(), 0U,
                                 uInt32(instance().sound().supportedDevices().size() - 1));
   myDevicePopup->setSelected(deviceId);
 
@@ -959,7 +968,16 @@ void VideoAudioDialog::handlePaletteUpdate()
   instance().frameBuffer().tiaSurface().paletteHandler().setAdjustables(paletteAdj);
 
   if(instance().hasConsole())
+  {
     instance().frameBuffer().tiaSurface().paletteHandler().setPalette();
+
+    constexpr int NUM_LUMA = 8;
+    constexpr int NUM_CHROMA = 16;
+
+    for(int idx = 0; idx < NUM_CHROMA; ++idx)
+      for(int lum = 0; lum < NUM_LUMA; ++lum)
+        myColor[idx][lum]->setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1157,10 +1175,11 @@ void VideoAudioDialog::addPalette(int x, int y, int w, int h)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void VideoAudioDialog::colorPalette()
 {
+  constexpr int NUM_LUMA = 8;
+  constexpr int NUM_CHROMA = 16;
+
   if(instance().hasConsole())
   {
-    constexpr int NUM_LUMA = 8;
-    constexpr int NUM_CHROMA = 16;
     const int order[2][NUM_CHROMA] =
     {
       {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15},
@@ -1176,11 +1195,14 @@ void VideoAudioDialog::colorPalette()
       ss << Common::Base::HEX1 << std::uppercase << color;
       myColorLbl[idx]->setLabel(ss.str());
       for(int lum = 0; lum < NUM_LUMA; ++lum)
-      {
         myColor[idx][lum]->setColor(color * NUM_CHROMA + lum * 2); // skip grayscale colors
-      }
     }
   }
+  else
+    // disable palette
+    for(int idx = 0; idx < NUM_CHROMA; ++idx)
+      for(int lum = 0; lum < NUM_LUMA; ++lum)
+        myColor[idx][lum]->setEnabled(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
