@@ -17,32 +17,40 @@ my %prop_type = (
   "Console.TVType"            => 11,
   "Console.SwapPorts"         => 12,
   "Controller.Left"           => 13,
-  "Controller.Right"          => 14,
-  "Controller.SwapPaddles"    => 15,
-  "Controller.PaddlesXCenter" => 16,
-  "Controller.PaddlesYCenter" => 17,
-  "Controller.MouseAxis"      => 18,
-  "Display.Format"            => 19,
-  "Display.VCenter"           => 20,
-  "Display.Phosphor"          => 21,
-  "Display.PPBlend"           => 22
+  "Controller.Left1"          => 14,
+  "Controller.Left2"          => 15,
+  "Controller.Right"          => 16,
+  "Controller.Right1"         => 17,
+  "Controller.Right2"         => 18,
+  "Controller.SwapPaddles"    => 19,
+  "Controller.PaddlesXCenter" => 20,
+  "Controller.PaddlesYCenter" => 21,
+  "Controller.MouseAxis"      => 22,
+  "Display.Format"            => 23,
+  "Display.VCenter"           => 24,
+  "Display.Phosphor"          => 25,
+  "Display.PPBlend"           => 26,
+  "Cart.Highscore"	      => 27
 );
 my @prop_type_as_string = (
-  "Cart.MD5",
-  "Cart.Manufacturer",
-  "Cart.ModelNo",
-  "Cart.Name",
-  "Cart.Note",
-  "Cart.Rarity",
-  "Cart.Sound",
-  "Cart.StartBank",
-  "Cart.Type",
-  "Console.LeftDiff",
-  "Console.RightDiff",
-  "Console.TVType",
-  "Console.SwapPorts",
+  "Cart.MD5",			
+  "Cart.Manufacturer",		
+  "Cart.ModelNo",		
+  "Cart.Name",			
+  "Cart.Note",			
+  "Cart.Rarity",		
+  "Cart.Sound",			   
+  "Cart.StartBank",		   
+  "Cart.Type",			   
+  "Console.LeftDiff",		
+  "Console.RightDiff",		
+  "Console.TVType",		    
   "Controller.Left",
+  "Controller.Left1",
+  "Controller.Left2",
   "Controller.Right",
+  "Controller.Right1",
+  "Controller.Right2",
   "Controller.SwapPaddles",
   "Controller.PaddlesXCenter",
   "Controller.PaddlesYCenter",
@@ -50,31 +58,42 @@ my @prop_type_as_string = (
   "Display.Format",
   "Display.VCenter",
   "Display.Phosphor",
-  "Display.PPBlend"
-);
+  "Display.PPBlend",
+  "Cart_Highscore"
+);				
 
 my @prop_defaults = (
-  "",
-  "",
-  "",
-  "",
-  "",
-  "",
-  "MONO",
-  "AUTO",
-  "AUTO",
-  "B",
-  "B",
-  "COLOR",
-  "NO",
-  "",
-  "",
-  "NO",
-  "AUTO",
-  "AUTO",
-  "0",
-  "NO",
-  "0"
+  "",		# Cart.MD5                 			
+  "",		# Cart.Manufacturer        
+  "",		# Cart.ModelNo             
+  "",		# Cart.Name                
+  "",		# Cart.Note                
+  "",		# Cart.Rarity              
+  "MONO",	# Cart.Sound               
+  "AUTO",	# Cart.StartBank           
+  "AUTO",	# Cart.Type                
+  "B",		# Console.LeftDiff         
+  "B",		# Console.RightDiff        
+  "COLOR",	# Console.TVType           
+  "NO",		# Console.SwapPorts        
+  "",		# Controller.Left          
+  "",		# Controller.Left1         
+  "",		# Controller.Left2         
+  "",		# Controller.Right         
+  "",		# Controller.Right1        
+  "",		# Controller.Right2        
+  "NO",		# Controller.SwapPaddles   
+  "0",		# Controller.PaddlesXCenter
+  "0",		# Controller.PaddlesYCenter
+  "AUTO",	# Controller.MouseAxis     
+  "AUTO",	# Display.Format           
+  "0",		# Display.VCenter          
+  "NO",		# Display.Phosphor         
+  "0",		# Display.PPBlend          
+  "1",		# Cart.Variations          
+  "",		# Cart.Formats             
+  "",		# Cart.Addresses           
+  ""	        # Cart.Highscore
 );
 
 # Load and parse a properties file into an hash table of property
