@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -102,33 +102,29 @@ StringList SerialPortUNIX::portNames()
 {
   StringList ports;
 
+  // Check if port is valid; for now that means if it can be opened
+  // Eventually we may extend this to do more intensive checks
+  auto isPortValid = [](const string& port) {
+    int handle = open(port.c_str(), O_RDWR | O_NOCTTY | O_NONBLOCK);
+    if(handle > 0)  close(handle);
+    return handle > 0;
+  };
+
   // Get all possible devices in the '/dev' directory
   FilesystemNode::NameFilter filter = [](const FilesystemNode& node) {
     return BSPF::startsWithIgnoreCase(node.getPath(), "/dev/ttyACM") ||
            BSPF::startsWithIgnoreCase(node.getPath(), "/dev/ttyUSB");
   };
   FSList portList;
-  portList.reserve(16);
+  portList.reserve(5);
 
   FilesystemNode dev("/dev/");
   dev.getChildren(portList, FilesystemNode::ListMode::All, filter, false);
 
   // Add only those that can be opened
   for(const auto& port: portList)
-    if(isValid(port.getPath()))
+    if(isPortValid(port.getPath()))
       ports.emplace_back(port.getPath());
 
   return ports;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool SerialPortUNIX::isValid(const string& port) const
-{
-  // For now, we can only detect whether the port could be opened
-  // Eventually we may extend this to do more intensive checks
-  int handle = open(port.c_str(), O_RDWR | O_NOCTTY | O_NONBLOCK);
-  if(handle > 0)
-    close(handle);
-
-  return handle > 0;
-}
