@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -39,31 +39,6 @@
 #include "Windows.hxx"
 #include "FSNodeWINDOWS.hxx"
 
-/**
- * Returns the last component of a given path.
- *
- * Examples:
- *			c:\foo\bar.txt would return "\bar.txt"
- *			c:\foo\bar\    would return "\bar\"
- *
- * @param str Path to obtain the last component from.
- * @return Pointer to the first char of the last component inside str.
- */
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* lastPathComponent(const string& str)
-{
-  if(str.empty())
-    return "";
-
-  const char* start = str.c_str();
-  const char* cur = start + str.size() - 2;
-
-  while(cur >= start && *cur != '\\')
-    --cur;
-
-  return cur + 1;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNodeWINDOWS::exists() const
 {
@@ -175,10 +150,8 @@ FilesystemNodeWINDOWS::FilesystemNodeWINDOWS()
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FilesystemNodeWINDOWS::FilesystemNodeWINDOWS(const string& p)
+  : _path{p.length() > 0 ? p : "~"}  // Default to home directory
 {
-  // Default to home directory
-  _path = p.length() > 0 ? p : "~";
-
   // Expand '~' to the users 'home' directory
   if(_path[0] == '~')
     _path.replace(0, 1, myHomeFinder.getHomePath());
