@@ -8,7 +8,7 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2020 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2021 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
@@ -22,12 +22,12 @@
 #include "OSystemWINDOWS.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void OSystemWINDOWS::getBaseDirAndConfig(string& basedir, string& cfgfile,
-        string& savedir, string& loaddir,
-        bool useappdir, const string& usedir)
+void OSystemWINDOWS::getBaseDirectories(string& basedir, string& homedir,
+                                        bool useappdir, const string& usedir)
 {
   HomeFinder homefinder;
   FilesystemNode appdata(homefinder.getAppDataPath());
+
   if(appdata.isDirectory())
   {
     basedir = appdata.getShortPath();
@@ -35,8 +35,9 @@ void OSystemWINDOWS::getBaseDirAndConfig(string& basedir, string& cfgfile,
       basedir += '\\';
     basedir += "Stella\\";
   }
-  FilesystemNode defaultLoadSaveDir(homefinder.getDesktopPath());
-  savedir = loaddir = defaultLoadSaveDir.getShortPath();
+
+  FilesystemNode defaultHomeDir(homefinder.getDesktopPath());
+  homedir = defaultHomeDir.getShortPath();
 
   // Check to see if basedir overrides are active
   if(useappdir)
@@ -44,12 +45,11 @@ void OSystemWINDOWS::getBaseDirAndConfig(string& basedir, string& cfgfile,
     char filename[MAX_PATH];
     GetModuleFileNameA(NULL, filename, sizeof(filename));
     FilesystemNode appdir(filename);
+
     appdir = appdir.getParent();
     if(appdir.isDirectory())
-      savedir = loaddir = basedir = appdir.getPath();
+      basedir = appdir.getPath();
   }
   else if(usedir != "")
-    savedir = loaddir = basedir = FilesystemNode(usedir).getPath();
-
-  cfgfile = basedir + "stella.ini";
+    basedir = FilesystemNode(usedir).getPath();
 }
